/* ------------------- Moonquake database library routines -------------------+
|                                                                             |
|                          "MqCheckObsMode" ver 1.00                          |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/* 
 *  Moonquake observation had two modes, FLAT and PEAKED. FLAT mode obser-
 *  vations has been executed during the following terms:
 *      Station 12:          69/323-326
 *                           74/289-75/099
 *      Station 12, 15 & 16: 75/180-77/086
 *     ( from personal communication, Nakamura, 1991 ).
 *  This function specifies data start time and station number and returns 
 *  observation mode variable obsmode.
 */

/* ============================= Include files ============================= */
#include "mqdb.h"

/* ========================== Program statements  ========================== */

MqMode MqCheckObsMode( MqTime mqtime, unsigned short station )
    {
    MqMode  obsmode;

    MqClearError();
    MqSetFunc( MQCHECKOBSMODE );

    MqCheckTime( &mqtime );
    if( MqErr.errno == EMQNOERROR )    {
	switch( mqtime.year )    {
	case 69:
	    if( station == 12 )
		( mqtime.day >= 323 && mqtime.day <= 326 ) ?
		    ( obsmode = MQ_FLAT ) : ( obsmode = MQ_PEAKED );
	    else
		obsmode = MQ_PEAKED;
	    break;
	case 74:
	    if( station == 12 )
		( mqtime.day >= 289 ) ? 
		    ( obsmode = MQ_FLAT ) : ( obsmode = MQ_PEAKED );
	    else 
		obsmode = MQ_PEAKED;
	    break;
	case 75:
	    if( station == 12 )
		( mqtime.day <= 99 || mqtime.day >= 180 ) ?
		    ( obsmode = MQ_FLAT ) : ( obsmode = MQ_PEAKED );
	    else if( station != 14 )
		( mqtime.day >= 180 ) ?
		    ( obsmode = MQ_FLAT ) : ( obsmode = MQ_PEAKED );
	    else
		obsmode = MQ_PEAKED;
	    break;
	case 76:
	    if( station != 14 )
		obsmode = MQ_FLAT;
	    else 
		obsmode = MQ_PEAKED;
	    break;
	case 77:
	    if( station != 14 )
		( mqtime.day <= 86 ) ? 
		    ( obsmode = MQ_FLAT ) : ( obsmode = MQ_PEAKED );
	    else
		obsmode = MQ_PEAKED;
	    break;
	default:
	    obsmode = MQ_PEAKED;
	    }
	}
    return obsmode;
    }

/* =========================== END of the program ========================== */

