/* ------------------- Moonquake database library routines -------------------+
|                                                                             |
|                           "MqCheckTime" ver 1.00                            |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/*
 *  This function checks values of MqTime structure and returns error structure
 *  setting error number. If invalid value is detected, the "EMQ_WARNING"-type
 *  error will be returned, but no error correction will be made.
 */

/* ============================= Include files ============================= */
#include "mqdb.h"

/* ========================== Program statements  ========================== */
void MqCheckTime( MqTime *mqtime )
    {
    MqSetFunc( MQCHECKTIME );
    MqClearError();

    if( mqtime->year < 1969 )         {    /* Year must be after 1969 */
	MqSetError( EMQSMALLYEAR );
	MqPerror( NULL );
	}

    else if( mqtime->year > 1977 )    {   /* Year must be before 1977 */
	MqSetError( EMQLARGEYEAR );
	MqPerror( NULL );
	}

    else if( mqtime->day  > 366 )     {   /* day must be less than 366 */
	MqSetError( EMQLARGEDAY );
	MqPerror( NULL );
	}

    else if( mqtime->hour > 23 )      {   /* hour must be less than 23 */
	MqSetError( EMQLARGEHOUR );
	MqPerror( NULL );
	}

    else if( mqtime->minute > 59 )    {   /* minute must be less than 59 */
	MqSetError( EMQLARGEMINUTE );
	MqPerror( NULL );
	}

    else if( mqtime->second > 59 )    {   /* second must be less than 59 */
	MqSetError( EMQLARGESECOND );
	MqPerror( NULL );
	}

    else if( mqtime->ms > 999 )       {   /* ms must be less than 999 */
	MqSetError( EMQLARGEMS );
	MqPerror( NULL );
	}

    else MqSetError( EMQNOERROR );
    }

/* =========================== END of the program ========================== */
