/* ------------------- Moonquake database library routines -------------------+
|                                                                             |
|                             "MqCreate" ver 1.00                             |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/*
 *  This is a utility subroutine of MQDB library that creates MQDB format file.
 *  Both headers and data parts are automatically made by setting data and 
 *  calling this function. No value will be returned.
 */

/* ============================= Include files ============================= */
#include <stdio.h>
#include <stdlib.h>
#include "mqdb.h"
#ifdef HAVE_XDR
#include <rpc/rpc.h>
#else
#include <types.h>
#include <xdr.h>
#endif

/* ========================== Program statements  ========================== */
void MqCreate( char *filename, MqAccess *access, MqHeader *header,
	                                                       double *data )
    {
    FILE     *fcre;      /* file descriptor */
    char     *string;    /* header string */
    XDR       xdrout;    /* XDR data structure */
    unsigned int  i;     /* just a loop counter */
    MqYesNo   fileout;   /* whether output is done for regular file */

    MqSetFunc( MQCREATE );
    MqClearError();

    if( NULL == ( string = calloc( MQHDRLENMAX, 1 ))) {
	MqSetError( EMQMEMALLOC );
	MqPerror( NULL );
	}

    if( NULL == access || NULL == header || NULL == data )    {
	MqSetError( EMQNULLPTR );
	MqPerror( NULL );
	}

    MqPutTopLine( string );

    if( MqErr.errno == EMQNOERROR )
	strcat( string, MqPutCreateDate() );

    if( access->AveAmp == MQ_YES && MqErr.errno == EMQNOERROR )
	strcat( string, MqPutAveAmp( header->AveAmp ));
    ( MqErr.errno == EMQNOERROR ) ?
        ( MqStatus.AveAmp = MQ_SUCCESS ) : ( MqStatus.AveAmp = MQ_FAIL );

    if( access->Channels == MQ_YES && MqErr.errno == EMQNOERROR )
	strcat( string, MqPutChannels( header->Channels ));
    ( MqErr.errno == EMQNOERROR ) ?
        ( MqStatus.Channels = MQ_SUCCESS ) : ( MqStatus.Channels = MQ_FAIL );

    if( access->DataFormat == MQ_YES && MqErr.errno == EMQNOERROR )
	strcat( string, MqPutDataFormat( header->DataFormat ));
    ( MqErr.errno == EMQNOERROR ) ?
        ( MqStatus.DataFormat = MQ_SUCCESS ) :
	    ( MqStatus.DataFormat = MQ_FAIL );

    if( access->MaxAmp == MQ_YES && MqErr.errno == EMQNOERROR )
	strcat( string, MqPutMaxAmp( header->MaxAmp ));
    ( MqErr.errno == EMQNOERROR ) ?
        ( MqStatus.MaxAmp = MQ_SUCCESS ) : ( MqStatus.MaxAmp = MQ_FAIL );

    if( access->DataType == MQ_YES && MqErr.errno == EMQNOERROR )
	strcat( string, MqPutDataType( header->DataType ));
    ( MqErr.errno == EMQNOERROR ) ?
        ( MqStatus.DataType = MQ_SUCCESS ) : ( MqStatus.DataType = MQ_FAIL );

    if( access->FileType == MQ_YES && MqErr.errno == EMQNOERROR )
	strcat( string, MqPutFileType( header->FileType ));
    ( MqErr.errno == EMQNOERROR ) ?
        ( MqStatus.FileType = MQ_SUCCESS ) : ( MqStatus.FileType = MQ_FAIL );

    if( access->NumOfData == MQ_YES && MqErr.errno == EMQNOERROR )
	strcat( string, MqPutNumOfData( header->NumOfData ));
    ( MqErr.errno == EMQNOERROR ) ?
        ( MqStatus.NumOfData = MQ_SUCCESS ) : ( MqStatus.NumOfData = MQ_FAIL );

    if( access->ObsMode == MQ_YES && MqErr.errno == EMQNOERROR )
	strcat( string, MqPutObsMode( header->ObsMode )); 
    ( MqErr.errno == EMQNOERROR ) ?
        ( MqStatus.ObsMode = MQ_SUCCESS ) : ( MqStatus.ObsMode = MQ_FAIL );

    if( access->OrgFileName == MQ_YES && MqErr.errno == EMQNOERROR )
	strcat( string, MqPutOrgFile( header->OrgFile ));
    ( MqErr.errno == EMQNOERROR ) ?
        ( MqStatus.ObsMode = MQ_SUCCESS ) : ( MqStatus.ObsMode = MQ_FAIL );

    if( access->SampRate == MQ_YES && MqErr.errno == EMQNOERROR )
	strcat( string, MqPutSampRate( header->SampRate ));
    ( MqErr.errno == EMQNOERROR ) ?
        ( MqStatus.SampRate = MQ_SUCCESS ) : ( MqStatus.SampRate = MQ_FAIL );

    if( access->Station == MQ_YES && MqErr.errno == EMQNOERROR )
	strcat( string, MqPutStation( header->Station ));
    ( MqErr.errno == EMQNOERROR ) ?
        ( MqStatus.Station = MQ_SUCCESS ) : ( MqStatus.Station = MQ_FAIL );

    if( access->StartRecord == MQ_YES && MqErr.errno == EMQNOERROR )
	strcat( string, MqPutStartRecord( header->StartRecord ));
    ( MqErr.errno == EMQNOERROR ) ?
        ( MqStatus.StartRecord = MQ_SUCCESS ) :
	    ( MqStatus.StartRecord = MQ_FAIL );

    if( access->EndRecord == MQ_YES && MqErr.errno == EMQNOERROR )
	strcat( string, MqPutEndRecord( header->EndRecord ));
    ( MqErr.errno == EMQNOERROR ) ?
        ( MqStatus.EndRecord = MQ_SUCCESS ) : ( MqStatus.EndRecord = MQ_FAIL );

    if( access->StartTime == MQ_YES && MqErr.errno == EMQNOERROR )
	strcat( string, MqPutStartTime( header->StartTime ));
    ( MqErr.errno == EMQNOERROR ) ?
        ( MqStatus.StartTime = MQ_SUCCESS ) : ( MqStatus.StartTime = MQ_FAIL );

    if( access->TapeNumber == MQ_YES && MqErr.errno == EMQNOERROR )
	strcat( string, MqPutTapeNumber( header->TapeNumber ));
    ( MqErr.errno == EMQNOERROR ) ?
        ( MqStatus.TapeNumber = MQ_SUCCESS ) :
	    ( MqStatus.TapeNumber = MQ_FAIL );

    strcat( string, MqSeparator );    /* separator output */
    strcat( string, MqLineEnd );      /* return code output */

/*
 *  File output is made from here.
 */

    ( NULL == filename ) ? ( fileout = MQ_NO ) : ( fileout = MQ_YES );

/*
 *  Outputs header part. A string that stores header contents will be freed
 *  after this output is completely made.
 */

    if( MqErr.errno == EMQNOERROR )    {
	if( fileout == MQ_NO )
	    printf( "%s", string );
	else {
	    if( NULL == ( fcre = fopen( filename, "w+t" ))) {
		MqSetError( EMQFILOPEN );
		MqPerror( filename );
		}
	    fprintf( fcre, "%s", string );
	    }
	}

    free( string );

    /* output data part */
    switch( header->FileType )    {
    case MQ_FULLTEXT:
	if( fileout == MQ_YES )
	    MqPutData( fcre, data, header );
	else
	    MqPutData( stdout, data, header );
	break;
    case MQ_COMPOSITE:
	if( fileout == MQ_YES )
	    fwrite( data, sizeof( double ), header->NumOfData, fcre );
	else
	    fwrite( data, sizeof( double ), header->NumOfData, stdout );
	break;
    case MQ_XDR:
	if( fileout == MQ_YES )
	    xdrstdio_create( &xdrout, fcre, XDR_ENCODE );
	else
	    xdrstdio_create( &xdrout, stdout, XDR_ENCODE );

	for( i = 0; i < header->NumOfData; i++ ) {
	    if( !xdr_double( &xdrout, data+i )) {
		MqSetError( EMQXDRWRITE );
		MqPerror( NULL );
		}
	    }
	break;
    default:
	MqSetError( EMQILFILEFORM );
	MqPerror( NULL );
	}

    if( fileout == MQ_YES ) fclose( fcre );
    }

/* =========================== END of the program ========================== */
