/* ------------------- Moonquake database library routines -------------------+
|                                                                             |
|                          "MqCreFileName" ver 1.00                           |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/*
 *  This subroutine creates filename string according to the regulations noted
 *  in the MQDB document. See the document in detail.
 */

/* ============================= Include files ============================= */
#include <mqdb.h>

/* ========================== Program statements  ========================== */
char *MqCreFileName( unsigned short station, MqTime mqtime, MQDT datatype )
    {
    char    statstr[2],   /* string representing station number */
	    yearstr[5],   /* string representing year */
            daystr[4],    /* string representing day */
            hourstr[3];   /* string representing hour */
    char    *filename;

    MqSetFunc( MQCREFILENAME );
    MqClearError();

    if( NULL == ( filename = calloc( MQFILENAMELEN, 1 ))) {
	MqSetError( EMQMEMALLOC );
	MqPerror( NULL );
	}

    MqCheckStation( station );    /* Data check ! */
    MqCheckTime( &mqtime );       /* Also data check */

    if( MqErr.errno != EMQNOERROR )
	return NULL;

    sprintf( statstr, "%2d", station );
    strncpy( filename,     statstr + 1, 1 );    /* creates filename char 1 */
    sprintf( yearstr, "%4d", mqtime.year );
    strncpy( filename + 1, yearstr + 3, 1 );    /* creates filename char 2 */


    sprintf( daystr, "%03d", mqtime.day );
    strcat( filename, daystr );             /* creates filename char 3-5 */

    sprintf( hourstr, "%02d", mqtime.hour );
    strcat( filename, hourstr );            /* creates filename char 6,7 */
    
    filename[7] = '0';    filename[8] = '\0';

    switch( datatype )    {   /* appends file extension */
    case MQ_LP:
	strcat( filename, LPFileNameExt );
	break;
    case MQ_SP:
	strcat( filename, SPFileNameExt );
	break;
    case MQ_TIDAL:
	strcat( filename, TDLFileNameExt );
	break;
	}

    return filename;
    }

/* =========================== END of the program ========================== */
