/* ------------------- Moonquake database library routines -------------------+
|                                                                             |
|                       "MqCreTmpHdrFileName" ver 1.00                        |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/*
 *  This function creates temporary header file name that stores contents of 
 *  header. The rule of this filename is:
 *
 *    [dir]/[base].(pid).(filedesc)
 *
 *  where, (pid) is current process id and (fildesc) is file description 
 *  number used in this database system. [dir] is currently "/tmp" and
 *  [base] is "mqdb".
 */

/* ============================= Include files ============================= */
#include <sys/types.h>
#include <unistd.h>
#include <mqdb.h>

/* ========================== Program statements  ========================== */
char *MqCreTmpHdrFileName( unsigned short fildesc ) 
    {
    char pidname[6];      /* string of process ID */
    char fildescname[6];  /* string of file descriptor */
    pid_t pid;            /* process ID */
    char *thfname;        /* resultant file name */

    MqSetFunc( MQCRETMPHDRFILENAME );
    MqClearError();

    if( NULL == ( thfname = calloc( FILENAME_MAX, 1 ))) {
	MqSetError( EMQMEMALLOC );
	MqPerror( NULL );
	}

    pid = getpid();

    sprintf( pidname, "%d", pid );          /* create process ID string */
    sprintf( fildescname, "%d", fildesc );  /* create fildesc string */

    strcpy( thfname, MqTmpDirName );        /* "/tmp" */
    strcat( thfname, "/" );                 /* directory separator */
    strcat( thfname, MqTmpFileBase );       /* "mqdb" */
    strcat( thfname, "." );
    strcat( thfname, pidname );
    strcat( thfname, "." );
    strcat( thfname, fildescname );

    if( MqErr.errno == EMQNOERROR )
	return thfname;
    else
	return NULL;
    }

/* =========================== END of the program ========================== */

