/* ------------------- Moonquake database library routines -------------------+
|                                                                             |
|                              "MqGetXXX" ver 1.00                            |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1995 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/*
 *  This file is a set of functions to read headers and data. All functions
 *  are prepared to be called from upper interface function MqRead(),
 *  MqReadData() ( planned ) and MqReadHeader() ( planned ).
 */

/* ============================= Include files ============================= */
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include "mqdb.h"

/* ========================== Program statements  ========================== */

/* ===== Read header of average amplitude ===== */
MqAmp MqGetAveAmp( char *header )
    {
    MqAmp    aveamp;
    long     pointer;
    char info[MQHDRLENMAX],        /* information string */
         hdrline[MQHDRLENMAX],     /* header line */
         aveampstr[MQHDRLENMAX];

    MqClearError();
    MqSetFunc( MQGETAVEAMP );

    strcpy( info, MqAveAmp );

    if( -1 != ( pointer = MqSearchHeader( header, info )))    {
	MqCutLine( header, hdrline, pointer );
	MqSeparateInfoData( hdrline, aveampstr );
	}

    aveamp.channels = MqCountSeparator( ',', aveampstr ) + 1;

    MqConvCharDouble( aveampstr, &aveamp );

    return aveamp;
    }

/* ===== Read header of channnels ===== */
unsigned short MqGetChannels( char *header )
    {
    int    pointer,                /* pointer of the top of a header */
           datalen,                /* length of the data header */
           i;                      /* loop counter */
    char   info[MQHDRLENMAX],      /* information string */
           hdrline[MQHDRLENMAX],   /* header line */
           channels[MQHDRLENMAX];

    strcpy( info, MqChannels );

    MqClearError();
    MqSetFunc( MQGETCHANNELS );

    if( -1 != ( pointer = MqSearchHeader( header, info )))    {
	MqCutLine( header, hdrline, pointer );
	MqSeparateInfoData( hdrline, channels );
	}
    else    {
	MqSetError( EMQNOHEADER );
	MqPerror( info );
	}

    datalen = strlen( channels );
    if( datalen == 0 )    {
	MqSetError( EMQZERODATA );
	MqPerror( NULL );
	}

    /* if non-numerical char found */
    for( i = 0; i < datalen; i++ )    {
	if( isdigit( channels[i] ) == 0 )  { 
	    MqSetError( EMQNONUM );
	    MqPerror( NULL );
	    }
	}

    if( MqErr.errno == EMQNOERROR )
	return (unsigned short)atol( channels );
    else
	return 0;
    }

/* ===== Read data part ===== */
void MqGetData( FILE *frea, double *data, MqHeader *header )    
    {
    unsigned long   j, k, l,   /* loop counter */
                    datanum;   /* number of data lines */
    char           *tmpdata;   /* temporary storage for data line */
    MqYesNo         eof;       /* EOF detector */
    MqAmp           amp;       /* actual data storage */

    MqClearError();
    MqSetFunc( MQGETDATA );

    datanum = header->NumOfData / header->Channels;

    l = k = 0;   amp.channels = header->Channels;
    eof = MQ_NO;
    while( 1 )    {
        if( NULL == ( tmpdata = calloc( MQHDRLINEMAX, 1 )))  {
            MqSetError( EMQMEMALLOC );
            MqPerror( NULL );
            }

        l = 0;
        while( 1 )   {
            if( frea == NULL )    {
                if( EOF == ( tmpdata[l] = getchar() )) {
                    eof = MQ_YES; break;
                    }
		}
            else    {
                if( EOF == ( tmpdata[l] = getc( frea ))) {
                    eof = MQ_YES;  break;
                    }
		}
            if( tmpdata[l-1] == 0x0d && tmpdata[l] == 0x0a && l != 0 )  break;

            l++;
            }    /* inner while loop end */
        if( eof == MQ_YES ) break;

        tmpdata[l-2] = '\0';
        if( MqIsComStart( tmpdata[0] ) == MQ_FALSE )    {
            MqConvCharDouble( tmpdata, &amp );
            for( j = 0; j < amp.channels; j++ )
                data[datanum*j+k] = amp.data[j];
            k++;
            }
        free( tmpdata );
        }
    }

/* ===== Read header of end record ===== */
unsigned long MqGetEndRecord( char *header )
    {
    int    pointer,                /* pointer of the top of a header */
           datalen,                /* length of the data header */
           i;                      /* loop counter */
    char   info[MQHDRLENMAX],      /* information string */
           hdrline[MQHDRLENMAX],   /* header line */
           endrecdata[MQHDRLENMAX];

    strcpy( info, MqEndRecord );

    MqClearError();
    MqSetFunc( MQGETENDRECORD );

    if( -1 != ( pointer = MqSearchHeader( header, info )))    {
	MqCutLine( header, hdrline, pointer );
	MqSeparateInfoData( hdrline, endrecdata );
	}
    else    {
	MqSetError( EMQNOHEADER );
	MqPerror( info );
	}

    datalen = strlen( endrecdata );
    if( datalen == 0 )    {
	MqSetError( EMQZERODATA );
	MqPerror( NULL );
	}

    /* if non-numerical char found */
    for( i = 0; i < datalen; i++ )    {
	if( isdigit( endrecdata[i] ) == 0 )  { 
	    MqSetError( EMQNONUM );
	    MqPerror( NULL );
	    }
	}

    if( MqErr.errno == EMQNOERROR )
	return (unsigned long)atol( endrecdata );
    else
	return 0;
    }

/* ===== Read header of data format ===== */
MQDF MqGetDataFormat( char *header )
    {
    MQDF   format;                /* data format */
    int    pointer,               /* pointer of the top of a header */
           i,                     /* loop counter */
           dfpos;                 /* position of Data Format */
    char   info[MQHDRLENMAX],     /* information string */
           hdrline[MQHDRLENMAX],  /* header line */
           dfchar[MQHDRLENMAX];

    strcpy( info, MqDataFormat );

    MqClearError();
    MqSetFunc( MQGETDATAFORMAT );

    if( -1 != ( pointer = MqSearchHeader( header, info )))    {
	MqCutLine( header, hdrline, pointer );
	MqSeparateInfoData( hdrline, dfchar );
	}
    else    {
	MqSetError( EMQNOHEADER );
	MqPerror( info );
	}
    
    for( i = 0; i < 2; i++ )    {
	if(( strcmp( dfchar, MqDataFormatName[i] ) == 0 ))
	   dfpos = i;
	}

    switch( dfpos )   {
    case 0:
	return MQ_OLD;	break;
    case 1:
	return MQ_NEW;	break;
    default:
	MqSetError( EMQILDATAFORM );
	MqPerror( dfchar );
	return MQ_OLD;
	}
    }

/* ===== Read header of data type ===== */
MQDT MqGetDataType( char *header )
    {
    int    pointer,               /* pointer of the top of a header */
           i,                     /* loop counter */
           typepos;
    char   info[MQHDRLENMAX],     /* information string */
           hdrline[MQHDRLENMAX],  /* header line */
           typechar[MQHDRLENMAX];

    strcpy( info, MqDataType );

    MqClearError();
    MqSetFunc( MQGETDATATYPE );

    if( -1 != ( pointer = MqSearchHeader( header, info )))    {
	MqCutLine( header, hdrline, pointer );
	MqSeparateInfoData( hdrline, typechar );
	}
    else    {
	MqSetError( EMQNOHEADER );
	MqPerror( info );
	}
    
    for( i = 0; i < 3; i++ )    {
	if(( strcmp( typechar, MqDataTypeName[i] ) == 0 ))
	   typepos = i;
	}

    switch( typepos )   {
    case 0:
	return MQ_LP;	break;
    case 1:
	return MQ_SP;	break;
    case 2:
	return MQ_TIDAL; break;
    default:
	MqSetError( EMQILDATATYPE );
	MqPerror( typechar );
	return MQ_LP;
	}
    }

/* ===== Read header of data file type ===== */
MQFT MqGetFileType( char *header )
    {
    long   pointer;
    char   info[MQHDRLENMAX],     /* information string */
           hdrline[MQHDRLENMAX],  /* header line */
           filetype[MQHDRLENMAX];

    MqClearError();
    MqSetFunc( MQGETFILETYPE );

    strcpy( info, MqFileType );

    if( -1 != ( pointer = MqSearchHeader( header, info )))    {
	MqCutLine( header, hdrline, pointer );
	MqSeparateInfoData( hdrline, filetype );
	}
    else    {
	MqSetError( EMQNOHEADER );
	MqPerror( info );
	}

    /* comparison of strings */
    if( strcmp( filetype, MqFileTypeName[0] ) == 0 )
	return MQ_FULLTEXT;
    else if( strcmp( filetype, MqFileTypeName[1] ) == 0 )
	return MQ_COMPOSITE;
    else if( strcmp( filetype, MqFileTypeName[2] ) == 0 )
	return MQ_XDR;
    else   {
	MqSetError( EMQILFILETYPE );
	MqPerror( info );
	return MQ_FULLTEXT;
	}
    }

/* ===== Read header of maximum amplitude ===== */
MqAmp MqGetMaxAmp( char *header )
    {
    MqAmp    maxamp;
    long     pointer;
    char info[MQHDRLENMAX],        /* information string */
         hdrline[MQHDRLENMAX],     /* header line */
         maxampstr[MQHDRLENMAX];

    MqClearError();
    MqSetFunc( MQGETMAXAMP );

    strcpy( info, MqMaxAmp );

    if( -1 != ( pointer = MqSearchHeader( header, info )))    {
	MqCutLine( header, hdrline, pointer );
	MqSeparateInfoData( hdrline, maxampstr );
	}

    maxamp.channels = MqCountSeparator( ',', maxampstr ) + 1;

    MqConvCharDouble( maxampstr, &maxamp );

    return maxamp;
    }

/* ===== Read header of number of data ===== */
unsigned long MqGetNumOfData( char *header )
    {
    int    pointer,                /* pointer of the top of a header */
           datalen,                /* length of the data header */
           i;                      /* loop counter */
    char   info[MQHDRLENMAX],      /* information string */
           hdrline[MQHDRLENMAX],   /* header line */
           numofdata[MQHDRLENMAX];

    strcpy( info, MqNumOfData );

    MqClearError();
    MqSetFunc( MQGETORGFILE );

    if( -1 != ( pointer = MqSearchHeader( header, info )))    {
	MqCutLine( header, hdrline, pointer );
	MqSeparateInfoData( hdrline, numofdata );
	}
    else    {
	MqSetError( EMQNOHEADER );
	MqPerror( info );
	}

    datalen = strlen( numofdata );
    if( datalen == 0 )    {
	MqSetError( EMQZERODATA );
	MqPerror( NULL );
	}

    /* if non-numerical char found */
    for( i = 0; i < datalen; i++ )    {
	if( isdigit( numofdata[i] ) == 0 )  { 
	    MqSetError( EMQNONUM );
	    MqPerror( NULL );
	    }
	}

    if( MqErr.errno == EMQNOERROR )
	return (unsigned long)atol( numofdata );
    else
	return 0;
    }

/* ===== Read header of data format ===== */
MqMode MqGetObsMode( char *header )
    {
    int    pointer,               /* pointer of the top of a header */
           i,                     /* loop counter */
           modepos;               /* position of Observation Mode */
    char   info[MQHDRLENMAX],     /* information string */
           hdrline[MQHDRLENMAX],  /* header line */
           modechar[MQHDRLENMAX];

    strcpy( info, MqObsMode );

    MqClearError();
    MqSetFunc( MQGETOBSMODE );

    if( -1 != ( pointer = MqSearchHeader( header, info )))    {
	MqCutLine( header, hdrline, pointer );
	MqSeparateInfoData( hdrline, modechar );
	}
    else    {
	MqSetError( EMQNOHEADER );
	MqPerror( info );
	}
    
    for( i = 0; i < 2; i++ )    {
	if(( strcmp( modechar, MqObsModeName[i] ) == 0 ))
	    modepos = i;
	}

    switch( modepos )   {
    case 0:
	return MQ_PEAKED;     break;
    case 1:
	return MQ_FLAT;	      break;
    default:
	MqSetError( EMQILOBSMODE );
	MqPerror( modechar );
	return MQ_PEAKED;
	}
    }

/* ===== Read header of original data file name ===== */
void MqGetOrgFile( char *header, char *orgfile )
    {
    int    pointer;               /* pointer of the top of a header */
    char   info[MQHDRLENMAX],     /* information string */
           hdrline[MQHDRLENMAX];  /* header line */

    strcpy( info, MqOrgFile );

    MqClearError();
    MqSetFunc( MQGETORGFILE );

    if( -1 != ( pointer = MqSearchHeader( header, info )))    {
	MqCutLine( header, hdrline, pointer );
	MqSeparateInfoData( hdrline, orgfile );
	}
    else    {
	MqSetError( EMQNOHEADER );
	MqPerror( info );
	}
    }

/* ===== Read header of sampling rate ===== */
MqAmp MqGetSampRate( char *header )
    {
    MqAmp    samprate;
    long     pointer;
    char info[MQHDRLENMAX],        /* information string */
         hdrline[MQHDRLENMAX],     /* header line */
         sampratestr[MQHDRLENMAX];

    MqClearError();
    MqSetFunc( MQGETSAMPRATE );

    strcpy( info, MqSampRate );

    if( -1 != ( pointer = MqSearchHeader( header, info )))    {
	MqCutLine( header, hdrline, pointer );
	MqSeparateInfoData( hdrline, sampratestr );
	}

    samprate.channels = MqCountSeparator( ',', sampratestr ) + 1;
    MqConvCharDouble( sampratestr, &samprate );
    return samprate;
    }

/* ===== Read header of start record ===== */
unsigned long MqGetStartRecord( char *header )
    {
    int    pointer,                /* pointer of the top of a header */
           datalen,                /* length of the data header */
           i;                      /* loop counter */
    char   info[MQHDRLENMAX],      /* information string */
           hdrline[MQHDRLENMAX],   /* header line */
           strecdata[MQHDRLENMAX];

    strcpy( info, MqStartRecord );

    MqClearError();
    MqSetFunc( MQGETSTARTRECORD );

    if( -1 != ( pointer = MqSearchHeader( header, info )))    {
	MqCutLine( header, hdrline, pointer );
	MqSeparateInfoData( hdrline, strecdata );
	}
    else    {
	MqSetError( EMQNOHEADER );
	MqPerror( info );
	}

    datalen = strlen( strecdata );
    if( datalen == 0 )    {
	MqSetError( EMQZERODATA );
	MqPerror( NULL );
	}

    /* if non-numerical char found */
    for( i = 0; i < datalen; i++ )    {
	if( isdigit( strecdata[i] ) == 0 )  { 
	    MqSetError( EMQNONUM );
	    MqPerror( NULL );
	    }
	}

    if( MqErr.errno == EMQNOERROR )
	return (unsigned long)atol( strecdata );
    else
	return 0;
    }

/* ===== Read start time ===== */
MqTime MqGetStartTime( char *header )
    {
    int    pointer,                /* pointer of the top of a header */
           datalen,                /* length of the data header */
           i;                      /* loop counter */
    char   info[MQHDRLENMAX],      /* information string */
           hdrline[MQHDRLENMAX],   /* header line */
           sttimedata[MQHDRLENMAX];
    MqTime mqtime;                 /* returning MqTime structure */

    strcpy( info, MqStartTime );

    MqClearError();
    MqSetFunc( MQGETSTARTTIME );

    if( -1 != ( pointer = MqSearchHeader( header, info )))    {
	MqCutLine( header, hdrline, pointer );
	MqSeparateInfoData( hdrline, sttimedata );
	}
    else    {
	MqSetError( EMQNOHEADER );
	MqPerror( info );
	}

    datalen = strlen( sttimedata );
    if( datalen == 0 )    {
	MqSetError( EMQZERODATA );
	MqPerror( NULL );
	}
    else
	mqtime = MqSeparateMqTime( sttimedata );

    MqCheckTime( &mqtime );

    if( MqErr.errno != EMQNOERROR )
	mqtime.year = mqtime.day = mqtime.hour =
	    mqtime.minute = mqtime.second = mqtime.ms = 0;

    return mqtime;
    }

/* ===== Read header of station ===== */
unsigned short MqGetStation( char *header )
    {
    int    pointer,                /* pointer of the top of a header */
           datalen,                /* length of the data header */
           i;                      /* loop counter */
    unsigned short    stnum;       /* station number */
    char   info[MQHDRLENMAX],      /* information string */
           hdrline[MQHDRLENMAX],   /* header line */
           station[MQHDRLENMAX],   /* data string of "Station" header */
           stnumchar[3];           /* numerical part of the data string */

    strcpy( info, MqStation );

    MqClearError();
    MqSetFunc( MQGETSTATION );

    if( -1 != ( pointer = MqSearchHeader( header, info )))    {
	MqCutLine( header, hdrline, pointer );
	MqSeparateInfoData( hdrline, station );
	}
    else    {
	MqSetError( EMQNOHEADER );
	MqPerror( info );
	}

    /* validity checks of data string: length must be 4 */
    datalen = strlen( station );
    if( datalen == 0 )    {
	MqSetError( EMQZERODATA );
	MqPerror( NULL );
	}
    else if( datalen != 4 )    {
	MqSetError( EMQLONGSTATION );
	MqPerror( station );
	}

    /* validity check: first two letters must be 'A' and 'P' */
    if( station[0] != 'A' || station[1] != 'P' )    {
	MqSetError( EMQILSTATION );
	MqPerror( station );
	}

    /* validity check: third and fourth letters must be numerical */
    if( isdigit( station[2] ) == 0 || isdigit( station[3] ) == 0 )  { 
	MqSetError( EMQNONUM );
	MqPerror( NULL );
	}
    else    {
	stnumchar[0] = station[2];
	stnumchar[1] = station[3];
	stnumchar[2] = '\0';
	}

    stnum = (unsigned short)atoi( stnumchar );
    MqCheckStation( stnum );    /* last validity check */
    if( MqErr.errno == EMQNOERROR )
 	return stnum;
    else
	return 0;
    }

/* ===== Read header of tape number ===== */
unsigned short MqGetTapeNumber( char *header )
    {
    int    pointer,                /* pointer of the top of a header */
           datalen,                /* length of the data header */
           i;                      /* loop counter */
    char   info[MQHDRLENMAX],      /* information string */
           hdrline[MQHDRLENMAX],   /* header line */
           tapenum[MQHDRLENMAX];   /* data string of "Tape Number" header */

    strcpy( info, MqTapeNumber );

    MqClearError();
    MqSetFunc( MQGETTAPENUMBER );

    if( -1 != ( pointer = MqSearchHeader( header, info )))    {
	MqCutLine( header, hdrline, pointer );
	MqSeparateInfoData( hdrline, tapenum );
	}
    else    {
	MqSetError( EMQNOHEADER );
	MqPerror( info );
	}

    /* validity checks of data string: length must be 4 */
    datalen = strlen( tapenum );
    if( datalen == 0 )    {
	MqSetError( EMQZERODATA );
	MqPerror( NULL );
	}

    /* validity check */
    for( i = 0; i < datalen; i++ )    {
	if( isdigit( tapenum[i] ) == 0 )  { 
	    MqSetError( EMQNONUM );
	    MqPerror( NULL );
	    }
	}

    if( MqErr.errno == EMQNOERROR )
 	return (unsigned short)atol( tapenum );
    else
	return 0;
    }

/* =========================== END of the program ========================== */
