/* ------------------- Moonquake database library routines -------------------+
|                                                                             |
|                           "MqGetTopLine" ver 1.00                           |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1995 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/*
 *  This function reads top line of MQDB format file and specifies whether 
 *  assigned file is MQDB format. Next, if file is MQDB format, this function
 *  reads library version and creator signature and stores these to the 
 *  structure libinfo, which will be returned.
 */

/* ============================= Include files ============================= */
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <mqdb.h>

/* ========================== Program statements  ========================== */
MqLibInfo MqGetTopLine( FILE *fin )
    {
    MqLibInfo   libinfo;
    char       *topline;
    unsigned int      i;
    unsigned long     toplinelen,   /* length of the top line */
                      progsignlen;  /* length of the progsign strings */

    MqSetFunc( MQGETTOPLINE );
    MqClearError();

    topline = calloc( MQHDRLENMAX, 1 );
    if( NULL == topline ) {
	MqSetError( EMQMEMALLOC );
	MqPerror( NULL );
	}

    /* first, reads two characters to specify */
    for( i = 0; i < 2; i++ ) {
	if( NULL == fin ) {         /* if file is stdin */
	    if( EOF == ( topline[i] = getchar() )) {
		MqSetError( EMQSHORTFILE );
		MqPerror( NULL );
		}
	    }
	else {                  /* actual file input */
	    if( EOF == ( topline[i] = getc( fin ))) {
		MqSetError( EMQSHORTFILE );
		MqPerror( NULL );
		}
	    }
	}

    /* and checks whether file is MQDB format */
    if( topline[0] != '@' && topline[1] != '@' ) {
	MqSetError( EMQILMQDBFORM );
	MqPerror( NULL );
	}

    /* reads the top line till detects first CR+LF */
    i = 2;
    while( 1 )  {
	if( fin == NULL ) {      /* if file is stdin */
	    if( EOF == ( topline[i] = getchar() )) {
		MqSetError( EMQNODATASEP );
		MqPerror( NULL );
		}
	    }
	else {                   /* actual file input */
	    if( EOF == ( topline[i] = getc( fin ))) {
		MqSetError( EMQNODATASEP );
		MqPerror( NULL );
		}
	    }
	if( topline[i-1] == 0x0d && topline[i] == 0x0a ) break;

	i++;
	if( i > MQHDRLENMAX ) {
	    MqSetError( EMQLONGHDR );
	    MqPerror( NULL );
	    break;
	    }
	}

    /*
     *  Counts number of characters, and exits if line is too short
     *  Top line must be 12 characters contained at least, that is
     *      @@           2
     *      SPC          1
     *      MQ100        5
     *      SPC          1
     *      ProgSign     1 ( at least )
     *      CR+LF        2
     */

    toplinelen = strlen( topline );

    if( toplinelen < MQTOPLINEMIN )  {
	MqSetError( EMQSHORTTOP );
	MqPerror( topline );
	}

    /* if SPC not found in 2nd and 8th chars, then exits */
    if( topline[2] != ' ' && topline[8] != ' ' ) {
	MqSetError( EMQNOTLSEP );
	MqPerror( topline );
	}

    /* if 3rd & 4th char is not alphabetical, then warns */
    if( isalpha( topline[3] ) == MQ_FALSE ||
       isalpha( topline[4] ) == MQ_FALSE )   {
	MqSetError( EMQILLIBTYPE );
	MqPerror( topline );
	libinfo.LibVer.LibType[0] = libinfo.LibVer.LibType[1] = ' ';
	}
    else  {
	libinfo.LibVer.LibType[0] = topline[3];
	libinfo.LibVer.LibType[1] = topline[4];
	}

    /* if 5th char ( release number ) is not numerical, then warns */
    if( isdigit( topline[5] ) == MQ_FALSE )  {
	MqSetError( EMQILLIBREL );
	MqPerror( topline );
	libinfo.LibVer.Release = '0';
	}
    else
	libinfo.LibVer.Release = topline[5];

    /* if 6th char ( major number ) is not numerical, then warns */
    if( isdigit( topline[6] ) == MQ_FALSE ) {
	MqSetError( EMQILLIBMAJ );
	MqPerror( topline );
	libinfo.LibVer.Major = '0';
	}
    else
	libinfo.LibVer.Major = topline[6];

    /* if 7th char ( minor number ) is not numerical, then warns */
    if( isdigit( topline[7] ) == MQ_FALSE ) {
	MqSetError( EMQILLIBMIN );
	MqPerror( topline );
	libinfo.LibVer.Minor = '0';
	}
    else
	libinfo.LibVer.Minor = topline[7];

    /* finally, separates program signature from top line strings and 
       copy it to the structure. */

    progsignlen = toplinelen - 11;
    for( i = 0; i < progsignlen; i++ )
	libinfo.ProgSign[i] = topline[i+9];
    libinfo.ProgSign[progsignlen] = '\0';

    free( topline );
    return libinfo;
    }

/* =========================== END of the program ========================== */
