/* ------------------- Moonquake database library routines -------------------+
|                                                                             |
|                              "MqInit" ver 1.00                              |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/*
 *  This is a initializer of the Moonquake Database System. MqInit() sets
 *  static variables MqErr, MqStatus and MqLibVersion as an default value.
 *  This function must be called at least once when using Moonquake Database
 *  System.
 */

/* ============================= Include files ============================= */
#include <mqdb.h>

/* ========================== Program statements  ========================== */
#ifdef FORTRAN
void mqinit_( char *progsign )
#else
void MqInit( char *progsign )
#endif
    {
    unsigned short  i;    /* loop counter */

    MqSetFunc( MQINIT );
    MqClearError();

    MqErr.warn = EMQ_WARNING;    /* warning level initialization */

       /* initialization of access status structure */
    MqStatus.AveAmp      = MQ_NOACCESS;
    MqStatus.Channels    = MQ_NOACCESS;
    MqStatus.DataCreDate = MQ_NOACCESS;
    MqStatus.DataFormat  = MQ_NOACCESS;
    MqStatus.DataModDate = MQ_NOACCESS;
    MqStatus.DataType    = MQ_NOACCESS;
    MqStatus.EndRecord   = MQ_NOACCESS;
    MqStatus.FileType    = MQ_NOACCESS;   
    MqStatus.MaxAmp      = MQ_NOACCESS;     
    MqStatus.NumOfData   = MQ_NOACCESS;  
    MqStatus.ObsMode     = MQ_NOACCESS;    
    MqStatus.OrgFileName = MQ_NOACCESS;
    MqStatus.SampRate    = MQ_NOACCESS;   
    MqStatus.Station     = MQ_NOACCESS;    
    MqStatus.StartRecord = MQ_NOACCESS;
    MqStatus.StartTime   = MQ_NOACCESS;  
    MqStatus.TapeNumber  = MQ_NOACCESS; 

    /* library version are set */
    strncpy( MqLib.LibVer.LibType, MQLIB_LIBTYPE, 2 );

    if( MQLIB_RELEASE < 0 || MQLIB_RELEASE > 9 ) {   /* release number */
	MqSetError( EMQILLIBREL );
	MqPerror( NULL );
	}
    else {
	sprintf( &MqLib.LibVer.Release, "%1d", MQLIB_RELEASE );
    }

    if( MQLIB_MAJOR < 0 || MQLIB_MAJOR > 9 ) {       /* major number */
	MqSetError( EMQILLIBMAJ );
	MqPerror( NULL );
	}
    else {
	sprintf( &MqLib.LibVer.Major,   "%1d", MQLIB_MAJOR );
	}

    if( MQLIB_MINOR < 0 || MQLIB_MINOR > 9 ) {       /* minor number */
	MqSetError( EMQILLIBMIN );
	MqPerror( NULL );
	}
    else {
	sprintf( &MqLib.LibVer.Minor,   "%1d", MQLIB_MINOR );
	}

    if( progsign != NULL ) {   /* copy program signature to structure */
	for( i = 0; i < 14; i++ ) {
	    MqLib.ProgSign[i] = progsign[i];
	    MqLib.ProgSign[15]='\0';
	    }
	}
    /* initialization of file descriptors */
    for( i = 0; i < MQFILDESCMAX; i++ ) {
	MqFilDesc[i].InUse     = MQ_NO;
	MqFilDesc[i].accmethod = MQ_TMP;
	MqFilDesc[i].iostream  = MQ_FILE;
	MqFilDesc[i].ReadWrite = MQ_UNKNOWN;
	MqFilDesc[i].filename  = NULL;
	sprintf( &MqFilDesc[i].libinfo.LibVer.Release, "%1d", MQLIB_RELEASE );
	sprintf( &MqFilDesc[i].libinfo.LibVer.Major, "%1d", MQLIB_MAJOR );
	sprintf( &MqFilDesc[i].libinfo.LibVer.Minor, "%1d", MQLIB_MINOR );
	}
    }

/* =========================== END of the program ========================== */
