/* ------------------- Moonquake database library routines -------------------+
|                                                                             |
|                               "MqPut" ver 2.00                              |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/*
 *  This file is a set of a subroutines "MqPutXXX", creating header strings.
 */

/* ============================= Include files ============================= */
#include <mqdb.h>

/* ========================== Program statements  ========================== */

/* ===== Create string of the average amplitude ===== */
char *MqPutAveAmp( MqAmp aveamp )
    {
    char *header,     /* resultant header string */
         *aveampstr,  /* storage for keeping data strings */
         *tmpstr;     /* temporary storage for keeping each data */
    unsigned short    i;    /* loop counter */

    MqSetFunc( MQPUTAVEAMP );
    MqClearError();

    if( NULL == ( aveampstr = calloc( 80, 1 )))   {
	MqSetError( EMQMEMALLOC );
	MqPerror( NULL );
	}

    if( aveamp.channels == 0 )    {
	MqSetError( EMQZEROAMPCHNL );
	MqPerror( NULL );
	}

    if( MqErr.errno == EMQNOERROR )    {
	for( i = 0; i < aveamp.channels; i++ )    {
	    if( NULL == ( tmpstr = calloc( 20, 1 )))   {
		MqSetError( EMQMEMALLOC );
		MqPerror( NULL );
		}
	    if( i == ( aveamp.channels - 1 ))
		sprintf( tmpstr, "%f", aveamp.data[i] );
	    else
		sprintf( tmpstr, "%f,", aveamp.data[i] );
	    
	    strcat( aveampstr, tmpstr );
	    free( tmpstr );
	    }
	}

    if( NULL == ( header = MqCatInfoData( MqAveAmp, aveampstr ))) {
	MqSetError( EMQCATINFODATA );
	MqPerror( NULL );
	}

    free( aveampstr );

    if( MqErr.errno == EMQNOERROR )
	return header;
    else
	return NULL;
    }

/* ===== Create string of the channels ===== */
char *MqPutChannels( unsigned short channels )
    {
    char       *channelnum,    /* string to store number of channels */
               *header;        /* resultant header string */

    MqSetFunc( MQPUTCHANNELS );
    MqClearError();

    if( NULL == ( channelnum = calloc( 2, 1 )))    {
	MqSetError( EMQMEMALLOC );
	MqPerror( NULL );
	}

    sprintf( channelnum, "%d", channels );

    if( NULL == ( header = MqCatInfoData( MqChannels, channelnum ))) {
	MqSetError( EMQCATINFODATA );
	MqPerror( NULL );
	}

    free( channelnum );
    if( MqErr.errno == EMQNOERROR )
	return header;
    else
	return NULL;
    }

/* ===== Create string of the data creation date ===== */
char *MqPutCreateDate( void )
    {
    time_t     cretime;         /* data creation date ( time_t ) */
    char      *cretime_char,    /* data creation date ( string ) */
              *header;          /* resultant header string */

    MqSetFunc( MQPUTCREATEDATE );
    MqClearError();

    if( NULL == ( cretime_char = calloc( 80, 1 )))    {
	MqSetError( EMQMEMALLOC );
	MqPerror( NULL );
	}

    cretime = time( NULL );
    strftime( cretime_char, 80, "%a %b %d %H:%M:%S %Y %Z",
	     localtime( &cretime ));

    if( NULL == ( header = MqCatInfoData( MqCreDate, cretime_char ))) {
	MqSetError( EMQCATINFODATA );
	MqPerror( NULL );
	}

    free( cretime_char );
    if( MqErr.errno == EMQNOERROR )
	return header;
    else
	return NULL;
    }

/* ===== Create data part ===== */
void MqPutData( FILE *fcre, double *mqdata, MqHeader *mqheader )
    {
    char    *dataline;
    int      linenum,     /* number of data lines */
             i, j;        /* loop counter */

    MqSetFunc( MQPUTDATA );
    MqClearError();

    if( NULL == ( dataline = calloc( MQHDRLINEMAX, 1 )))   {
	MqSetError( EMQMEMALLOC );
	MqPerror( NULL );
	}

    if( NULL == mqdata || NULL == mqheader )    {
	MqSetError( EMQNULLPTR );
	MqPerror( NULL );
	}

    if( mqheader->Channels == 0 )    {
	MqSetError( EMQZEROCHANNEL );
	MqPerror( NULL );
	}

    if( mqheader->NumOfData == 0 )   {
	MqSetError( EMQZERONOD );
	MqPerror( NULL );
	}

    if( mqheader->NumOfData % mqheader->Channels != 0 )    {
	MqSetError( EMQILNOD );
	MqPerror( NULL );
	}

    linenum = mqheader->NumOfData / mqheader->Channels;

    for( i = 0; i < linenum; i++ )    {
	for( j = 0; j < mqheader->Channels; j++ )    {
	    if( fcre == NULL )    {
		printf( "%lf", mqdata[i+linenum*j] );
		( j == ( mqheader->Channels - 1 )) ?
		    ( printf( "%s", MqLineEnd )) :
			( printf( "%s", MqDataSeparator ));
		}
	    else    {
		fprintf( fcre, "%lf", mqdata[i+linenum*j] );
		( j == ( mqheader->Channels - 1 )) ?
		    ( fprintf( fcre, "%s", MqLineEnd )) :
			( fprintf( fcre, "%s", MqDataSeparator ));
		}
	    }
	}

    free( dataline );
    }

/* ===== Create header strings of data format ===== */
char *MqPutDataFormat( MQDF format )
    {
    char     *header;

    MqSetFunc( MQPUTDATAFORMAT );
    MqClearError();

    switch( format )    {
    case MQ_OLD:
	if( NULL == ( header = MqCatInfoData( MqDataFormat, "OLD" ))) {
	    MqSetError( EMQCATINFODATA );
	    MqPerror( NULL );
	    }
	break;
    case MQ_NEW:
	if( NULL == ( header = MqCatInfoData( MqDataFormat, "NEW" ))) {
	    MqSetError( EMQCATINFODATA );
	    MqPerror( NULL );
	    }
	break;
    default:
	MqSetError( EMQILDATAFORM );
	MqPerror( NULL );
	}

    if( MqErr.errno == EMQNOERROR )
	return header;
    else
	return NULL;
    }

/* ===== Create header strings of data type ===== */
char *MqPutDataType( MQDT mqtype )
    {
    char     *header;     /* resultant header string */

    MqSetFunc( MQPUTDATATYPE );
    MqClearError();

    switch( mqtype )    {
    case MQ_LP:
	if( NULL == ( header = MqCatInfoData( MqDataType, "LP" ))) {
	    MqSetError( EMQCATINFODATA );
	    MqPerror( NULL );
	    }
	break;
    case MQ_SP:
	if( NULL == ( header = MqCatInfoData( MqDataType, "SP" ))) {
	    MqSetError( EMQCATINFODATA );
	    MqPerror( NULL );
	    }
	break;
    case MQ_TIDAL:
	if( NULL == ( header = MqCatInfoData( MqDataType, "TIDAL" ))) {
	    MqSetError( EMQCATINFODATA );
	    MqPerror( NULL );
	    }
	break;
    default:
	MqSetError( EMQILDATATYPE );
	MqPerror( NULL );
	}

    if( MqErr.errno == EMQNOERROR )
	return header;
    else
	return NULL;
    }

/* ===== Create header strings of end record ===== */
char *MqPutEndRecord( unsigned long record )
    {
    char  *endrec,   /* strings for end record */
          *header;   /* resultant header string */

    MqSetFunc( MQPUTENDRECORD );
    MqClearError();

    if( NULL == ( endrec = calloc( 16, 1 )))    {
	MqSetError( EMQMEMALLOC );
	MqPerror( NULL );
	}

    sprintf( endrec, "%ld", record );

    if( NULL == ( header = MqCatInfoData( MqEndRecord, endrec ))) {
	MqSetError( EMQCATINFODATA );
	MqPerror( NULL );
	}

    free( endrec );

    if( MqErr.errno == EMQNOERROR )
	return header;
    else
	return NULL;
    }

/* ===== Create string of the file type header ===== */
char *MqPutFileType( MQFT file )
    {
    char *header,    /* resultant header string */
         *filetype;  /* string of data "File Type" */

    MqSetFunc( MQPUTFILETYPE );
    MqClearError();

    if( NULL == ( filetype = calloc( 255, 1 )))   {
	MqSetError( EMQMEMALLOC );
	MqPerror( NULL );
	}

    switch( file )    {
    case MQ_FULLTEXT:
	strcpy( filetype, MqFileTypeName[0] );
	break;
    case MQ_COMPOSITE:
	strcpy( filetype, MqFileTypeName[1] );
	break;
    case MQ_XDR:
	strcpy( filetype, MqFileTypeName[2] );
	break;
    default:
	MqSetError( EMQILFILETYPE );
	MqPerror( NULL );
	}

    if( NULL == ( header = MqCatInfoData( MqFileType, filetype ))) {
	MqSetError( EMQCATINFODATA );
	MqPerror( NULL );
	}

    if( MqErr.errno == EMQNOERROR )
	return header;
    else
	return NULL;
    }

/* ===== Create string of the maximum amplitude ===== */
char *MqPutMaxAmp( MqAmp maxamp )
    {
    char *header,     /* resultant header string */
         *maxampstr,  /* storage for keeping data strings */
         *tmpstr;     /* temporary storage for keeping each data */
    unsigned short    i;    /* loop counter */

    MqSetFunc( MQPUTMAXAMP );
    MqClearError();

    if( NULL == ( maxampstr = calloc( 80, 1 )))   {
	MqSetError( EMQMEMALLOC );
	MqPerror( NULL );
	}

    if( maxamp.channels == 0 )    {
	MqSetError( EMQZEROAMPCHNL );
	MqPerror( NULL );
	}

    if( MqErr.errno == EMQNOERROR )    {
	for( i = 0; i < maxamp.channels; i++ )    {
	    if( NULL == ( tmpstr = calloc( 20, 1 )))   {
		MqSetError( EMQMEMALLOC );
		MqPerror( NULL );
		}
	    if( i == ( maxamp.channels - 1 ))
		sprintf( tmpstr, "%f", maxamp.data[i] );
	    else
		sprintf( tmpstr, "%f,", maxamp.data[i] );

	    strcat( maxampstr, tmpstr );
	    free( tmpstr );
	    }
	}

    if( NULL == ( header = MqCatInfoData( MqMaxAmp, maxampstr ))) {
	MqSetError( EMQCATINFODATA );
	MqPerror( NULL );
	}
    
    free( maxampstr );

    if( MqErr.errno == EMQNOERROR )
	return header;
    else
	return NULL;
    }

/* ===== Create header strings of original tape sequence number ===== */
char *MqPutNumOfData( unsigned long nod )
    {
    char   *numchar,     /* strings of "Number of Data" */
           *header;      /* resultant header string */

    if( NULL == ( numchar = calloc( 80, 1 )))    {
	MqSetError( EMQMEMALLOC );
	MqPerror( NULL );
	}

    MqSetFunc( MQPUTNUMOFDATA );
    MqClearError();

    sprintf( numchar, "%ld", nod );
    if( NULL == ( header = MqCatInfoData( MqNumOfData, numchar ))) {
	MqSetError( EMQCATINFODATA );
	MqPerror( NULL );
	}

    free( numchar );

    if( MqErr.errno == EMQNOERROR )
	return header;
    else
	return NULL;
    }

/* ===== Create string of the observation mode ===== */
char *MqPutObsMode( MqMode obsmode )
    {
    char     *header;   /* resultant header string */

    switch( obsmode )    {
    case MQ_FLAT:
	if( NULL == ( header = MqCatInfoData( MqObsMode, "FLAT" ))) {
	    MqSetError( EMQCATINFODATA );
	    MqPerror( NULL );
	    }
	break;
    case MQ_PEAKED:
	if( NULL == ( header = MqCatInfoData( MqObsMode, "PEAKED" ))) {
	    MqSetError( EMQCATINFODATA );
	    MqPerror( NULL );
	    }
	break;
    default:
	MqSetError( EMQILOBSMODE );
	MqPerror( NULL );
	}

    if( MqErr.errno == EMQNOERROR )
	return header;
    else
	return NULL;
    }

/* ===== Create string of the original data file name ===== */
char *MqPutOrgFile( char *filename )
    {
    char      *header;    /* resultant header string */
   
    MqSetFunc( MQPUTORGFILE );
    MqClearError();

    if( filename == NULL )   {
	MqSetError( EMQNULLPTR );
	MqPerror( NULL );
	}

    if( NULL == ( header = MqCatInfoData( MqOrgFile, filename ))) {
	MqSetError( EMQCATINFODATA );
	MqPerror( NULL );
	}

    if( MqErr.errno == EMQNOERROR )
	return header;
    else
	return NULL;
    }

/* ===== Create string of the maximum amplitude ===== */
char *MqPutSampRate( MqAmp samprate )
    {
    char *header,     /* resultant header string */
         *sampstr,    /* storage for keeping data strings */
         *tmpstr;     /* temporary storage for keeping each data */
    unsigned short    i;    /* loop counter */

    MqSetFunc( MQPUTSAMPRATE );
    MqClearError();

    if( NULL == ( sampstr = calloc( 80, 1 )))   {
	MqSetError( EMQMEMALLOC );
	MqPerror( NULL );
	}

    if( samprate.channels == 0 )   {
	MqSetError( EMQZEROAMPCHNL );
	MqPerror( NULL );
	}

    if( MqErr.errno == EMQNOERROR )    {
	for( i = 0; i < samprate.channels; i++ )    {
	    if( NULL == ( tmpstr = calloc( 20, 1 )))   {
		MqSetError( EMQMEMALLOC );
		MqPerror( NULL );
		}
	    if( i == ( samprate.channels - 1 ))
		sprintf( tmpstr, "%f", samprate.data[i] );
	    else
		sprintf( tmpstr, "%f,", samprate.data[i] );

	    strcat( sampstr, tmpstr );
	    free( tmpstr );
	    }
	}

    if( NULL == ( header = MqCatInfoData( MqSampRate, sampstr ))) {
	MqSetError( EMQCATINFODATA );
	MqPerror( NULL );
	}
    
    free( sampstr );

    if( MqErr.errno == EMQNOERROR )
	return header;
    else
	return NULL;
    }

/* ===== Create header strings of start record ===== */
char *MqPutStartRecord( unsigned long record )
    {
    char  *startrec,    /* string of "Start Record" */
          *header;      /* resultant header string */

    MqSetFunc( MQPUTSTARTRECORD );
    MqClearError();

    if( NULL == ( startrec = calloc( 16, 1 )))    {
	MqSetError( EMQMEMALLOC );
	MqPerror( NULL );
	}

    sprintf( startrec, "%ld", record );

    if( NULL == ( header = MqCatInfoData( MqStartRecord, startrec ))) {
	MqSetError( EMQCATINFODATA );
	MqPerror( NULL );
	}

    free( startrec );

    if( MqErr.errno == EMQNOERROR )
	return header;
    else
	return NULL;
    }

/* ===== Create header strings of start time ===== */
char *MqPutStartTime( MqTime start )
    {
    char  *StartTime,   /* string of "Start Time" */
          *header;      /* resultant header string */

    MqSetFunc( MQPUTSTARTTIME );
    MqClearError();

    if( NULL == ( StartTime = calloc( 80, 1 )))    {
	MqSetError( EMQMEMALLOC );
	MqPerror( NULL );
	}

    if( MqErr.errno == EMQNOERROR )    {
	sprintf( StartTime, "%d %d %d %d %d %d",
		start.year, start.day, start.hour, start.minute,
		start.second, start.ms );
	if( NULL == ( header = MqCatInfoData( MqStartTime, StartTime ))) {
	    MqSetError( EMQCATINFODATA );
	    MqPerror( NULL );
	    }
	}

    free( StartTime );

    if( MqErr.errno == EMQNOERROR )
	return header;
    else
	return NULL;
    }

/* ===== Create strings of Observation station ===== */
char *MqPutStation( unsigned short station )
    {
    char  *stachar,     /* string of "station" */
          *header;      /* resultant header string */

    MqClearError();
    MqSetFunc( MQPUTSTATION );

    if( NULL == ( stachar = calloc( 10, 1 )))    {
	MqSetError( EMQMEMALLOC );
	MqPerror( NULL );
	}

    MqCheckStation( station );

    if( MqErr.errno == EMQNOERROR )   {
	sprintf( stachar, "AP%d", station );
	if( NULL == ( header = MqCatInfoData( MqStation, stachar ))) {
	    MqSetError( EMQCATINFODATA );
	    MqPerror( NULL );
	    }
	}

    free( stachar );

    if( MqErr.errno == EMQNOERROR )
	return header;
    else
	return NULL;
    }

/* ===== Create header strings of original tape sequence number ===== */
char *MqPutTapeNumber( unsigned short datanum )
    {
    char   *datachar,    /* strings of "datanum" */
           *header;      /* resultant header string */

    MqSetFunc( MQPUTTAPENUMBER );
    MqClearError();

    if( NULL == ( datachar = calloc( 80, 1 )))    {
	MqSetError( EMQMEMALLOC );
	MqPerror( NULL );
	}

    sprintf( datachar, "%d", datanum );
    if( NULL == ( header = MqCatInfoData( MqTapeNumber, datachar ))) {
	MqSetError( EMQCATINFODATA );
	MqPerror( NULL );
	}

    free( datachar );

    if( MqErr.errno == EMQNOERROR )
	return header;
    else
	return NULL;
    }

/* =========================== END of the program ========================== */
