/* ------------------- Moonquake database library routines -------------------+
|                                                                             |
|                              "MqRead" ver 1.10                              |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/* ============================= Include files ============================= */
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include "mqdb.h"
#ifdef HAVE_XDR
#include <rpc/rpc.h>
#else
#include <types.h>
#include <xdr.h>
#endif

/* ========================== Program statements  ========================== */
double *MqRead( char *fin, MqAccess *access, MqHeader *header )
    {
    double         *data;
    FILE           *frea;
    char           *tmphdr,     /* temporary header string storage */
                   *tmpskip;    /* character for skipping data */
    XDR             xdrin;      /* XDR data structure */
    unsigned int    i = 1, j, datanum, readnum;

    MqSetFunc( MQREAD );
    MqClearError();

    if( NULL == access || NULL == header )    {
	MqSetError( EMQNULLPTR );
	MqPerror( NULL );
	}

    tmphdr = calloc( MQHDRLENMAX, 1 );
    tmpskip = calloc( MQHDRLENMAX, 1 );

    if( NULL == tmphdr || NULL == tmpskip )  {
	MqSetError( EMQMEMALLOC );
	MqPerror( NULL );
	}

    if( NULL != fin )    {
	if( NULL == ( frea = fopen( fin, "r+t" )))    {
	    perror( fin );
	    MqSetError( EMQFILOPEN );
	    MqPerror( fin );
	    }
	MqLib = MqGetTopLine( frea );   /* Reads top line */
	tmphdr = MqGetHeader( frea );              /* gets header */
	}
    else {
	MqLib = MqGetTopLine( NULL );   /* Reads top line */
	tmphdr = MqGetHeader( NULL );              /* gets header */
	frea = NULL;
	}

    if( access->Channels == MQ_YES )
	header->AveAmp = MqGetAveAmp( tmphdr );
    ( MqErr.errno == EMQNOERROR ) ?
	( MqStatus.AveAmp = MQ_SUCCESS ) : ( MqStatus.AveAmp = MQ_FAIL );
	
    if( access->Channels == MQ_YES )
	header->Channels = MqGetChannels( tmphdr );
    ( MqErr.errno == EMQNOERROR ) ?
	( MqStatus.Channels = MQ_SUCCESS ) : ( MqStatus.Channels = MQ_FAIL );

    if( access->DataFormat == MQ_YES )
	header->DataFormat = MqGetDataFormat( tmphdr );
    ( MqErr.errno == EMQNOERROR ) ?
	( MqStatus.DataFormat = MQ_SUCCESS ) :
	    ( MqStatus.DataFormat = MQ_FAIL );

    if( access->DataType == MQ_YES )
	header->DataType = MqGetDataType( tmphdr );
    ( MqErr.errno == EMQNOERROR ) ?
	( MqStatus.DataType = MQ_SUCCESS ) : ( MqStatus.DataType = MQ_FAIL );

    if( access->EndRecord == MQ_YES )
	header->EndRecord = MqGetEndRecord( tmphdr );
    ( MqErr.errno == EMQNOERROR ) ?
	( MqStatus.EndRecord = MQ_SUCCESS ) :
	    ( MqStatus.EndRecord = MQ_FAIL );

    if( access->FileType == MQ_YES )
	header->FileType = MqGetFileType( tmphdr );
    ( MqErr.errno == EMQNOERROR ) ?
	( MqStatus.FileType = MQ_SUCCESS ) : ( MqStatus.FileType = MQ_FAIL );

    if( access->MaxAmp == MQ_YES )
	header->MaxAmp = MqGetMaxAmp( tmphdr );
    ( MqErr.errno == EMQNOERROR ) ?
	( MqStatus.MaxAmp = MQ_SUCCESS ) : ( MqStatus.MaxAmp = MQ_FAIL );

    if( access->NumOfData == MQ_YES )
	header->NumOfData = MqGetNumOfData( tmphdr );
    ( MqErr.errno == EMQNOERROR ) ?
	( MqStatus.NumOfData = MQ_SUCCESS ) : ( MqStatus.NumOfData = MQ_FAIL );

    if( access->ObsMode == MQ_YES )
	header->ObsMode = MqGetObsMode( tmphdr );
    ( MqErr.errno == EMQNOERROR ) ?
	( MqStatus.ObsMode = MQ_SUCCESS ) : ( MqStatus.ObsMode = MQ_FAIL );

    if( access->OrgFileName == MQ_YES )
	MqGetOrgFile( tmphdr, header->OrgFile );
    ( MqErr.errno == EMQNOERROR ) ?
	( MqStatus.OrgFileName = MQ_SUCCESS ) :
	    ( MqStatus.OrgFileName = MQ_FAIL );

    if( access->SampRate == MQ_YES )
	header->SampRate = MqGetSampRate( tmphdr );
    ( MqErr.errno == EMQNOERROR ) ?
	( MqStatus.SampRate = MQ_SUCCESS ) : ( MqStatus.SampRate = MQ_FAIL );

    if( access->StartRecord == MQ_YES )
	header->StartRecord = MqGetStartRecord( tmphdr );
    ( MqErr.errno == EMQNOERROR ) ?
	( MqStatus.StartRecord = MQ_SUCCESS ) :
	    ( MqStatus.StartRecord = MQ_FAIL );

    if( access->StartTime == MQ_YES )
	header->StartTime = MqGetStartTime( tmphdr );
    ( MqErr.errno == EMQNOERROR ) ?
	( MqStatus.StartTime = MQ_SUCCESS ) :
	    ( MqStatus.StartTime = MQ_FAIL );

    if( access->Station == MQ_YES )
	header->Station = MqGetStation( tmphdr );
    ( MqErr.errno == EMQNOERROR ) ?
	( MqStatus.Station = MQ_SUCCESS ) : ( MqStatus.Station = MQ_FAIL );

    if( access->TapeNumber == MQ_YES )
	header->TapeNumber = MqGetTapeNumber( tmphdr );
    ( MqErr.errno == EMQNOERROR ) ?
	( MqStatus.TapeNumber = MQ_SUCCESS ) :
	    ( MqStatus.TapeNumber = MQ_FAIL );

    free( tmphdr );

    /* skips characters next to the header/data separator "@@" */
    i = 0;
    while( 1 )    { 
	if( fin == NULL )    {
	    if( EOF == ( tmpskip[i] = getchar() ))    {
		MqSetError( EMQNODATASEP );
		MqPerror( NULL );
		}
	    }
	else    {
	    if( EOF == ( tmpskip[i] = getc( frea )))   {
		MqSetError( EMQNODATASEP );
		MqPerror( NULL );
		}
	    }
	if( tmpskip[i-1] == 0x0d && tmpskip[i] == 0x0a && i != 0 ) break;

	i++;
	}

    if( header->NumOfData == 0 )    {
	MqSetError( EMQZERONOD );
	MqPerror( NULL );
	}
    else    {
	if( NULL == ( data = calloc( header->NumOfData, sizeof( double ))))
	    return NULL;
	}

    if( header->Channels == 0 )    {
	MqSetError( EMQZEROCHANNEL );
	MqPerror( NULL );
	}

    free( tmpskip );

   /* Read the data part */
    switch( header->FileType )    {
    case MQ_FULLTEXT:
	MqGetData( frea, data, header );
	break;
    case MQ_COMPOSITE:
	if( NULL != fin )
	    fread( data, sizeof( double ), header->NumOfData, frea );
	else {
	    for( i = 0; i < header->NumOfData; i++ )
		readnum = fread( data+i, sizeof( double ), 1, stdin );
	    }
	break;
    case MQ_XDR:
	if( NULL != fin )
	    xdrstdio_create( &xdrin, frea, XDR_DECODE );
	else
	    xdrstdio_create( &xdrin, stdin, XDR_DECODE );

	for( i = 0; i < header->NumOfData; i++ ) {
	    if( !xdr_double( &xdrin, data+i )) {
		MqSetError( EMQXDRREAD );
		MqPerror( NULL );
		}
	    }
	break;
    default:
	MqSetError( EMQILFILEFORM );
	MqPerror( NULL );
	}

    if( NULL != fin ) fclose( frea );

    return data;
    }

/* =========================== END of the program ========================== */
