/* ------------------- Moonquake database library routines -------------------+
|                                                                             |
|                               "MQCD" ver 1.00                               |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|         Copyright (c) 1993 by Jun-ya Terazono, all rights reserved.         |
+----------------------------------------------------------------------------*/

/* ============================= Include files ============================= */
#include <mqdb.h>
#ifdef HAVE_XDR
#include <rpc/rpc.h>
#else
#include <types.h>
#include <xdr.h>
#endif

/* ========================== Program statements  ========================== */
void mqcd_( unsigned short *fd, double *data )
    {
    char *tmpfile;      /* contents of temporary header file */
    char  topline[80];  /* top of MQDB format file */
    unsigned long  i;   /* loop counter */
    XDR xdrout;
    MqHeader header;
    FILE *fout;

    MqSetFunc( MQCD );
    MqClearError();

    MqCheckFilDesc( *fd, MQ_WRITE );   /* this must be checked first !! */

    MqPutTopLine( topline );

    /* create temporary header file name */
    if( NULL == ( tmpfile = MqReadTmpHdrFile( *fd ))) {
	MqSetError( EMQREADTMP );
	MqPerror( NULL );
	}

    header.FileType = MqGetFileType( tmpfile );
    header.NumOfData = MqGetNumOfData( tmpfile );
    header.Channels = MqGetChannels( tmpfile );

    if( MqFilDesc[*fd].iostream == MQ_FILE ) { /* ordinary file output */
	if( NULL == ( fout = fopen( MqFilDesc[*fd].filename, "w" ))) {
	    MqSetError( EMQFILOPEN );
	    MqPerror( NULL );
	    }
	fprintf( fout, "%s", topline );
	fprintf( fout, "%s", tmpfile );
	fprintf( fout, "%s", MqSeparator );
	fprintf( fout, "%s", MqLineEnd );
	}
    else if( MqFilDesc[*fd].iostream == MQ_STDOUT ) { /* put to stdout */
	printf( "%s", topline );
	printf( "%s", tmpfile );
	printf( "%s", MqSeparator );
	printf( "%s", MqLineEnd );
	}

    /* output data part */
    switch( header.FileType ) {
    case MQ_FULLTEXT:
        if( MqFilDesc[*fd].iostream == MQ_STDOUT )
            MqPutData( stdout, data, &header );
        else
            MqPutData( fout, data, &header );
        break;
    case MQ_COMPOSITE:
        if( MqFilDesc[*fd].iostream == MQ_STDOUT )
            fwrite( data, sizeof( double ), header.NumOfData, stdout );
        else
            fwrite( data, sizeof( double ), header.NumOfData, fout );
        break;
    case MQ_XDR:
        if( MqFilDesc[*fd].iostream == MQ_STDOUT )
            xdrstdio_create( &xdrout, fout, XDR_ENCODE );
        else
            xdrstdio_create( &xdrout, stdout, XDR_ENCODE );

        for( i = 0; i < header.NumOfData; i++ ) {
            if( !xdr_double( &xdrout, data+i )) {
                MqSetError( EMQXDRWRITE );
                MqPerror( NULL );
                }
            }
        break;
    default:
        MqSetError( EMQILFILEFORM );
        MqPerror( NULL );
        }

    if( MqFilDesc[*fd].iostream == MQ_FILE ) fclose( fout );
    }

/* =========================== END of the program ========================== */

