/* ------------------- Moonquake database library routines -------------------+
|                                                                             |
|                              "MQOPEN" ver 1.00                              |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/* ============================= Include files ============================= */
#include <mqdb.h>

/* ========================== Program statements  ========================== */
void mqopen_( char *filename, short *rw, short *io, unsigned short *fildesc )
    {
    unsigned short    filenamelen, i, fd;   
    MQIO              inpflag = MQ_FILE;      /* stream IO type */
    char              newfile[FILENAME_MAX],  /* rearranged filename */
                     *header,                 /* header strings storage */
                     *date,                   /* header strings of "date" */
                     *tmpfilename;
    FILE             *fin,   /* file descriptor of input file */
                     *ftmp;  /* file descriptor of temporary header file */
    MqLibInfo         libinfo;
    unsigned int      headerlen, j;

    MqSetFunc( MQOPEN );
    MqClearError();

    /* eliminate white space in original filename variable */
    filenamelen = strlen( filename );
    for( i = 0; i < filenamelen; i++ )
	( filename[i] == ' ' ) ? 
	    ( newfile[i] = '\0' ) : ( newfile[i] = filename[i] );

    /* assumes standard input if file name is only a space */
    if( strlen( newfile ) == 0 ) inpflag = MQ_STDIN;

    /* memory allocation for header string storage */
    header = calloc( MQHDRLENMAX, 1 );
    if( NULL == header ) {
	MqSetError( EMQMEMALLOC );
	MqPerror( NULL );
	}

    /* actual file input */
    fd = *fildesc = MqSetFilDesc( newfile, *rw, *io );

    /* MQRW set */
    if( *rw == 0 )         /* read */
	MqFilDesc[fd].ReadWrite = MQ_READ;
    else if( *rw == 1 )    /* write */
	MqFilDesc[fd].ReadWrite = MQ_WRITE;
    else {    /* INVALID !! */
	MqSetError( EMQILMQRW );
	MqPerror( NULL );
	}

    /* MQIO set */
    switch( *io ) {
    case 1:
	MqFilDesc[fd].iostream = MQ_FILE;
	break;
    case 2:
	MqFilDesc[fd].iostream = MQ_STDIN;
	break;
    case 3:
	MqFilDesc[fd].iostream = MQ_STDOUT;
	break;
    case 4:
	MqFilDesc[fd].iostream = MQ_STDERR;
	break;
    default:
	MqSetError( EMQILMQIO );
	MqPerror( NULL );
	}

    if( inpflag == MQ_STDIN ) MqFilDesc[fd].iostream = MQ_STDIN;

    /* open file for reading */
    if( MqFilDesc[fd].ReadWrite == MQ_READ ) {
	if( inpflag == MQ_FILE ) {
	    if( NULL == ( fin = fopen( newfile, "r" ))) {
		MqSetError( EMQFILOPEN );
		MqPerror( newfile );
		perror( newfile );
		}
	    libinfo = MqGetTopLine( fin );
	    header  = MqGetHeader( fin );
	    }
	else if( inpflag == MQ_STDIN ) {      /* if input is from stdin */
	    libinfo = MqGetTopLine( NULL );
	    header  = MqGetHeader( NULL );
	    }
	else {
	    MqSetError( EMQILMQIO );
	    MqPerror( NULL );
	    }

	tmpfilename = MqCreTmpHdrFileName( *fildesc );
	if( NULL == ( ftmp = fopen( tmpfilename, "w" ))) {
	    MqSetError( EMQOPENTMP );
	    MqPerror( NULL );
	    }
	headerlen = strlen( header );

	for( j = 0; j < headerlen; j++ ) {
	    fputc( header[j], ftmp );
	    }
    
	fclose( ftmp );
	if ( inpflag == MQ_FILE ) fclose( fin );
	}

	else if( MqFilDesc[fd].ReadWrite == MQ_WRITE ) {
	if( NULL == ( date = MqPutCreateDate() )) {
	    MqSetError( EMQCREATEHDR );
	    MqPerror( (char *)MqCreDate );
	    }
	else {
	    tmpfilename = MqCreTmpHdrFileName( *fildesc );
	    if( NULL == ( ftmp = fopen( tmpfilename, "w" ))) {
		MqSetError( EMQOPENTMP );
		MqPerror( NULL );
		}
	    fputs( date, ftmp );
	    fclose( ftmp );
	    }
	}
	else {
	    MqSetError( EMQILMQRW );
	    MqPerror( NULL );
	    }
    free( header );
    }

/* =========================== END of the program ========================== */

