/* ------------------- Moonquake database library routines -------------------+
|                                                                             |
|                               "MQRD" ver 1.00                               |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1995 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/* ============================= Include files ============================= */
#include <mqdb.h>
#ifdef HAVE_XDR
#include <rpc/rpc.h>
#else
#include <types.h>
#include <xdr.h>
#endif

/* ========================== Program statements  ========================== */
void mqrd_( unsigned short *fd, unsigned short *channels, unsigned long *nod,
	   unsigned short *filetype, double *data )
    {
    FILE *fin;         /* input data file */
    MqLibInfo info;    /* MQDB information described in the top line */
    MqHeader  header;  /* Header information ( temporary storage ) */
    int  skip1 = ' ', skip2 = ' '; /* header strings skipped when reading */
    XDR  xdrin;        /* XDR data structure */
    unsigned int i,                /* loop counter */
                 readnum;          /* number of read data by fread() */

    MqSetFunc( MQRD );
    MqClearError();

    MqCheckFilDesc( *fd, MQ_READ );

    /* checks illegal input values */
    if( *nod == 0 ) {      /* if number of data is zero */
	MqSetError( EMQILNOD );
	MqPerror( NULL );
	}
    if( *channels == 0  ) {
	MqSetError( EMQZEROCHANNEL );
	MqPerror( NULL );
	}

    /* open input file */
    if( NULL == ( fin = fopen( MqFilDesc[*fd].filename, "r" ))) {
	MqSetError( EMQFILOPEN );
	MqPerror( MqFilDesc[*fd].filename );
	}

    info = MqGetTopLine( fin );    /* read top line */

    while( 1 ) {  /* reads out header strings when finds '@@' */
	skip1 = getc( fin );
	skip2 = getc( fin );
	if( skip1 == '@' && skip2 == '@' ) break;

	ungetc( skip2, fin );
	}
    skip1 = getc( fin );    /* reads out 'CR' */
    skip2 = getc( fin );    /* reads out 'LF' */

    switch( *filetype ) {
    case 0:    /* unknown */
	header.FileType = MQ_UNKNOWN;
	break;
    case 1:    /* TEXT format */
	header.FileType = MQ_FULLTEXT;
	break;
    case 2:    /* BINARY format */
	header.FileType = MQ_COMPOSITE;
	break;
    case 3:    /* XDR format */
	header.FileType = MQ_XDR;
	break;
    default:   /* none of the above */
        MqSetError( EMQILFILETYPE );
        MqPerror( NULL );
	}

    switch( header.FileType ) {
    case MQ_FULLTEXT:
	/* sets header data for using MqGetData() */
	header.Channels  = *channels;
	header.NumOfData = *nod;
	if( NULL != fin )
	    MqGetData( fin, data, &header );
	else
	    MqGetData( stdin, data, &header );

	break;
    case MQ_COMPOSITE:
	if( NULL != fin )
	    fread( data, sizeof( double ), *nod, fin );
	else {
	    for( i = 0; i < *nod; i++ ) 
		readnum = fread( data+i, sizeof( double ), 1, stdin );
	    }
	break;
    case MQ_XDR:
	if( NULL != fin )
	    xdrstdio_create( &xdrin, fin, XDR_DECODE );
	else
	    xdrstdio_create( &xdrin, stdin, XDR_DECODE );

	for( i = 0; i < *nod; i++ ) {
	    if( !xdr_double( &xdrin, data+i )) {
		MqSetError( EMQXDRREAD );
		MqPerror( NULL );
		}
	    }
	break;
    default:
        MqSetError( EMQILFILETYPE );
        MqPerror( NULL );
	}
    fclose( fin );
    }

/* =========================== END of the program ========================== */
