/* ----------- Moonquake database library routines ( header file )------------+
|                                                                             |
|                             "mqerror.h" ver 1.00                            |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1995 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/*
 *  This is a template file for "mqerror.h", which defines error numbers,
 *  function numbers and error level numbers. These definition and error 
 *  messages are automatically generated from list file "mqerror.list" by a
 *  command "mqgenerr", processed by macro preprocessor "m4". "mqerror.h" is 
 *  automatically included in "mqdb.h", so you do not need to 
 *  individually. You only need to  "mqdb.h".
 */

/* =========================== Macro definitions  ========================== */

#ifndef _MQERROR_H
#define _MQERROR_H

/*
 *  Error numbers.
 *  This part is automatically generated by mqgenerr and processed by
 *  "m4".
 */

#define EMQNOERROR 255
#define EMQONEDATA 256
#define EMQILERRNUM 257
#define EMQILERRLEVEL 258
#define EMQILLIBTYPE 259
#define EMQILLIBREL 260
#define EMQILLIBMAJ 261
#define EMQILLIBMIN 262
#define EMQEXISTHEADER 263
#define EMQSMALLYEAR 264
#define EMQLARGEYEAR 265
#define EMQLARGEDAY 266
#define EMQLARGEHOUR 267
#define EMQLARGEMINUTE 268
#define EMQLARGESECOND 269
#define EMQLARGEMS 270
#define EMQZEROHDR 271
#define EMQZEROINFO 272
#define EMQZERODATA 273
#define EMQZEROHDRLINE 274
#define EMQZEROAMPCHNL 275
#define EMQLONGINFO 276
#define EMQMULTINFO 277
#define EMQNOSEMICOLON 278
#define EMQNOCOMMA 279
#define EMQNOMQTIMESEP 280
#define EMQILMQTIMESEP 281
#define EMQILPOS 282
#define EMQILMQDBFORM 283
#define EMQILFILEFORM 284
#define EMQLONGSTATION 285
#define EMQILSTATION 286
#define EMQILDATAFORM 287
#define EMQILDATATYPE 288
#define EMQILFILETYPE 289
#define EMQILOBSMODE 290
#define EMQNODATASEP 291
#define EMQLONGHDR 292
#define EMQNOHEADER 293
#define EMQNONUM 294
#define EMQILMQRW 295
#define EMQILMQIO 296
#define EMQZEROFILE 297
#define EMQNULLPTR 298
#define EMQFILOPEN 299
#define EMQMEMALLOC 300
#define EMQSTRCPY 301
#define EMQSTRCAT 302
#define EMQILSTNNUM 303
#define EMQSHORTTOP 304
#define EMQNOTLSEP 305
#define EMQZEROPSGN 306
#define EMQLONGPSGN 307
#define EMQZEROCHANNEL 308
#define EMQZERONOD 309
#define EMQILNOD 310
#define EMQSHORTFILE 311
#define EMQOPENTMP 312
#define EMQREADTMP 313
#define EMQWRITETMP 314
#define EMQREMOVETMP 315
#define EMQMQIO 316
#define EMQCATINFODATA 317
#define EMQSTAT 318
#define EMQCREATEHDR 319
#define EMQFILDESCMAX 320
#define EMQFDNOTINUSE 321
#define EMQFDREAD 322
#define EMQFDWRITE 323
#define EMQILRW 324
#define EMQXDRWRITE 325
#define EMQXDRREAD 326
#define EMQERROR 327

#define MQERRBASE 255
#define MQNTCBASE 255
#define MQWNGBASE 257
#define MQFTLBASE 298


/* 
 *  Error levels.
 *   EMQ_NOTICE:  Trifle error. Returned value will be ensured.
 *   EMQ_WARNING: Harmful but recoverable. Errors will be automatically
 *                corrected. Returned value will not be ensured.
 *   EMQ_FATAL:   Severe error. Program continuation disabled.
 *   EMQ_NOMESSAGE: No messages are output even if FATAL error occured.
 */

#define EMQ_NOTICE     1
#define EMQ_WARNING    2
#define EMQ_FATAL      3
#define EMQ_NOMESSAGE  4
#define EMQ_LEVELMAX   4

/* ================= Static / extern  Variables definitions ================ */

struct {
    unsigned long    errno;  /* error number */
    unsigned long    func;   /* function that an error occured */
    unsigned short   level;  /* error level ( see above ) */
    unsigned short   warn;   /* warning level */
    }  MqErr;                /* error-monitoring variable */

    /*
     * Error messages.
     * This part is automatically generated by mqgenerr and processed by
     * "m4".
     */
static const char *MqErrorMsg[] = { 
"Program exited with no error",
"No data separator, not need to call this function",
"Illegal error number",
"Illegal error level",
"Illegal library type detected",
"Illegal library release number detected",
"Illegal library major number detected",
"Illegal library minor number detected",
"Header strings already exists",
"Too small year value in MqTime",
"Too large year value in MqTime",
"Too large day value in MqTime",
"Too large hour value in MqTime",
"Too large minute value in MqTime",
"Too large second value in MqTime",
"Too large microsecond value in MqTime",
"Length of the header string is zero",
"Length of the information string is zero",
"Length of the data string is zero",
"Length of the header line is zero",
"Number of channels in MqAmp are zero",
"Information string is longer than header",
"Multiple header strings detected",
"No separator character \':\' found",
"No separator character , found",
"No separator character \' \' found",
"Illegal separator found in Start_time header",
"Illegal position",
"Assigned file may not be a MQDB format",
"File Format is not specified or unknown",
"Data length of \"Station\" header is too long",
"Illegal station header found",
"Illegal data format name",
"Illegal data type",
"Illegal file type",
"Illegal observation mode",
"Illegal character(s) found in \"Station\" header",
"Header/Data separator not found",
"No such header found",
"Non-numerical character(s) found in header",
"Invalid value in MQRW",
"Invalid value in MQIO",
"Length of file name is zero",
"Null ( maybe uninitialized ) pointer detected",
"Failed to open a file",
"Memory allocation error",
"strcpy() error",
"strcat() error",
"Illegal station number",
"Top line is too short, may be has no program signature",
"Top line separator not found",
"Length of the string \"ProgSign\" is zero",
"Length of the string \"ProgSign\" too long",
"The number of channels is zero",
"The number of data is zero",
"The number of data disagrees with channels",
"File length is only zero or one byte",
"Failed to open temporary header file",
"Failed to read temporary header file",
"Failed to write temporary header file",
"Failed to remove temporary header file",
"Invalid MQIO value",
"MqCatInfoData() failed",
"stat() failed",
"Create of header failed",
"Number of File descriptor exceeds limit",
"Specified file descriptor is not in use",
"File descriptor is allocated as read mode",
"File descriptor is allocated as write mode",
"Illegal file read/write mode",
"Failed to write file in XDR conversion",
"Failed to read file in XDR conversion",
"Error ( unknown reason )"
};

    /* error level specification */
static const char *MqErrorLevel[] =
    { "NOERROR", "NOTICE", "WARNING", "FATAL" };

/* ====================== Macro function definitions  ====================== */
/*
 *  Macro function definitions.
 *   MqClearError() ... Clears error number of MqErr.
 *   MqSetError()   ... Sets error number for MqErr.
 *   MqSetFunc()    ... Sets function number for MqErr.
 */

#define MqClearError()      MqErr.errno = 255
#define MqSetError(x)       MqErr.errno = x
#define MqSetFunc(x)        MqErr.func = x

#endif      /* _MQERROR_H */

/* =========================== END of the program ========================== */
