/* ----------- Moonquake database library routines ( header file )------------+
|                                                                             |
|                              "mqfunc.h" ver 1.00                            |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/*
 *  This include file defines function numbers and function names. This file
 *  is automatically included in "mqdb.h", so you do not need to include this
 *  file individually. You only need to include "mqdb.h".
 */

/* =========================== Macro definitions  ========================== */

#ifndef _MQFUNC_H
#define _MQFUNC_H

/*
 *  Number of the function. These REAL values will be subject to change
 *  without notice. So you must use defined macro name as a function number.
 *  Defined macro names are only capitalised version of each function name.
 */

#define MQLIB                 0

     /* Upper level interface functions */
#define MQCREATE              1
#define MQINIT                2
#define MQREAD                3
#define MQREADDATA            4
#define MQREADHEADER          5

    /* middle level interface function */
#define MQCLOSE               6
#define MQCREFILENAME         7
#define MQGETAVEAMP           8
#define MQGETCHANNELS         9
#define MQGETCREATEDATE      10
#define MQGETDATA            11
#define MQGETDATADRIFT       12
#define MQGETDATAFORMAT      13
#define MQGETDATATYPE        14
#define MQGETENDRECORD       15
#define MQGETFILETYPE        16
#define MQGETHEADER          17
#define MQGETINITMOTION      18
#define MQGETMAXAMP          19
#define MQGETNUMOFDATA       20
#define MQGETOBSMODE         21
#define MQGETORGFILE         22
#define MQGETRISETIME        23
#define MQGETSAMPRATE        24
#define MQGETSTARTRECORD     25
#define MQGETSTARTTIME       26
#define MQGETSTATION         27
#define MQGETTAPENUMBER      28
#define MQGETTOPLINE         29
#define MQMODDATE            30
#define MQOPEN               31
#define MQPUTAVEAMP          32
#define MQPUTCHANNELS        33
#define MQPUTCREATEDATE      34
#define MQPUTDATA            35
#define MQPUTDATADRIFT       36
#define MQPUTDATAFORMAT      37
#define MQPUTDATATYPE        38
#define MQPUTENDRECORD       39
#define MQPUTFILETYPE        40
#define MQPUTINITMOTION      41
#define MQPUTMAXAMP          42
#define MQPUTNUMOFDATA       43
#define MQPUTORGFILE         44
#define MQPUTRISETIME        45
#define MQPUTSAMPRATE        46
#define MQPUTSTARTRECORD     47
#define MQPUTSTARTTIME       48
#define MQPUTSTATION         49
#define MQPUTTAPENUMBER      50
#define MQPUTTOPLINE         51

    /* lower level interface functions */
#define MQCATINFODATA        52
#define MQCOUNTSEPARATOR     53
#define MQCONVCHARDOUBLE     54
#define MQCUTLINE            55
#define MQREADTMPHDRFILE     56
#define MQSEARCHHEADER       57
#define MQSEPARATEINFODATA   58
#define MQSEPARATEMQTIME     59
#define MQSETFILDESC         60

    /* utility functions */
#define MQCALCAVEAMP         61
#define MQCALCMAXAMP         62
#define MQCHECKFILDESC       63
#define MQCHECKLIBVER        64
#define MQCHECKOBSMODE       65
#define MQCHECKSAMPRATE      66
#define MQCHECKSTATION       67
#define MQCHECKTIME          68
#define MQCRETMPHDRFILENAME  69
#define MQDIFFTIME           70
#define MQPERROR             71
#define MQSETACCESSNO        72
#define MQSETACCESSYES       73
#define MQSETWARNLEVEL       74

    /* FORTRAN subroutines */
#define MQCLOS               75
#define MQCAA                76  /* Write Average Amplitude */
#define MQCCH                77  /* Write Channels */
#define MQCD                 78  /* Write Data */
#define MQCDCD               79  /* Write Data Create Date */
#define MQCDF                80  /* Write Data Format */
#define MQCDMD               81  /* Write Data Modified Date */
#define MQCDT                82  /* Write Data Type */
#define MQCERC               83  /* Write End Record */
#define MQCFT                84  /* Write File Type */
#define MQCMA                85  /* Write Maximum Amplitude */
#define MQCNOD               86  /* Write Number of Data */
#define MQCOM                87  /* Write Observation Mode */
#define MQCOFN               88  /* Write Original Data File Name */
#define MQCSR                89  /* Write Sampling Rate */
#define MQCSTN               90  /* Write Station */
#define MQCSRC               91  /* Write Start Record */
#define MQCST                92  /* Write Start Time */
#define MQCTN                93  /* Write Original Tape Sequence Number */
#define MQRAA                94  /* Read Average Amplitude */
#define MQRCH                95  /* Read Channels */
#define MQRD                 96  /* Read Data */
#define MQRDCD               97  /* Read Data Create Date */
#define MQRDF                98  /* Read Data Format */
#define MQRDMD               99  /* Read Data Modified Date */
#define MQRDT               100  /* Read Data Type */
#define MQRERC              101  /* Read End Record */
#define MQRFT               102  /* Read File Type */
#define MQRMA               103  /* Read Maximum Amplitude */
#define MQRNOD              104  /* Read Number of Data */
#define MQROM               105  /* Read Observation Mode */
#define MQROFN              106  /* Read Original Data File Name */
#define MQRSR               107  /* Read Sampling Rate */
#define MQRSTN              108  /* Read Station */
#define MQRSRC              109  /* Read Start Record */
#define MQRST               110  /* Read Start Time */
#define MQRTN               111  /* Read Original Tape Sequence Number */

    /* place this definition last */
#define MQFUNCEND           112

/* ================= Static / extern  Variables definitions ================ */

/* 
 *  Function names. This variable is correlated with the error number defini-
 *  tions above. If you modify this definition, you must also change this.
 */

static const char *MqFuncName[] = {
    "MQLIB",    /* general referrings */

/* upper level interface functions */
    "MqCreate", "MqInit", "MqRead", "MqReadData", "MqReadHeader",

/* middle level interface functions */
    "MqClose",
    "MqCreFileName",
    "MqGetAveAmp",
    "MqGetChannels",
    "MqGetCreateDate",
    "MqGetData",
    "MqGetDataDrift",
    "MqGetDataFormat",
    "MqGetDataType",
    "MqGetEndRecord",
    "MqGetFileType",
    "MqGetHeader",
    "MqGetInitMotion",
    "MqGetMaxAmp",
    "MqGetNumOfData",
    "MqGetObsMode",
    "MqGetOrgFile",
    "MqGetRiseTime",
    "MqGetSampRate",
    "MqGetStartRecord",
    "MqGetStartTime",
    "MqGetStation",
    "MqGetTapeNumber",
    "MqGetTopLine",
    "MqModDate",
    "MqOpen",
    "MqPutAveAmp",
    "MqPutChannels",
    "MqPutCreateDate",
    "MqPutData",
    "MqPutDataDrift",
    "MqPutDataFormat",
    "MqPutDataType",
    "MqPutEndRecord",
    "MqPutFileType",
    "MqPutInitMotion",
    "MqPutMaxAmp",
    "MqPutNumOfData",
    "MqPutOrgFile",
    "MqPutRiseTime",
    "MqPutSampRate",
    "MqPutStartRecord",
    "MqPutStartTime",
    "MqPutStation",
    "MqPutTapeNumber",
    "MqPutTopLine",

/* lower level interface functions */
    "MqCatInfoData", 
    "MqCountSeparator",
    "MqConvCharDouble",
    "MqCutLine",
    "MqCreTmpFileName",
    "MqSearchHeader",
    "MqSeparateInfoData",
    "MqSeparateMqTime",
    "MqSetFilDesc",

/* utility functions */
    "MqCalcAveAmp",
    "MqCalcMaxAmp",
    "MqCheckFilDesc",
    "MqCheckLibVer",
    "MqCheckObsMode",
    "MqCheckSampRate",
    "MqCheckStation",
    "MqCheckTime",
    "MqCreFileName",
    "MqCreTmpHdrFileName",
    "MqDiffTime"
    "MqPerror",
    "MqSetAccessNo",
    "MqSetAccessYes",
    "MqSetWarnLevel",

/* FORTRAN subroutines */
    "MQCLOS",
    "MQCAA",
    "MQCCH",
    "MQCD",	
    "MQCDCD",
    "MQCDF",
    "MQCDMD",
    "MQCDT",
    "MQCERC",
    "MQCFT",
    "MQCMA",
    "MQCNOD",
    "MQCOM",
    "MQCOFN",
    "MQCSR",
    "MQCSTN",
    "MQCSRC",
    "MQCST",
    "MQCTN",
    "MQRAA",
    "MQRCH",
    "MQRD",
    "MQRDCD",
    "MQRDF",
    "MQRDMD",
    "MQRDT",
    "MQRERC",
    "MQRFT",
    "MQRMA",
    "MQRNOD",
    "MQROM",
    "MQROFN",
    "MQRSR",
    "MQRSTN",
    "MQRSRC",
    "MQRST",
    "MQRTN"
};

#endif      /* _MQFUNC_H */

/* =========================== END of the program ========================== */
