/* ------------------- Moonquake database utility programs -------------------+
|                                                                             |
|                             "mqgenerr" ver 1.00                             |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|           Copyright (c) 1995 Jun-ya Terazono, all rights reserved.          |
+----------------------------------------------------------------------------*/

/*
 *  This utility program creates error message file and error macro definition
 *  file, named "errmsg" and "errdef", respectively, from definition file
 *  "mqerror.list". Created files are used to generate header file "mqerror.h"
 *  by macro-handling command "m4".
 */

/* ============================= Include files ============================= */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* ===================== Static variable declarations ====================== */
static char infile[] = "mqerror.list";  /* error list file */
static char msgfile[] = "errmsg";       /* error message file */
static char deffile[] = "errdef";       /* macro definition file */

/* ========================= Function declarations ========================= */
char * CutMessage( char * ); 
void OutputDefines( char * );

/* ========================== Program statements  ========================== */
main()
    {
    char buf[BUFSIZ],
         defines[BUFSIZ],   /* macro definition strings */
         level[BUFSIZ],     /* warning level definition strings */
         *message;          /* error message strings */
    FILE *fin,    /* error list */
         *fdef,   /* error macro definition file */
         *fmsg;   /* error message definition file */
    unsigned short i = 255,
                   buflen,   /* length of input buffer string */
                   warning,
                   fatal,
                   detect = 0;

    if( NULL == ( fin = fopen( infile, "r" ))) {
	perror( infile );
	exit( 1 );
	}

    if( NULL == ( fdef = fopen( deffile, "w" ))) {
	perror( deffile );
	exit( 1 );
	}

    if( NULL == ( fmsg = fopen( msgfile, "w" ))) {
	perror( msgfile );
	exit( 1 );
	}

    fprintf( fmsg, "static const char *MqErrorMsg[] = { \n" );

    while( NULL != fgets( buf, BUFSIZ, fin )) {
	if( buf[0] == 'E' ) {
	    buflen = strlen( buf );
	    buf[buflen-1] = '\0';
	    sscanf( buf, "%s%s", defines, level );
	    if( ( 0 == strcmp( "WARNING", level )) && detect < 1 ) {
		detect = 1;
		warning = i;
		}
	    else if( ( 0 == strcmp( "FATAL", level )) && detect < 2 ) {
		detect = 2;
		fatal = i;
		}
	    if( NULL == ( message = CutMessage( buf )))
	       exit( 2 );
	    fprintf( fdef, "#define %s %d\n", defines, i );
	    fprintf( fmsg, "%s", message );
	    if( 0 != strcmp( "EMQERROR", defines ))
		fprintf( fmsg, ",\n" );
	    else
		fprintf( fmsg, "\n" );

	    i++;
	    }
	}

    fprintf( fmsg, "};" );

    fprintf( fdef, "\n" );
    fprintf( fdef, "#define MQERRBASE 255\n" );
    fprintf( fdef, "#define MQNTCBASE 255\n" );
    fprintf( fdef, "#define MQWNGBASE %d\n", warning );
    fprintf( fdef, "#define MQFTLBASE %d\n", fatal );

    fclose( fin );
    fclose( fdef );
    fclose( fmsg );

    return 0;
    }

/* ===== Cut messages from one input line ===== */
char * CutMessage( char *buf )
    {
    char *message;
    
    if( NULL != ( message = strchr( buf, '"' )))  
	return message;
    else
	return NULL;
    }

/* =========================== END of the program ========================== */

