/* ----------------------- Moonquake database utilities ----------------------+
|                                                                             |
|                              "dbconv" ver 1.00                              |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/
/*
 *  This program converts raw Moonquake data to Moonquake database format
 *  ( text file ).
 */

/* ============================= Include files ============================= */
#include <ctype.h>
#include <limits.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "mqdb.h"       /* Moonquake database header file */
#include "dbconv.h"     /* header file of this program */

/* ========================== Program statements  ========================== */

main( int argc, char **argv )
    {
    MqInit( dbsign );

    optflag.debug = optflag.endrec = optflag.file = optflag.startrec = FALSE;
    optflag.lp = optflag.sp = optflag.tidal = optflag.table = TRUE;
    optflag.verbose = SILENT;

    SetOptionFlag( argc, argv );
    GetTime( PROG_START );
    GetFileSize();         /* stat() is used */
    CheckParams();

    if( optflag.verbose >= VERBOSE ) DisplayParams();
    MakeTmpFileName();
    ReadRawFile();    /* Read record from raw file */
    RearrangeOutput();

    signal( SIGKILL, SignalHandling );
    signal( SIGINT, SignalHandling );
    signal( SIGTERM, SignalHandling );
    CreateMqFile();

    GetTime( PROG_END );
    if( optflag.debug == FALSE ) RemoveTmpFiles();
    }

/* ==== Evaluate switching characters from command line ===== */
void EvalSwitchChar( int optchar )

/*
 *  This function evaluates switching characters from command line and set
 *  parameters.
 *    optchar: switching character ( IN )
 */

    {
    switch( optchar )    {
    case 'd':    /* debug mode */
	optflag.debug = TRUE;
	break;
    case 't':    /* tidal output only */
	if( optflag.tidal == TRUE && optflag.lp == TRUE &&
	   optflag.sp == TRUE )    {
	    optflag.lp = optflag.sp = FALSE;
	    optflag.tidal = TRUE;
	    }
	break;
    case 's':   /* SP output only */
	if( optflag.tidal == TRUE && optflag.lp == TRUE &&
	   optflag.sp == TRUE )    {
	    optflag.lp = optflag.tidal = FALSE;
	    optflag.sp = TRUE;
	    }
	break;
    case 'l':   /* LP output only */
	if( optflag.tidal == TRUE && optflag.lp == TRUE &&
	   optflag.sp == TRUE )    {
	    optflag.sp = optflag.tidal = FALSE;
	    optflag.lp = TRUE;
	    }
	break;
    case 'n':   /* no file output */
	optflag.lp = optflag.sp = optflag.tidal = optflag.table = FALSE;
	break;
    case 'v':   /* verbose mode */
	optflag.verbose = VERBOSE;
	break;
    case 'V':   /* very-verbose mode */
	optflag.verbose = VERYVERBOSE;
	break;
    default:
	DisplayError( EDBILOPT, NULL );
	}
    }

/* ===== Check assigned parameters and exit if it is invalid ===== */
void CheckParams( void )
    {
    if( params.startrec == 0 ) params.startrec = 1;
    if( params.endrec == 0 )   params.endrec = params.records;

    if( params.startrec < 0 )
	DisplayError( EDBMINUSREC, NULL );
    if( params.endrec < 0 )
	DisplayError( EDBMINUSREC, NULL );
    if( params.endrec > RECORDMAX )
	DisplayError( EDBEXMAXREC, NULL );
    if( params.endrec > params.records )
	DisplayError( EDBLARGEREC, NULL );
    if( params.startrec > params.endrec )
	DisplayError( EDBSTARTEND, NULL );
    }

/* ===== Get file size using NON-ANSI function stat() ===== */
int GetFileSize( void )
    {
    struct stat   infilestat;     /* status of input file */

    stat( params.infile, &infilestat );
    params.records = infilestat.st_size / RECORDSIZE;

    if( infilestat.st_size % RECORDSIZE != 0 )
	DisplayError( EDBILFORM, params.infile );

    return infilestat.st_size;
    }

/* ===== create temporary file name by using start time ===== */
void MakeTmpFileName( void )

/* 
 *  This function creates a name of temporary file name using start time.
 *  This function creates length of eight characters strings and stores this 
 *  to params.tmp*. Base name of the temporary file is a form of xxxxx000, 
 *  where xxxxx denotes alphabetical characters. First character represents
 *  month, second date, third hour, fourth minute and fifth second. Each 
 *  number is converted to alphanumeric characters in the order shown in 
 *  string variable "timestr". For example, month "12" is converted to one
 *  character 'C', minute 56 converted to 'u'. 
 *  If environmental variable "DBWORKDIR" is set, all temporary files are 
 *  made in this directory.
 */

    {
    char tmpbase[9];    /* basename of temporary files */
    char *dbworkdir;    /* environmental variable DBWORKDIR */

    /* notice: tm_mon ranges from 0 to 11, not 1 to 12. */
    tmpbase[0] = timestr[params.progtime[0]->tm_mon + 1];
    tmpbase[1] = timestr[params.progtime[0]->tm_mday];
    tmpbase[2] = timestr[params.progtime[0]->tm_hour];
    tmpbase[3] = timestr[params.progtime[0]->tm_min];
    /* notice: tm_sec ranges from 0 to 61, not to 60 */
    tmpbase[4] = timestr[params.progtime[0]->tm_sec];
    tmpbase[5] = tmpbase[6] = tmpbase[7] = '0';
    tmpbase[8] = '\0';


    if( NULL != ( dbworkdir = getenv( "DBWORKDIR" )))    {
	if( optflag.verbose == VERYVERBOSE )
	    printf( "Working directory is %s\n", dbworkdir );
	strcat( dbworkdir, "/" );
	strcpy( params.tmpdtt, dbworkdir );
	strcpy( params.tmpdrt, dbworkdir );
	strcpy( params.tmplp,  dbworkdir );
	strcpy( params.tmpsp,  dbworkdir );
	strcpy( params.tmptidal, dbworkdir );
	}
    else    {
	strcpy( params.tmpdtt, tmpbase );
	strcpy( params.tmpdrt, tmpbase );
	strcpy( params.tmplp, tmpbase );
	strcpy( params.tmpsp, tmpbase );
	strcpy( params.tmptidal, tmpbase );
	}

    if( optflag.verbose == VERYVERBOSE )
	printf( "temporary file name is %s\n\n", tmpbase );

    strcat( params.tmplp, fileext[LP] );
    strcat( params.tmpsp, fileext[SP] );
    strcat( params.tmptidal, fileext[TIDAL] );
    strcat( params.tmpdtt, fileext[DTT] );
    strcat( params.tmpdrt, fileext[DRT] );
    }

/* ===== set separate record descriptor ===== */
void SearchRecSeparator( int records, OUTPAR *outpar,
			   double *timediff, unsigned short *seprec )
    {
    int    i;           /* loop counter */

    seprec[0] = 1;
    seprec[1] = DATASTART;
    seprec[records] = DATAEND;

    for( i = 2; i < records; i++ )    {
	if( outpar[i].format == OLD )    {   /* OLD format */
	    if(( OLDRECTIME - GAPLIMIT ) < timediff[i] &&
	       ( OLDRECTIME + GAPLIMIT ) > timediff[i] )
		seprec[i] = CONT;
	    else if(( OLDRECTIME - RANGELIMIT ) < timediff[i] &&
	       ( OLDRECTIME + RANGELIMIT ) > timediff[i] )
		seprec[i] = GAP;
	    else
		seprec[i] = SEP;
	    }
	else    {    /* NEW format */
	    if(( NEWRECTIME - GAPLIMIT ) < timediff[i] &&
	       ( NEWRECTIME + GAPLIMIT ) > timediff[i] )
		seprec[i] = CONT;
	    else if(( NEWRECTIME - RANGELIMIT ) < timediff[i] &&
	       ( NEWRECTIME + RANGELIMIT ) > timediff[i] )
		seprec[i] = GAP;
	    else
		seprec[i] = SEP;
	    }
	}

    for( i = 1; i <= records; i++ )
	if( seprec[i] == SEP ) seprec[0]++;
    }

/* ===== Control function for creating Moonquake database file ===== */
void CreateMqFile( void )

/*
 *  This is a main function to control creating Moonquake database format 
 *  files. 
 */

    {
    FILE    *tmpdrt;
    int      events,       /* number of events specified in DRT file */
             count = 0;    /* counter used in while loop */
    double  *mqdata;       /* moonquake data */
    char     buf[BUFSIZ],  /* data read buffer */
             filename[FILENAME_MAX];
    MqHeader header;       /* structure for Database header */
    MqTime   mqtime;       /* moonquake time table */
    EVDESC   evdesc;       /* events descriptor */

    if( NULL == ( tmpdrt = fopen( params.tmpdrt, "r" )))
	DisplayError( EDBFOPEN, params.tmpdrt );

    while( feof( tmpdrt ) == 0 )    {
	if( NULL == fgets( buf, BUFSIZ, tmpdrt )) break;
	if( count == 0 ) 
	    events = atoi( buf );
	else    {
	    SetEventDesc( buf, &evdesc );

	    ReadTmpDTT( filename, &mqtime, &evdesc );
	    if( optflag.verbose == VERYVERBOSE )    {
		printf( "Output Record: %d - %d, ",
		       evdesc.start, evdesc.end );
		printf( "time: %d %d %d %d %d %d\n",
		       mqtime.year, mqtime.day, mqtime.hour, mqtime.minute,
		       mqtime.second, mqtime.ms );
		}

	    if( optflag.lp == TRUE )    {
		SetMqHeader( &header, &mqtime, MQ_LP, &evdesc, filename );
		if( NULL == ( mqdata = calloc( header.NumOfData,
					      sizeof( double ))))
		    DisplayError( EDBMEMALLOC, NULL );
		ReadTmpLP( mqdata, evdesc, header.DataFormat );
		OutputMqFile( &header, mqdata );
		free( mqdata );
		}
	    if( optflag.sp == TRUE )    {
		SetMqHeader( &header, &mqtime, MQ_SP, &evdesc, filename );
		if( NULL == ( mqdata = calloc( header.NumOfData,
					      sizeof( double ))))
		    DisplayError( EDBMEMALLOC, NULL );
		ReadTmpSP( mqdata, evdesc );
		OutputMqFile( &header, mqdata );
		free( mqdata );
		}
	    if( optflag.tidal == TRUE )    {
		SetMqHeader( &header, &mqtime, MQ_TIDAL, &evdesc, filename );
		if( NULL == ( mqdata = calloc( header.NumOfData,
					      sizeof( double ))))
		    DisplayError( EDBMEMALLOC, NULL );
		ReadTmpTIDAL( mqdata, evdesc, header.DataFormat );
		OutputMqFile( &header, mqdata );
		free( mqdata );
		}
	    }
	count++;
	}

    if( count - 1 != events ) DisplayError( EDBDIFFEV, NULL );
    fclose( tmpdrt );
    }

/* ===== Remove Temporary Data files ===== */
void RemoveTmpFiles( void )

/*
 *  This function deletes temporary data files made by this program.
 */

    {
    int   rmcheck = 0;    /* check parameters whether a file removed */

    if( optflag.table == TRUE )    {      /* removes table file */
	rmcheck = remove( params.tmpdtt );
	if( rmcheck != 0 ) DisplayError( EDBRMDTT, params.tmpdtt );
	rmcheck = remove( params.tmpdrt );
	if( rmcheck != 0 ) DisplayError( EDBRMDRT, params.tmpdtt );
	}

    if( optflag.lp == TRUE )    {         /* removes LP file */
	rmcheck = remove( params.tmplp );
	if( rmcheck != 0 ) DisplayError( EDBRMLP, params.tmplp );
	}

    if( optflag.sp == TRUE )    {         /* removes SP file */
	rmcheck = remove( params.tmpsp );
	if( rmcheck != 0 ) DisplayError( EDBRMSP, params.tmpsp );
	}

    if( optflag.tidal == TRUE )    {      /* removes tidal file */
	rmcheck = remove( params.tmptidal );
	if( rmcheck != 0 ) DisplayError( EDBRMTDL, params.tmptidal );
	}
    }

/* ===== Signal Handling ===== */
void SignalHandling( signal )
    {
    RemoveTmpFiles();

    if( signal == SIGINT )    {
	DisplayError( EDBSIGINT, NULL );
	}
    else if( signal == SIGKILL )    {
	DisplayError( EDBSIGKILL, NULL );
	}
    else if( signal == SIGTERM )    {
	DisplayError( EDBSIGTERM, NULL );
	}
    
    fprintf( stderr, "Temporary files are removed.\n\n" );
    GetTime( PROG_INTR );

    exit( signal );
    }

/* ===== Rearrange output file from file type ===== */
void RearrangeOutput( void )

/*
 *  This subroutine specifies file type and re-sets file outputs. For example,
 *  if input file is NEW format, this subroutine sets SP output as NO.
 */

    {
    switch( params.outpar.format )    {
    case MQ_OLD:
	break;
    case MQ_NEW:
	if( optflag.sp == TRUE ) optflag.sp = FALSE;
	fprintf( stderr,
		"File format is NEW. No SP channel output is made.\n" );
	break;
    default:
	DisplayError( EDBILTYPE, NULL );
	}
    }

/* =========================== END of the program ========================== */
