/* --------------- Moonquake database utilities ( header file ) --------------+
|                                                                             |
|                             "dbconv.h" ver 1.00                             |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/*
 *  This is a header file to be used for Moonquake data formatting program 
 *  "dbconv.h". This file is also useful for examining data structure of 
 *  Moonquake raw data file.
 */

/* ============================== Include files ============================ */
#include "dbmqraw.h"
#include "dberror.h"

/* =========================== Macro declarations ========================== */

    /* true and false */
#define TRUE      1
#define FALSE     0

    /* data format type ( old/new ) */
#define OLD       0  /* "old" format */
#define NEW       1  /* "new" format */

    /* data separation level */
#define CONT      0  /* data record is continuous */
#define DATASTART 1  /* start of the data */
#define GAP       2  /* data record has gap */      
#define SEP       3  /* data record is separated */
#define DATAEND   4  /* end of the data */

   /* data ( output file ) type */
#define LP      0    /* LP output */
#define SP      1    /* SP output */
#define TIDAL   2    /* tidal output */
#define DTT     3    /* data time table */
#define DRT     4    /* data record table */

    /* verbose mode */
#define SILENT       0   /* silent mode ( this is default ) */
#define VERBOSE      1   /* verbose mode ( -v option ) */
#define VERYVERBOSE  2   /* Very verbose mode ( -V option ) */

    /* time mode ( a mode which gets time ) */
#define PROG_START   0   /* get time when program started */
#define PROG_END     1   /* get time when program ended */
#define PROG_INTR    2   /* get time when program was interrupted */

    /* for non ANSI header file */
#ifndef FILENAME_MAX
#define FILENAME_MAX    255
#endif

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS    0
#endif

#ifndef EXIT_FAILURE
#define EXIT_FAILURE    1
#endif

/* ========================= Structure declarations ======================== */

    /* file reading pointer ( position of record/field/cell ) */
typedef struct {
    int   record;    /* current RECORD */
    int   field;     /* current FIELD */
    int   cell;      /* current CELL */
    }  RECPTR;

    /* data structure for TIDAL data output */
typedef struct {
    unsigned int    x;
    unsigned int    y;
    unsigned int    z;
    }   TIDALOUT;

    /* data for temporary data storage */
typedef struct {
    unsigned int    lpx[LP_DATA_NUM+1];    /* LP component ( x ) */
    unsigned int    lpy[LP_DATA_NUM+1];    /* LP component ( y ) */
    unsigned int    lpz[LP_DATA_NUM+1];    /* LP component ( z ) */
    unsigned int    spz[SP_DATA_NUM+1];    /* SP component ( z only ) */
    TIDALOUT        tidal;                 /* TIDAL data */
    }    TMPOUT_OLD;

typedef struct {
    unsigned int    lpx[LP_DATA_NUM+1];    /* LP component ( x ) */
    unsigned int    lpy[LP_DATA_NUM+1];    /* LP component ( y ) */
    unsigned int    lpz[LP_DATA_NUM+1];    /* LP component ( z ) */
    TIDALOUT        tidal;                 /* TIDAL data */
    }    TMPOUT_NEW;

typedef union  {
    TMPOUT_OLD    tmpout_old[FIELD_NUM_OLD];
    TMPOUT_NEW    tmpout_new[FIELD_NUM_NEW];
    }  TMPOUT;

    /* header parameters to be output to a file or display */
typedef struct {
    unsigned short  pse;        /* PSE data number */
    unsigned short  seqno;      /* original tape sequence No. */
    unsigned short  format;     /* data format ( OLD or NEW ) */
    unsigned short  station;    /* station number */
    }    OUTPAR;

    /* header for storing event start/end record and gap line */
typedef struct {
    unsigned short    start;
    unsigned short    end;
    unsigned short    comnum;
    unsigned short    com[RECORDMAX];
    }    EVDESC;

/* ========================= Variable settings ======================== */
    /* data format */
static char *format[] = { "OLD", "NEW" };

    /* verbose modes */
static char *verbose[] = { "SILENT", "VERBOSE", "VERYVERBOSE" };

    /* Yes/No selections */
static char *yesno[] = { "NO", "YES" };

    /* number of fields in each format */
static unsigned short fieldnum[] = { FIELD_NUM_OLD, FIELD_NUM_NEW };

    /* strings used to generate temporary work files */
static char timestr[] = 
    "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";

    /* file extensions */
static char *fileext[] = { ".lp", ".sp", ".tdl", ".dtt", ".drt" };

    /* time mode */
static char *timemodestr[] = { "started", "ended", "interrupted" };

    /* program signature */
static char dbsign[] = "dbconv-1.00";

    /* used for data table */
static char tableheader[] =
    "----- Record output data -------------------------------------------\n";
static char contents[] =
    " REC  PSE  SEQ  STN            date           FMT     Time Diff\n";
static char line[] =
    "--------------------------------------------------------------------\n";
static char diffline[] =
    "********************************************************************\n";
static char gapline[] = "*****\n";
static char recnote[] =
    "REC: Record No.\n";
static char psenote[] =
    "PSE: Apollo Passive Seismic Experiment\n";
static char seqnote[] =
    "SEQ: Original data type sequence No.\n";
static char stnnote[] =
    "STN: Apollo seismic station number.\n";
static char fmtnote[] =
    "FMT: Data format ( old/new )\n";
static char diffnote[] =
    "Diff Time: time difference from a previous record\n\n";
static char opform0[] =
    " %3d   %d  %5d  %2d  "
    " %d %3dd %02d:%02d %02d.%03d  %s  ------------- %d\n";
static char opform[]  =
    " %3d   %d  %5d  %2d  "
    " %d %3dd %02d:%02d %02d.%03d  %s  %13.3f %d\n";

    /* option assignments */
struct {
    int  debug;          /* debug mode */
    int  endrec;         /* argument number of end record ( or 0 ) */
    int  file;           /* argument number of file ( or 0 ) */
    int  lp;             /* LP output only ( -l ) */
    int  sp;             /* SP output only ( -s ) */
    int  startrec;       /* argument number of start record ( or 0 ) */
    int  table;          /* temporary data table */
    int  tidal;          /* tidal output only ( -t ) */
    int  verbose;        /* verbose mode ( see macro definitions ) */
    }    optflag;

    /* program parameters */
struct {
    char     infile[FILENAME_MAX];    /* input data file name */
    char     tmpdtt[FILENAME_MAX];    /* temporary file name ( time table ) */
    char     tmpdrt[FILENAME_MAX];    /* temporary file name ( rec. table ) */
    char     tmplp[FILENAME_MAX];     /* temporary file name ( LP ) */
    char     tmpsp[FILENAME_MAX];     /* temporary file name ( SP ) */
    char     tmptidal[FILENAME_MAX];  /* temporary file name ( tidal ) */
    int      records;                 /* number of records */
    int      startrec;                /* start record of reading */
    int      endrec;                  /* end record of reading */
    OUTPAR   outpar;                  /* header output parameters */
    time_t   time[3];
    struct tm  *progtime[3];          /* start/end/intr time */
    }  params;

/* ========================= Function declarations ========================= */
extern void CheckParams( void );

#ifdef NOLONGLONG
extern void ConvMqTime( unsigned long, unsigned long, MqTime *, int );
#else
extern void ConvMqTime( unsigned long long, MqTime *, int );
#endif

extern void CreateMqFile( void );
extern void DisplayError( int, char * );
extern void DisplayParams( void );
void DisplayRecordTable( OUTPAR *, MqTime *, double *, unsigned short * );
extern void DisplayUsage( void );
extern void EvalSwitchChar( int );
extern int  GetFileSize( void );
extern void GetTime( int );
extern void MakeTmpFileName( void );
extern void OutputDataTmpOld( TMPOUT, RECPTR );
extern void OutputDataTmpNew( TMPOUT, RECPTR );
extern void OutputMqFile( MqHeader *, double * );
extern void OutputTmpRecTable( unsigned short * );
extern void OutputTmpTimeTable( MqTime *, unsigned short * );
extern void RearrangeOutput( void );
extern void ReadFieldNew( RECPTR, FIELD_NEW, MqTime *, TMPOUT_NEW * );
extern void ReadFieldOld( RECPTR, FIELD_OLD, MqTime *, TMPOUT_OLD * );
extern void ReadRawFile( void );
extern void ReadRecord( FILE *, double *, OUTPAR *, MqTime * );
extern void ReadTmpDTT( char *, MqTime *, EVDESC * );
extern void ReadTmpLP( double *, EVDESC, MQDF );
extern void ReadTmpSP( double *, EVDESC );
extern void ReadTmpTIDAL( double *, EVDESC, MQDF );
extern void RearrangeMqData( MqHeader *, double * );
extern void RemoveTmpFiles( void );
extern void SearchRecSeparator( int, OUTPAR *, double *, unsigned short * );
extern void SetEventDesc( char *, EVDESC * );
extern void SetMqHeader( MqHeader *, MqTime *, MQDT, EVDESC *, char * );
extern void SetOptionFlag( int, char ** );
extern void SetParams( int, char ** );
extern void SignalHandling( int );

/* =========================== END of the program ========================== */
