/* ----------------------- Moonquake database utilities ----------------------+
|                                                                             |
|                             "dbdisp.c" ver 1.00                             |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/
/*
 *  This is a part of the source file of the program "dbread". This file is a
 *  set of funcions relating display outputs ( error message outputs, status
 *  displayings, etc. ).
 */

/* ============================= Include files ============================= */
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "mqdb.h"       /* Moonquake database header file */
#include "dbconv.h"     /* header file of this program */

/* ========================== Program statements  ========================== */

/* ===== Display error message and exit if necessary ===== */
void DisplayError( int errornum, char *errhdr )
    {
    if( errhdr != NULL ) perror( errhdr );
    else
    fprintf( stderr, "dbconv: %s.\n", erroroutput[errornum] );

    if( errornum < 16 )     {
	DisplayUsage();
	exit( errornum );
	}
    }

/* ===== Display usage ===== */
void DisplayUsage()
    {
    fprintf( stderr, "    usage: dbconv [ -tlsnvVd ]"
                     " file [ startrec ] [ endrec ]\n" );
    fprintf( stderr, "       -l: LP data only\n" );
    fprintf( stderr, "       -s: SP data only\n" );
    fprintf( stderr, "       -t: tidal data only\n" );
    fprintf( stderr, "       -n: no file output\n" );
    fprintf( stderr, "       -v: verbose mode\n" );
    fprintf( stderr, "       -V: Very verbose mode\n" );
    fprintf( stderr, "       -d: debug mode\n" );
    }

/* ===== Display header of the record ===== */
void DisplayRecordTable( OUTPAR *outpar, MqTime *mqtime,
		double *timediff, unsigned short *seprec )

/*
 *   outpar:   output parameters ( IN )
 *   mqtime:   time in microsecond ( IN )
 *   timediff: time difference in seconds ( IN )
 *   seprec:   separated record descriptor ( IN )
 */

    {
    int    i;    /* loop counter */

    printf( tableheader );
    printf( contents );
    printf( line );

    for( i = 0; i < params.records; i++ )    {
	if( i == 0 )
	    printf( opform0, 
		   i + 1, outpar[i].pse, outpar[i].seqno, outpar[i].station,
		   mqtime[i].year,   mqtime[i].day,    mqtime[i].hour,
		   mqtime[i].minute, mqtime[i].second, mqtime[i].ms,
		   format[outpar[i].format], timediff[i], seprec[i] );
	else    {
	    if( seprec[i] == SEP ) printf( diffline );
	    else if( seprec[i] == GAP ) printf( gapline );
	    printf( opform,
		   i + 1, outpar[i].pse, outpar[i].seqno, outpar[i].station,
		   mqtime[i].year,   mqtime[i].day,    mqtime[i].hour,
		   mqtime[i].minute, mqtime[i].second, mqtime[i].ms,
		   format[outpar[i].format], timediff[i], seprec[i] );
	    }
	}

    printf( line );
    printf( recnote );
    printf( psenote );
    printf( seqnote );
    printf( stnnote );
    printf( fmtnote );	
    printf( diffnote );
    }

/* ===== Display contents of parameters ===== */
void DisplayParams( void )
    {
    printf( "\n====== Parameter settings ======\n" );
    printf( "Verbose mode:    %s\n", verbose[optflag.verbose] );
    printf( "LP output:       %s\n", yesno[optflag.lp] );
    printf( "SP output:       %s\n", yesno[optflag.sp] );
    printf( "tidal output:    %s\n", yesno[optflag.tidal] );
    printf( "table output:    %s\n", yesno[optflag.table] );
    printf( "Input file name: %s\n", params.infile );
    printf( "Records in file: %d\n", params.records );
    printf( "Start record of reading:  %d\n", params.startrec );
    printf( "End record of reading:    %d\n", params.endrec );
    printf( "================================\n\n" );
    }

/* =========================== END of the program ========================== */
