/* --------------- Moonquake database utilities ( header file ) --------------+
|                                                                             |
|                            "dberror.h" ver 1.00                             |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/*
 *  This is a header file to be used for Moonquake data formatting program
 *  "dbconv". This is a file defining error numbers and messages.
 */

/* =========================== Macro declarations ========================== */
#define EDBARGS      1   /* no arguments */
#define EDBFOPEN     2   /* failed to open input file */
#define EDBILFORM    3   /* illegal data file format */
#define EDBMINUSONLY 4   /* only a minus character specified */
#define EDBILOPT     5   /* illegal option */
#define EDBNOFILE    6   /* no input file name specified */
#define EDBBOTHREQ   7
#define EDBILFORMNUM 8   /* illegal data format number detected */
#define EDBOTMPTBL   9   /* failed to open tmpfile ( data table ) */
#define EDBOTMPLP   10   /* failed to open tmpfile ( LP ) */
#define EDBOTMPSP   11   /* failed to open tmpfile ( SP ) */
#define EDBOTMPTDL  12   /* failed to open tmpfile ( tidal ) */
#define EDBMINUSREC 13   /* assigned record is less than 0 */
#define EDBEXMAXREC 14   /* read records exceeded maximum tolerable number */
#define EDBLARGEREC 15   /* assigned end record is more than that file has */
#define EDBSTARTEND 16   /* assigned start record is more than end record */
#define EDBMEMALLOC 17   /* failed in memory allocation */
#define EDBRMDTT    18   /* failed to remove temporary time table file */
#define EDBRMDRT    19   /* failed to remove temporary record table file */
#define EDBRMLP     20   /* failed to remove temporary LP data file */ 
#define EDBRMSP     21   /* failed to remove temporary SP data file */
#define EDBRMTDL    22   /* failed to remove temporary TIDAL data file */
#define EDBZERODRT  23   /* data record file has a zero length line */
#define EDBILDRT    24   /* illegal data record table format */
#define EDBCHARDRT  25   /* illegal character in data record table */
#define EDBDIFFEV   26   /* different event number in data record file */
#define EDBSIGKILL  27   /* SIGKILL detected */
#define EDBSIGINT   28   /* SIGINT detected */
#define EDBSIGTERM  29   /* SIGTERM detected */
#define EDBILTYPE   30   /* illegal data type */
#define EDBFILENAME 31   /* Cannot create MQDB filename */
#define EDBERROR    32   /* SET THIS DEFINITION LAST */

/* ========================= Variable settings ======================== */
static char *erroroutput[] = {
    "No error",                            /* EDBNOERROR */
    "No arguments",                        /* EDBARGS */
    "Failed in opening input file",        /* EDBFOPEN */
    "Illegal data file format",            /* EDBILFORM */
    "Only minus character specified",      /* EDBMINUSONLY */
    "Illegal option character specified",  /* EDBILOPT */
    "No input file name specified",        /* EDBNOFILE */
    "Both start record and end record is required",   /* EDBBOTHREQ */
    "Illegal data format number detected",            /* EDBILFORMNUM */
    "Failed in opening temporary output file ( data table )", /* EDBOTMPTBL */
    "Failed in opening temporary output file ( LP )",         /* EDBOTMPLP */
    "Failed in opening temporary output file ( SP )",         /* EDBOTMPSP */
    "Failed in opening temporary output file ( tidal )",      /* EDBOTMPTDL */
    "Assigned record is ( are ) less than 0",                 /* EDBMINUSREC */
    "Read record number exceeded maximum tolerable value",    /* EDBEXMAXREC */
    "Assigned end record is more than that file has",         /* EDBLARGEREC */
    "Assigned start record is more than assigned end record", /* EDBSTARTEND */
    "Failed in memory allocation",                     /* EDBMEMALLOC */
    "Failed to remove temporary time table file",      /* EDBRMDTT */
    "Failed to remove temporary data record file",     /* EDBRMDRT */
    "Failed to remove temporary LP data file",         /* EDBRMLP */
    "Failed to remove temporary SP data file",         /* EDBRMSP */
    "Failed to remove temporary TIDAL data file",      /* EDBRMTDL */
    "Data record file has a zero length line",         /* EDBZERODRT */
    "Illegal temporary data record format",            /* EDBILDRT */
    "Illegal character in data record file detected",  /* EDBCHARDRT */
    "Different number of event in data record file",   /* EDBDIFFEV */
    "SIGKILL detected ( killed from other terminal )", /* EDBSIGKILL */
    "SIGINT detected ( interrupted from console )",    /* EDBSIGINT */
    "SIGTERM detected ( interrupted by system )",      /* EDBSIGTERM */
    "Illegal data type",                               /* EDBILTYPE */
    "Cannot create MQDB filename",                     /* EDBFILENAME */
    "Other error ( reason unknown )"      /* EDBERROR */
    };

/* =========================== END of the program ========================== */
