/* ----------------------- Moonquake database utilities ----------------------+
|                                                                             |
|                            "dboutput.c" ver 1.01                            |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/*
 *  This file is a part of the "dbconv". Functions for outputting cooked data
 *  to files are gathered in this file.
 */

/*  History of modifications:
 *    1.01  (94.07.06)  A Bug in RearrangeMqData() was fixed.
 *    ( substraction of average value is executed for only three data )
 */   

/* ============================= Include files ============================= */
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "mqdb.h"       /* Moonquake database header file */
#include "dbconv.h"     /* header file of this program */

/* ========================== Program statements  ========================== */

/* ===== Output raw data to temporary files ( OLD format )===== */
void OutputDataTmpOld( TMPOUT tmpout, RECPTR recptr )
    {
    int    i, j;   /* loop counter */
    FILE  *lp, *sp, *tidal;
    TIDALOUT  tdlout;

        /* LP output */
    if( optflag.lp == TRUE )    {
	if( NULL == ( lp = fopen( params.tmplp, "a" )))
	    DisplayError( EDBOTMPLP, params.tmplp );

	for( i = 0; i < FIELD_NUM_OLD; i++ )   {
	    for( j = 1; j <= LP_DATA_NUM; j++ )
		fprintf( lp, "%d,%d,%d\n",
			tmpout.tmpout_old[i].lpx[j], 
			tmpout.tmpout_old[i].lpy[j],
			tmpout.tmpout_old[i].lpz[j] );
	    }
	fclose( lp );
	}

        /* SP output */
    if( optflag.sp == TRUE )    {
	if( NULL == ( sp = fopen( params.tmpsp, "a" )))
	    DisplayError( EDBFOPEN, params.tmpsp );

	for( i = 0; i < FIELD_NUM_OLD; i++ )    {
/*	    for( j = 1; j <= SP_DATA_NUM; j++ )
		fprintf( sp, "%d\n", tmpout.tmpout_old[i].spz[j] );*/
		fwrite( tmpout.tmpout_old[i].spz+1, sizeof(unsigned int),
		       SP_DATA_NUM, sp );
	    }
	fclose( sp );
	}

        /* TIDAL output */
    if( optflag.tidal == TRUE )     {
	if( NULL == ( tidal = fopen( params.tmptidal, "a" )))
	    DisplayError( EDBFOPEN, params.tmptidal );

	for( i = 0; i < FIELD_NUM_OLD; i++ )    {
	    if( i % 2 != 0 )    {
		tdlout.x = tmpout.tmpout_old[i].tidal.x;
		tdlout.y = tmpout.tmpout_old[i].tidal.y;
		fprintf( tidal, "%d,%d,%d\n", tdlout.x, tdlout.y, tdlout.z );
		}
	    else
		tdlout.z = tmpout.tmpout_old[i].tidal.z;
	    }

	fclose( tidal );
	}
    }

/* ===== Output raw data to temporary files ( NEW format )===== */
void OutputDataTmpNew( TMPOUT tmpout, RECPTR recptr )
    {
    int          i, j;   /* loop counter */
    FILE        *lp, *sp, *tidal;
    TIDALOUT     tdlout;

        /* LP output */
    if( optflag.lp == TRUE )    {
	if( NULL == ( lp = fopen( params.tmplp, "a" )))
	    DisplayError( EDBFOPEN, params.tmplp );

	for( i = 0; i < FIELD_NUM_NEW; i++ )   {
	    for( j = 1; j <= LP_DATA_NUM; j++ )
		fprintf( lp, "%d,%d,%d\n",
			tmpout.tmpout_new[i].lpx[j], 
			tmpout.tmpout_new[i].lpy[j],
			tmpout.tmpout_new[i].lpz[j] );
	    }
	fclose( lp );
	}

       /* TIDAL output */
    if( optflag.tidal == TRUE )    {
	if( NULL == ( tidal = fopen( params.tmptidal, "a" )))
	    DisplayError( EDBFOPEN, params.tmptidal );

	for( i = 0; i < FIELD_NUM_NEW; i++ )    {
	    if( i % 2 != 0 )    {
		tdlout.x = tmpout.tmpout_new[i].tidal.x;
		tdlout.y = tmpout.tmpout_new[i].tidal.y;
		fprintf( tidal, "%d,%d,%d\n", tdlout.x, tdlout.y, tdlout.z );
		}
	    else
		tdlout.z = tmpout.tmpout_new[i].tidal.z;
	    }

	fclose( tidal );
	}
    }

/* ===== Output temporary data time table to a file ===== */
void OutputTmpTimeTable( MqTime *mqtime, unsigned short *seprec )

/*
 *  This function outputs a temporary data table about record and time.
 *  File format is:
 *    line 1: input data file name
 *    line 2 - : record number and time. 
 */

    {
    FILE    *dtt;      /* file pointer for temporary time table file */
    int     i;         /* loop counter */

    if( NULL == ( dtt = fopen( params.tmpdtt, "w" )))
	DisplayError( EDBFOPEN, params.tmpdtt );

    fprintf( dtt, "%s\n", params.infile );

    for( i = 0; i < params.records; i++ )
	fprintf( dtt, "%d %d %d %d %d %d %d\n",
		i + 1,
		mqtime[i].year, mqtime[i].day, mqtime[i].hour,
		mqtime[i].minute, mqtime[i].second, mqtime[i].ms  );

    fclose( dtt );
    }

/* ===== Output temporary data record table to a file ===== */
void OutputTmpRecTable( unsigned short *seprec )

/*
 *   This function outputs temporary data record table. The format of this
 *   file is:
 *     line 1: number of separated events
 *     line 2 -: starting and ending record number and one that has a gap.
 *     first number is starting record number and last is ending, and gap
 *     records except two.
 */

    {
    FILE    *drt;    /* file pointer for temporary record table */
    int     i;       /* loop counter */

    if( NULL == ( drt = fopen( params.tmpdrt, "w" )))
	DisplayError( EDBFOPEN, params.tmpdrt );

    fprintf( drt, "%d\n", seprec[0] );

    for( i = 1; i <= params.records; i++ )    {
	switch( seprec[i] )    {
	case GAP:
	    fprintf( drt, "%d ", i + 1 );
	    break;
	case SEP:
	    fprintf( drt, "%d\n", i );
	    fprintf( drt, "%d ", i + 1 );
	    break;
	case DATASTART:
	    fprintf( drt, "%d ", i );
	    break;
	case DATAEND:
	    fprintf( drt, "%d\n", i );
	    fclose( drt );
	    break;
	    }
	}

    fclose( drt );
    }

/* ===== Output Moonquake database format file ===== */
void OutputMqFile( MqHeader *header, double *mqdata )
    {
    MqAccess   access;    /* access control structure */
    char      *filename,  /* file name ( full name ) */
              *basename,  /* base name */
              *dbworkdir; /* environmental variable "DBWORKDIR" */

    /* output MQDB file name is arranged */
    if( NULL == ( filename = calloc( FILENAME_MAX, 1 )))
	DisplayError( EDBMEMALLOC, NULL );

    if( NULL == ( dbworkdir = calloc( FILENAME_MAX, 1 )))
	DisplayError( EDBMEMALLOC, NULL );

    MqSetAccessYes( &access );   /* set access control list */

    /* Output file name is made, and if environemtal variable DBWORKDIR is set,
       file will be output at the directory which assigned by this. */
    if( NULL ==
       ( basename = MqCreFileName( header->Station,
				  header->StartTime, header->DataType ))) {
	DisplayError( EDBFILENAME, NULL );
	exit( EDBFILENAME );
	}

    if( NULL != ( dbworkdir = getenv( "DBWORKDIR" )))    {
	strcpy( filename, dbworkdir );
	if( dbworkdir[strlen(dbworkdir)-1] != '/' )
	    strcat( filename, "/" );
	strcat( filename, basename );
	}
    else
	strcpy( filename, basename );

    /* Average and Maximum value of the amplitude will be set */
    RearrangeMqData( header, mqdata );

    if( optflag.verbose == VERYVERBOSE )
	printf( "Created filename: %s\n", filename );

    MqCreate( filename, &access, header, mqdata );

    free( dbworkdir );
    free( filename );
    }

/* ===== Rearrange Moonquake data ===== */
void RearrangeMqData( MqHeader *header, double *mqdata )

/*
 *  This function re-arranges Moonquake data by substracting average amplitude
 *  and searching maximum amplitude. These calculation is performed using 
 *  MQDB utility functions.
 */

    {
    unsigned short i;        /* loop counter */
    unsigned long  datanum;  /* number of data in one channel */

    /* calculation of average amplitude */
    if( header->Channels == 0 )
	exit( EXIT_FAILURE );

    datanum = header->NumOfData / header->Channels;

    for( i = 0; i < header->Channels; i++ )
	header->AveAmp.data[i] = 
	    MqCalcAveAmp( mqdata, header->Channels + i * datanum, datanum );

    /* calculation of maximum amplitude */
    for( i = 0; i < header->Channels; i++ )
	header->MaxAmp.data[i] = 
	    MqCalcMaxAmp( mqdata, header->Channels + i * datanum, datanum );
    }

/* =========================== END of the program ========================== */
