/* ----------------------- Moonquake database utilities ----------------------+
|                                                                             |
|                             "dbread.c" ver 1.00                             |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/*
 *  This is a part of "dbread" file that contains about data reading functions
 *  of Moonquake raw data file.
 */

/* ============================= Include files ============================= */
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "mqdb.h"       /* Moonquake database header file */
#include "dbconv.h"     /* header file of this program */

/* ========================== Program statements  ========================== */

/* ===== Read FIELD ( OLD data format ) ===== */
void ReadFieldOld( RECPTR recptr, FIELD_OLD field,
		  MqTime *mqtime, TMPOUT_OLD *tmpout_old )
    {
    int  j;    /* loop counter */

    for( j = 0; j < CELL_NUM_OLD; j++ )    {
	switch( j + 1 )    {
	case 1:
	    if( recptr.field == 0 )
#ifdef NOLONGLONG
		ConvMqTime( field.field.fheader.time1,
			   field.field.fheader.time2, mqtime, recptr.record );
#else
	        ConvMqTime( field.field.fheader.time, mqtime, recptr.record );
#endif
	    break;
	case 4:
	    tmpout_old->spz[1]  = field.field.cell[j].datacell.data1;
	    tmpout_old->spz[2]  = field.field.cell[j].datacell.data2;
	    tmpout_old->spz[3]  = field.field.cell[j].datacell.data3;
	    break;
	case 5:
	    tmpout_old->lpx[1]  = field.field.cell[j].datacell.data1;
	    tmpout_old->spz[4]  = field.field.cell[j].datacell.data2;
	    tmpout_old->lpy[1]  = field.field.cell[j].datacell.data3;
	    break;
	case 6:
	    tmpout_old->spz[5]  = field.field.cell[j].datacell.data1;
	    tmpout_old->lpz[1]  = field.field.cell[j].datacell.data2;
	    tmpout_old->spz[6]  = field.field.cell[j].datacell.data3;
	    break;
	case 7:
 	    tmpout_old->spz[7]  = field.field.cell[j].datacell.data1;
	    tmpout_old->spz[8]  = field.field.cell[j].datacell.data2;
	    tmpout_old->spz[9]  = field.field.cell[j].datacell.data3;
	    break;
	case 8:
	    tmpout_old->spz[10] = field.field.cell[j].datacell.data1;
	    tmpout_old->spz[11] = field.field.cell[j].datacell.data2;
	    tmpout_old->lpx[2]  = field.field.cell[j].datacell.data3;
	    break;
	case 9:
	    tmpout_old->spz[12] = field.field.cell[j].datacell.data1;
	    tmpout_old->lpy[2]  = field.field.cell[j].datacell.data2;
	    tmpout_old->spz[13] = field.field.cell[j].datacell.data3;
	    break;
	case 10:
	    tmpout_old->lpz[2]  = field.field.cell[j].datacell.data1;
	    tmpout_old->spz[14] = field.field.cell[j].datacell.data2;
	    tmpout_old->spz[15] = field.field.cell[j].datacell.data3;
	    break;
	case 11:
	    /* data1 is housekeeping */
	    tmpout_old->spz[16] = field.field.cell[j].datacell.data2;
            /* TIDAL ( even FIELD: X, odd FIELD: Z ) */
	    if( recptr.field % 2 != 0 )
		tmpout_old->tidal.x = field.field.cell[j].datacell.data3;
	    else
		tmpout_old->tidal.z = field.field.cell[j].datacell.data3;
	    break;
	case 12:
	    tmpout_old->spz[17] = field.field.cell[j].datacell.data1;
            /* TIDAL ( even FIELD: Y, odd FIELD: temperature ) */
	    if( recptr.field % 2 != 0 )
	       tmpout_old->tidal.y = field.field.cell[j].datacell.data2;
	    tmpout_old->spz[18] = field.field.cell[j].datacell.data3;
	    break;
	case 13:
	    tmpout_old->spz[19] = field.field.cell[j].datacell.data1;
	    tmpout_old->lpx[3]  = field.field.cell[j].datacell.data2;
	    tmpout_old->spz[20] = field.field.cell[j].datacell.data3;
	    break;
	case 14:
	    tmpout_old->lpy[3]  = field.field.cell[j].datacell.data1;
	    tmpout_old->spz[21] = field.field.cell[j].datacell.data2;
	    tmpout_old->lpz[3]  = field.field.cell[j].datacell.data3;
	    break;
	case 15:
	    /* data1 is command verification */
	    tmpout_old->spz[22] = field.field.cell[j].datacell.data2;
	    tmpout_old->spz[23] = field.field.cell[j].datacell.data3;
	    break;
	case 16:
	    tmpout_old->spz[24] = field.field.cell[j].datacell.data1;
	    tmpout_old->spz[25] = field.field.cell[j].datacell.data2;
	    tmpout_old->lpx[4]  = field.field.cell[j].datacell.data3;
	    break;
	case 17:
	    tmpout_old->spz[26] = field.field.cell[j].datacell.data1;
	    tmpout_old->lpy[4]  = field.field.cell[j].datacell.data2;
	    tmpout_old->spz[27] = field.field.cell[j].datacell.data3;
	    break;
	case 18:
	    tmpout_old->lpz[4]  = field.field.cell[j].datacell.data1;
	    tmpout_old->spz[28] = field.field.cell[j].datacell.data2;
	    tmpout_old->spz[29] = field.field.cell[j].datacell.data3;
	    break;
	default:
	    break;
	    }
	}
    }

/* ===== Read FIELD ( NEW data format ) ===== */
void ReadFieldNew( RECPTR recptr, FIELD_NEW field, MqTime *mqtime,
		  TMPOUT_NEW *tmpout_new )
    {
    int   j;    /* loop counter */

    for( j = 0; j < CELL_NUM_NEW; j++ )    {
	switch( j + 1 )    {
	case 1:
	    if( recptr.field == 0 )
#ifdef NOLONGLONG
		ConvMqTime( field.field.fheader.time1,
			   field.field.fheader.time2, mqtime, recptr.record );
#else
		ConvMqTime( field.field.fheader.time, mqtime, recptr.record );
#endif
	    break;
	case 3:
	    /* data1 is ALSEP number */
	    break;
	case 4:
	    tmpout_new->lpx[1] = field.field.cell[j].datacell.data1;
	    tmpout_new->lpy[1] = field.field.cell[j].datacell.data2;
	    tmpout_new->lpz[1] = field.field.cell[j].datacell.data3;
	    break;
	case 5:
	    tmpout_new->lpx[2] = field.field.cell[j].datacell.data1;
	    tmpout_new->lpy[2] = field.field.cell[j].datacell.data2;
	    tmpout_new->lpz[2] = field.field.cell[j].datacell.data3;
	    break;
	case 6:
	    /* data1 is housekeeping */
	    /* data2 is TIDAL: even FIELD is X and odd FIELD is Z */
	    /* data3 is TIDAL and temp.: even FIELD is Y and odd is temp. */
	    if( recptr.field % 2 != 0 )    {
		tmpout_new->tidal.x = field.field.cell[j].datacell.data2;
		tmpout_new->tidal.y = field.field.cell[j].datacell.data3;
		}
	    else
		tmpout_new->tidal.z = field.field.cell[j].datacell.data2;
	    break;
	case 7:
	    tmpout_new->lpx[3] = field.field.cell[j].datacell.data1;
	    tmpout_new->lpy[3] = field.field.cell[j].datacell.data2;
	    tmpout_new->lpz[3] = field.field.cell[j].datacell.data3;
	    break;
	case 8:
	    /* data1 is command verification */
	    tmpout_new->lpx[4] = field.field.cell[j].datacell.data2;
	    tmpout_new->lpy[4] = field.field.cell[j].datacell.data3;
	    break;
	case 9:
	    tmpout_new->lpz[4] = field.field.cell[j].datacell.data1;
	    break;
	default:
	    break;
	    }
	continue;
	}
    }

/* ===== Read a Moonquake raw data file ===== */
void ReadRawFile( void )
    {
    FILE           *fin;         /* input file */
    OUTPAR          outpar[RECORDMAX];      /* output parameters */
    MqTime          mqtime[RECORDMAX];
                                 /* Moonquake database time expression */
    double         *timediff;    /* time difference between two records */
    unsigned short *seprec;      /* record separation descriptor */

    if( NULL == ( timediff =
	 (double *)calloc( params.records, sizeof(double))))
	DisplayError( EDBMEMALLOC, NULL );
    if( NULL == ( seprec =
	 (unsigned short *)calloc( ( params.records + 1 ), sizeof(short))))
	DisplayError( EDBMEMALLOC, NULL );
    
    if( NULL == ( fin = fopen( params.infile, "r+b" )))
	DisplayError( EDBFOPEN, params.infile );

    ReadRecord( fin, timediff, outpar, mqtime );
    params.outpar.pse = outpar[0].pse;
    params.outpar.seqno = outpar[0].seqno;
    params.outpar.station = outpar[0].station;
    params.outpar.format = outpar[0].format;

    SearchRecSeparator( params.records, outpar, timediff, seprec );

    if( optflag.verbose >= VERBOSE )
	DisplayRecordTable( outpar, mqtime, timediff, seprec );

    if( optflag.table == TRUE )    {
	OutputTmpTimeTable( mqtime, seprec ); 
	OutputTmpRecTable( seprec );
	}

    fclose( fin );
    free( timediff );
    free( seprec );
    }

/* ===== Read one record ===== */
void ReadRecord( FILE *fin, double *timediff, OUTPAR *outpar, MqTime *mqtime )
    {
    RECORD          record;      /* data record */
    RECPTR          recptr;      /* position of record reading */
    unsigned int    readdata;    /* number of read characters by fread() */
    TMPOUT          tmpout;      /* temporary data storage variable */
    
    for( recptr.record = 0; recptr.record < params.endrec;
	recptr.record++ )    {
	readdata = fread( record.record, sizeof( char ),
			 RECORDSIZE, fin );     /* reading from a file */

	if( readdata != RECORDSIZE ) DisplayError( EDBILFORM, NULL );

	outpar[recptr.record].pse     = record.header.pse;
	outpar[recptr.record].seqno   = record.header.seqno;
	outpar[recptr.record].format  = record.header.dataform;
	outpar[recptr.record].station = record.header.station;
	  
	if( record.header.dataform != OLD && record.header.dataform != NEW )
	    DisplayError( EDBILFORMNUM, NULL );

	if( recptr.record >= ( params.startrec - 1 ))    {
	    mqtime[recptr.record].year = record.header.year;
	
	    for( recptr.field = 0;
		recptr.field < fieldnum[record.header.dataform];
		recptr.field++ )    {
		if( record.header.dataform == OLD )
		    ReadFieldOld( recptr,
				 record.record_old.field[recptr.field],
				 mqtime, &(tmpout.tmpout_old[recptr.field]) );
		else
		    ReadFieldNew( recptr,
				 record.record_new.field[recptr.field],
			     mqtime, &(tmpout.tmpout_new[recptr.field]) );
		}/* end of reading one field */

	    if( recptr.record != 0 )
		timediff[recptr.record] = MqDiffTime( mqtime[recptr.record],
						     mqtime[recptr.record-1]);
	    
	    ( record.header.dataform == OLD ) ? 
		( OutputDataTmpOld( tmpout, recptr )) :
		( OutputDataTmpNew( tmpout, recptr ));
	    }
	}    /* end of reading one record */
    }


/* ===== Read temporary DTT ( Data Time Table ) file ===== */
void ReadTmpDTT( char *filename, MqTime *mqtime, EVDESC *evdesc )

/*
 *  This function reads temporary DTT ( Data Time Table ) file and returns 
 *  start time of the event and input file name.
 *    params:   assigned data parameters ( IN )
 *    filename: input data file name described in DTT file ( OUT )
 *    mqtime:   data start time ( OUT )
 *    evdesc:   event data descriptor ( IN )
 */ 

    {
    FILE   *tmpdtt;    /* file descriptor for temporary DTT file */
    int     i,         /* loop counter */
            records,   /* record number in DRT file */
            buflen;    /* length of the buffer */
    char buf[BUFSIZ];

    if( NULL == ( tmpdtt = fopen( params.tmpdtt, "r" )))
	DisplayError( EDBFOPEN, params.tmpdtt );

    for( i = 0; i <= params.records; i++ )    {
	fgets( buf, BUFSIZ, tmpdtt );
	if( i == 0 )    {
	    buflen = strlen( buf );
	    strncpy( filename, buf, buflen - 1 );
	    }
	else if( i == evdesc->start )
	    sscanf( buf, "%d %hd %hd %hd %hd %hd %hd\n",
		   &records, &mqtime->year, &mqtime->day,
		   &mqtime->hour, &mqtime->minute,
		   &mqtime->second, &mqtime->ms );
	}

    fclose( tmpdtt );
    }

/* ===== Read temporary data file ( LP ) ===== */
void ReadTmpLP( double *mqdata, EVDESC evdesc, MQDF dataformat )
    {
    FILE    *tmplp;
    unsigned long   start,     /* start number of data */
                    end,       /* end number of data */
                    datanum,   /* number of data */
                    fieldnum;  /* number of FIELDs */
    int      i, j;
    char     buf[BUFSIZ];

    if( NULL == ( tmplp = fopen( params.tmplp, "rt" )))
	DisplayError( EDBFOPEN, params.tmplp );

    switch( dataformat )    {
    case MQ_OLD:
	fieldnum = FIELD_NUM_OLD;
	break;
    case MQ_NEW:
	fieldnum = FIELD_NUM_NEW;
	break;
	}

    start = (unsigned long)( evdesc.start - 1 ) * LP_DATA_NUM * fieldnum + 1 ;
     end  = (unsigned long)evdesc.end   * LP_DATA_NUM * fieldnum;

    datanum = end - start + 1;

    if( optflag.verbose == VERYVERBOSE )
	fprintf( stderr, "Reading LP data: from %ld to %ld\n", start, end );

    for( i = 1, j = 0; i <= end; i++ )    {
	fgets( buf, BUFSIZ, tmplp );
	if( i >= start )    {
	    sscanf( buf, "%lf,%lf,%lf",
		   mqdata+j, mqdata+datanum+j, mqdata+datanum*2+j );
	    j++;
	    }
	}

    fclose( tmplp );
    }

/* ===== Read temporary data file ( SP ) ===== */
void ReadTmpSP( double *mqdata, EVDESC evdesc )
    {
    FILE    *tmpsp;
    unsigned long   start, end;
    unsigned int    spint;        /* temporary input variable */
    char     buf[BUFSIZ];
    int      i, j;    /* loop counter */

    if( NULL == ( tmpsp = fopen( params.tmpsp, "r" )))
	DisplayError( EDBFOPEN, params.tmpsp );

    start =
	(unsigned long)( evdesc.start - 1 ) * SP_DATA_NUM * FIELD_NUM_OLD + 1 ;
     end  =
	 (unsigned long)evdesc.end   * SP_DATA_NUM * FIELD_NUM_OLD;

    if( optflag.verbose == VERYVERBOSE )
	fprintf( stderr, "Reading SP data: from %ld to %ld\n", start, end );

    for( i = 1, j = 0; i <= end; i++ )    {
/*	fgets( buf, BUFSIZ, tmpsp );*/
	fread( &spint, sizeof(unsigned int), 1, tmpsp );
	if( i >= start )    {
/*	    sscanf( buf, "%lf", mqdata+j );*/
/*	    fread( &spint, sizeof(unsigned int), 1, tmpsp );*/
	    mqdata[j] = (double)spint;
	    j++;
	    }
	}

    fclose( tmpsp );
    }

/* ===== Read temporary data file ( TIDAL ) ===== */
void ReadTmpTIDAL( double *mqdata, EVDESC evdesc, MQDF dataformat )
    {
    FILE    *tmptidal;
    unsigned long   start, end, tdldata, tdldatanum;
    char     buf[BUFSIZ];
    int      i, j;    /* loop counter */

    if( NULL == ( tmptidal = fopen( params.tmptidal, "rt" )))
	DisplayError( EDBFOPEN, params.tmptidal );

    if( dataformat == MQ_OLD )
	tdldata = FIELD_NUM_OLD / 2;
    else if( dataformat == MQ_NEW )
	tdldata = FIELD_NUM_NEW / 2;

    start = (unsigned long)( evdesc.start - 1 ) * tdldata + 1 ;
     end  = (unsigned long)evdesc.end * tdldata;
    tdldatanum = end - start + 1;

    if( optflag.verbose == VERYVERBOSE )
	fprintf( stderr, "Reading TIDAL data: from %ld to %ld\n", start, end );

    for( i = 1, j = 0; i <= end; i++ )    {
	fgets( buf, BUFSIZ, tmptidal );
	if( i >= start )    {
	    sscanf( buf, "%lf,%lf,%lf", 
		   mqdata+j, mqdata+tdldatanum+j, mqdata+tdldatanum*2+j );
	    j++;
	    }
	}

    fclose( tmptidal );
    }

/* =========================== END of the program ========================== */
