/* ----------------------- Moonquake database utilities ----------------------+
|                                                                             |
|                              "dbset.c" ver 1.00                             |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/*
 *   This file contains functions for settings of variables.
 */

/* ============================= Include files ============================= */
#include <stdio.h>
#include <stdlib.h>
#include "mqdb.h"      /* Moonquake databse header files */
#include "dbconv.h"    /* header file of this program */

/* ========================== Program statements  ========================== */

/* ===== Check command line options and set if these are valid ===== */
void SetOptionFlag( int argc, char **argv )

/*
 *  This function checks option arguments and stores option flag variables
 *  by specifying each option characters.
 *    argc:     number of command line options   ( IN )
 *    argv:     command line option arguments    ( IN )
 */

    {
    int   i, j,             /* loop counters */
          length,           /* length of option arguments */
          nominusopt = 0;   /* number of option arguments not begin with - */

    if( argc == 1 ) DisplayError( EDBARGS, NULL );
    
    for( i = 1; i < argc; i++ )    {
	if( *argv[i] == '-' )    {
	    length = strlen( argv[i] );
	    if( length == 1 ) DisplayError( EDBMINUSONLY, NULL );
	    for( j = 1; j < length; j++ )
		EvalSwitchChar( *(argv[i]+j) );
	    }
	else    {
	    if( nominusopt == 0 ) optflag.file = i;
	    if( nominusopt == 1 ) optflag.startrec = i;
	    if( nominusopt == 2 ) optflag.endrec = i;
	    nominusopt++;
	    }
	}

    SetParams( nominusopt, argv );
    }

/* ===== Set Parameters from command line ===== */
void SetParams( int nominusopt, char **argv )    
    {
    switch( nominusopt )    {
    case 0:
	DisplayError( EDBARGS, NULL );
    case 1:
	strcpy( params.infile, argv[optflag.file] );
	params.startrec = params.endrec = 0;
	break;
    case 2:
	DisplayError( EDBBOTHREQ, NULL );
    case 3:
	strcpy( params.infile, argv[optflag.file] );
	params.startrec = atoi( argv[optflag.startrec] );
	params.endrec   = atoi( argv[optflag.endrec] );
    default:
	DisplayError( EDBILOPT, NULL );
	}
    }

/* ===== Set Event Descriptor ===== */
void SetEventDesc( char *drtline, EVDESC *evdesc )

/*
 *    This function searches strings *drtline and separates characters in this
 *    string into start/end records and comment records number.
 */

    {
    unsigned int    i, j, k = 0,   /* loop counter */
                    linelen,       /* length of *drtline */
                    spcount = 0,   /* count of separator */
                    start = 0;     /* start of the token */
    char           *tmp;           /* temporary storage */
    unsigned short *tmprec;        /* temporary storage for record numbers */

    linelen = strlen( drtline );
    if( linelen == 0 ) DisplayError( EDBZERODRT, NULL );

    for( i = 0; i < linelen; i++ )    {
	if( drtline[i] == ' ' ) spcount++;
	if( isdigit( drtline[i] == 0 )) DisplayError( EDBILDRT, NULL );
	}

    if( spcount == 0 ) DisplayError( EDBILDRT, NULL );
    if( NULL == 
       ( tmprec = calloc( spcount + 1, sizeof(short))))
	DisplayError( EDBMEMALLOC, NULL );

    for( i = 0; i < linelen; i++ )    {
	if( drtline[i] == ' ' || drtline[i] == '\n' )    {
	    if( NULL == 
	       ( tmp = calloc( i - start + 1, sizeof(short))))
		DisplayError( EDBMEMALLOC, NULL );
	    strncpy( tmp, drtline + start, i - start );
	    tmprec[k] = (unsigned short)atoi( tmp );
	    k++;
	    start = i + 1;
	    free( tmp );
	    }
	}

    evdesc->start  = tmprec[0];
    evdesc->end    = tmprec[spcount];
    evdesc->comnum = spcount - 1;

    if( spcount > 2 )
	for( i = 0; i < spcount - 1; i++ )
	    *(evdesc->com+i) = tmprec[i+1];

    free( tmprec );
    }

/* ====== Set MqHeader structure ===== */
void SetMqHeader( MqHeader *header, MqTime *mqtime, 
		 MQDT wavetype, EVDESC *evdesc, char *filename )
    {
    unsigned short    records;    /* number of records in one event */

    records = evdesc->end - evdesc->start + 1;

    if( params.outpar.format == OLD )    {
	header->DataFormat = MQ_OLD;
	if( wavetype == MQ_LP ) 
	    header->NumOfData = LP_DATA_NUM * FIELD_NUM_OLD * records * 3;
	else if( wavetype == MQ_SP )
	    header->NumOfData = SP_DATA_NUM * FIELD_NUM_OLD * records;
	else if( wavetype == MQ_TIDAL )
	    header->NumOfData = FIELD_NUM_OLD * records / 2 * 3;
	}	    
    else if( params.outpar.format == NEW )   {
	header->DataFormat = MQ_NEW;
	if( wavetype == MQ_LP )
	    header->NumOfData = LP_DATA_NUM * FIELD_NUM_NEW * records * 3;
	else if( wavetype == MQ_TIDAL )
	    header->NumOfData = FIELD_NUM_NEW * records / 2 * 3;
	}
    else DisplayError( EDBILFORMNUM, NULL );

    if( wavetype == MQ_LP ) {
        header->Channels = 3;
	header->AveAmp.channels = 3;
	header->MaxAmp.channels = 3;
	}
    else if( wavetype == MQ_SP ) {
	header->AveAmp.channels = 1;
	header->Channels = 1;
	header->MaxAmp.channels = 1;
	}
    else if( wavetype == MQ_TIDAL ) {
	header->AveAmp.channels = 3;
	header->Channels = 3;
	header->MaxAmp.channels = 3;
	}

    header->SampRate.channels = 1;
    header->SampRate.data[0] = MqCheckSampRate( wavetype );

    header->DataType = wavetype;
    header->EndRecord = (unsigned long)evdesc->end;
    header->FileType = MQ_FULLTEXT;

    strcpy( header->OrgFile, filename );
    header->StartRecord = (unsigned long)evdesc->start;
    header->StartTime  = *mqtime;
    header->Station    = params.outpar.station;
    header->TapeNumber = params.outpar.seqno;
    }

/* =========================== END of the program ========================== */
