/* ----------------------- Moonquake database utilities ----------------------+
|                                                                             |
|                             "dbtime.c" ver 1.00                             |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/
/*
 *  This file contains functions related to time.
 */

/* ============================= Include files ============================= */
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "mqdb.h"       /* Moonquake database header file */
#include "dbconv.h"     /* header file of this program */

/* ========================== Program statements  ========================== */

/* ===== Convert time data to Moonquake database format structure ===== */
#ifdef NOLONGLONG
void ConvMqTime( unsigned long time1, unsigned long time2,
		MqTime *mqtime, int record )

/*
 *  This function converts time data in input data file expressed in "long"
 *  variable to Moonquake database format structure MqTime. Long variable
 *  time is input and MqTime structure will be returned.
 *  If your C compiler does not support "long long", you should compile with
 *  flag -DNOLONGLONG.
 *    time1 ( or time2 ): bit-expressed time in long ( or long long ) ( IN )
 *    mqtime: time in microsecond ( OUT ) 
 *    record: current record number ( IN )
 */

    {
    unsigned long   tmp1;

    mqtime[record].day = (unsigned long)( time1 / 5400000 );
    tmp1 = time1 - 5400000 * mqtime[record].day;

    tmp1 = tmp1 * 16 + time2;

    mqtime[record].hour = (unsigned long)( tmp1 / 3600000 );
    tmp1 = tmp1 - 3600000 * mqtime[record].hour;
    
    mqtime[record].minute = (unsigned long)( tmp1 / 60000  );
    tmp1 = tmp1 - 60000 * mqtime[record].minute;

    mqtime[record].second = (unsigned long)( tmp1 / 1000 );
    mqtime[record].ms = tmp1 - 1000 * mqtime[record].second;
    }

#else
void ConvMqTime( unsigned long long time, MqTime *mqtime, int record )
    {
    unsigned long long tmp1, tmp2;

    tmp1 = time;

       /* microsecond */
    mqtime[record].ms = time % 1000;
    tmp1 = ( time - mqtime[record].ms ) / 1000;

    mqtime[record].second = tmp1 % 60;
    tmp1 = ( tmp1 - mqtime[record].second ) / 60;

    mqtime[record].minute = tmp1 % 60;
    tmp1 = ( tmp1 - mqtime[record].minute ) / 60;

    mqtime[record].hour = tmp1 % 24;
    tmp1 = ( tmp1 - mqtime[record].hour ) / 24;
    mqtime[record].day = tmp1;
    }
#endif

/* ===== Get and display time ===== */
void GetTime( int timemode )
    {
    time_t    thetime;
    double    psclock;
    char      timedisp[23];
    struct tm   *tmptime;

    params.time[timemode] = time( &thetime );

    params.progtime[timemode] = localtime( &(params.time[timemode]) );
    strftime( timedisp, 23, "%Y %b %d %H:%M:%S", params.progtime[timemode] );

    if( optflag.verbose != SILENT )
    printf( "dbconv %s on %s \n", timemodestr[timemode],
	   timedisp );

#ifndef NODIFFTIME
    if( timemode ==  PROG_END )    {
	psclock = difftime( params.time[PROG_END], params.time[PROG_START] );
	if( optflag.verbose != SILENT )
	    printf( "Consumed time is %.1f sec.\n", psclock );
	}
#endif
    }

/* =========================== END of the program ========================== */
