/* ----------------------- Moonquake database utilities ----------------------+
|                                                                             |
|                               "mqsac" ver 1.00                              |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1994-1995 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/*
 *  This program converts MQDB format files to SAC ( Seismic Analyze Code )
 *  format file and vice versa ( SAC -> MQDB facility is not supported ).
 *  Main routine and some other functions such as argument processing and
 *  variable conversions are written in this file.
 *    program usage: mqsac infile outfile
 *                   mqsac infile
 */

/* ============================= Include files ============================= */
#include "mqsac.h"

/* ========================== Program statements  ========================== */
main( int argc, char **argv )
    {
    MqHeader header;   /* header contentes resides in this variable */
    MqAccess access;   /* header access control variable */
    double  *data;     /* actual data */

    InitParams();
    strcpy( params.progname, argv[0] );

    if( argc != 1 ) SetOptions( argc, argv );
    else            params.help = MQ_YES;

    if( params.help == MQ_YES ) {
	fprintf( stderr, "%s", helpmsg );
	exit( EMSNOERROR );
	}

    MqInit( progsign );
    MqSetAccessYes( &access );

    if( NULL == ( data = MqRead( params.infile, &access, &header )))
	DisplayError( params.infile, EMSREADFAIL );

    switch( header.DataType ) {
    case MQ_LP:
	OutputSACfileLP( &header, data );
	break;
    case MQ_SP:
	OutputSACfileSP( &header, data );
	break;
    case MQ_TIDAL:
	OutputSACfileTDL( &header, data );
	break;
    default:
	/* OutputSACfile( header, data ); */
	fprintf( stderr, "Only moonquake data are supported\n" );
	}

    exit( EMSNOERROR );
    }

/* ===== Set optinal argument and parameters ===== */
void SetOptions( int argc, char **argv )
    {
    unsigned short   i, j,         /* loop character */
                    *argcount,     /* count of argument */
                     fncount = 0,  /* file name count */
                     incount = 0,  /* input file name position */
                     outcount= 0,  /* output file name position */
                     optnum;       /* number of options */
    char *optchar;

    argcount = calloc( argc, sizeof( short ));
    for( i = 0; i < argc; i++ ) argcount[i] = FALSE;
    argcount[0] = TRUE;   /* this is always true */

    optnum = sizeof( optidx ) / ( OPTLEN + 4 + 4 );

    for( i = 0; i < argc; i++ ) {
	for( j = 0; j < optnum; j++ ) {
	    if( NULL == ( optchar = calloc( 18, 1 )))
		DisplayError( NULL, EMSMEMALLOC );
	
	    optchar[0] = '-';
	    strcat( optchar + 1, optidx[j].opt );

	    if( argv[i][0] == '-' ) {
		if( 0 ==
		   strncmp( argv[i], optchar, strlen( optchar ))) {
		    switch( optidx[j].optargs ) {
		    case MQ_NO:
			SetParams( optidx[j].func, NULL );
			argcount[i] = TRUE;
			break;
		    case MQ_YES:
			SetParams( optidx[j].func, argv[i+1] );
			argcount[i] = argcount[i+1] = TRUE;
			i++;
			break;
		    default:

			break;
			}
		    }
		}
	    free( optchar );
	    }
	}

    for( i = 0; i < argc; i++ ) {
	if( argcount[i] == FALSE ) {
	    if( fncount == 0 )
		incount = outcount = i;
	    else if( fncount == 1 )
		outcount = i;
	    else
		DisplayError( NULL, EMSNOARGS );

	    fncount++;
	    }
	}

    switch( fncount ) {
    case 0:
	fprintf( stderr, "%s", helpmsg );
	exit( EMSNOERROR );
    case 1:
	strcpy( params.infile, argv[incount] );
	MakeOutFileNameBase( params.infile, params.outfile );
	break;
    case 2:
	strcpy( params.infile, argv[incount] );
	strcpy( params.outfile, argv[outcount] );
	break;
    default:
	DisplayError( params.progname, EMSILOPT );
	}

    free( argcount );
    }

/* ===== Set parameters ===== */
void SetParams( unsigned short func, char *args )
    {
    switch( func ) {
    case VERBOSE:
	params.verbose = MQ_YES;
	break;
    case HELP:
	params.help = MQ_YES;
	break;
    default:
	DisplayError( NULL, EMSILOPT );
	}
    }

/* ==== Put error message and exits if necessary ===== */
void DisplayError( char *message, unsigned short errnum )
    {
    if( message == NULL )
	fprintf( stderr, "%s.\n", error[errnum] );
    else
	fprintf( stderr, "%s: %s.\n", message, error[errnum] );

    if( error[errnum].level == MS_HELP )
        fprintf( stderr, "%s", helpmsg );

    if( error[errnum].level == MS_FATAL || error[errnum].level == MS_HELP )
        exit( errnum );
    }

/* ===== Initialize program parameters ===== */
void InitParams( void )
    {
    params.help    = MQ_NO;  /* help output is false */
    params.verbose = MQ_NO;  /* silent by default */
    }

/* ===== Make output file name ===== */
void MakeOutFileNameBase( char *infile, char *outfile )
    {
    char *outdir,     /* output directory name ( if specified ) */
         *dirpos,     /* position of '/' in *infile */
         *infiletmp,  /* work variable, same as *infile */
         *infilebase, /* base part of *infile */
         *infiledir,  /* dir part of *infile */
         *infilebody; /* base part without extensions */
    unsigned int len;

    if( NULL == ( outdir = calloc( FILENAME_MAX, 255 )))
	DisplayError( NULL, EMSMEMALLOC );
    if( NULL == ( infilebase = calloc( FILENAME_MAX, 255 )))
	DisplayError( NULL, EMSMEMALLOC );
    if( NULL == ( infiletmp = calloc( FILENAME_MAX, 255 )))
	DisplayError( NULL, EMSMEMALLOC );
    if( NULL == ( infiledir = calloc( FILENAME_MAX, 255 )))
	DisplayError( NULL, EMSMEMALLOC );
    if( NULL == ( infilebody = calloc( FILENAME_MAX, 255 )))
	DisplayError( NULL, EMSMEMALLOC );

    strcpy( infiletmp, infile );

/*
 * If environmental variable "MQSACDIR" is given, output directory is set 
 * to there.
 */
    outdir = getenv( "MQSACDIR" );
    if( outdir != NULL ) {
	len = strlen( outdir );
	if( outdir[len-1] != '/' ) strcat( outdir, "/" );
	}

    dirpos = (char *)strrchr( infiletmp, '/' );

    if( NULL == dirpos ) {   /* if no '/' found */
	infilebody = ExtractExt( infiletmp );
	if( outdir != NULL )
	    strcpy( outfile, outdir );
	
	strcat( outfile, infilebody );
	}
    else {    /* if infile contains '/' */
	strcpy( infiledir, infiletmp );
	dirpos = (char *)strrchr( infiledir, '/' );
	strcpy( infilebase, dirpos+1 );

	*(dirpos+1) = '\0';
	( outdir != NULL ) ?
	    ( strcpy( outfile, outdir )) : ( strcpy( outfile, infiledir ));

	infilebody = ExtractExt( infilebase );
	strcat( outfile, infilebody );
	}	

    free( infiletmp );
    free( infilebase );
    free( infiledir );
    free( infilebody );
    }

/* ===== Extract file name extension ===== */
char *ExtractExt( char *filename )
    {
    char body[FILENAME_MAX],     /* base name without extension */
         *dotpos;   /* position of '.' */

    dotpos = (char *)strchr( filename, '.' );

    if( NULL == dotpos )         /* if '.' not found */
	return filename;
    else {
	strcpy( body, filename );
	dotpos = (char *)strchr( body, '.' );
	*dotpos = '\0';
	return body;
	}
    }

/* ===== Output SAC format file ( LP data ) ===== */
void OutputSACfileLP( MqHeader *header, double *data )
    {
    char  *outfile;    /* output file name */
    unsigned short i, j;  /* loop counter */
    unsigned long  datanum, true, false,
                   nerr;
    float  begin, end, 
          *rdata;

    true  = TRUE;
    false = FALSE;
    datanum = header->NumOfData / header->Channels;
    if( NULL == ( rdata = calloc( datanum, sizeof( float ))))
	DisplayError( NULL, EMSMEMALLOC );

    for( i = 0; i < 3; i++ ) {
	if( NULL == ( outfile = calloc( FILENAME_MAX, 1 )))
	    DisplayError( NULL, EMSMEMALLOC );

	strcpy( outfile, params.outfile );
	strcat( outfile, LPFileNameExt );

	switch( i ) {
	case 0:
	    strcat( outfile, "x" );
	    break;
	case 1:
	    strcat( outfile, "y" );
	    break;
	case 2:
	    strcat( outfile, "z" );
	    break;
	    }

	strcat( outfile, SACext );
	fprintf( stderr, "Output file is %s\n", outfile );
	
	for( j = 0; j < datanum; j++ )
	    rdata[j] = (float)data[j+i*datanum];

	begin = rdata[0];
	end   = rdata[datanum-1];

	nerr = 0;
	newhdr_();    /* create new header */

	setn_( field_npts, &datanum, &nerr );
	if( params.verbose == MQ_YES )
	    fprintf( stderr, "NPTS set\n" );

	setf_( field_b, &begin, &nerr );
	if( params.verbose == MQ_YES )
	    fprintf( stderr, "B set\n" );

	setf_( field_e, &end, &nerr );
	if( params.verbose == MQ_YES )
	    fprintf( stderr, "E set\n" );

	seti_( field_iftype, iftype[ITIME], &nerr );
	if( params.verbose == MQ_YES )
	    fprintf( stderr, "IFTYPE set\n" );

	setl_( field_leven, &true, &nerr );
	if( params.verbose == MQ_YES )
	    fprintf( stderr, "LEVEN set\n" );

	nerr = SetDELTA( header->SampRate );
	nerr = SetTime( header->StartTime );

	seti_( field_iztype, iztype[IB], &nerr );
	if( params.verbose == MQ_YES )
	    fprintf( stderr, "IZTYPE set\n" );

	seti_( field_ievtyp, ievtyp[IQUAKE], &nerr );
	if( params.verbose == MQ_YES )
	    fprintf( stderr, "IEVTYP set\n" );

	if( nerr != 0 )
	    fprintf( stderr, "  Some error occured\n" );

	for( j = strlen( outfile ); j < 255; j++ ) 
	    outfile[j] = ' ';

	writes_( outfile, rdata, &nerr );
	if( params.verbose == MQ_YES )
	    fprintf( stderr, "WSAC0 completed\n" );

	free( outfile );
	}
    free( rdata );
    }

/* ===== Output SAC format file ( SP data ) ===== */
void OutputSACfileSP( MqHeader *header, double *data )
    {
    char *outfile;

    if( NULL == ( outfile = calloc( FILENAME_MAX, 1 )))
	DisplayError( NULL, EMSMEMALLOC );

    free( outfile );
    }

/* ===== Output SAC format file ( TIDAL data ) ===== */
void OutputSACfileTDL( MqHeader *header, double *data )
    {
    char *outfile;

    if( NULL == ( outfile = calloc( FILENAME_MAX, 1 )))
	DisplayError( NULL, EMSMEMALLOC );

    free( outfile );
    }

/* ===== Set DELTA header field ===== */
long SetDELTA( MqAmp amp )
    {
    float delta;     /* stored increment value */
    long  nerr;      /* error number */

    if( amp.channels != 1 )
	DisplayError( NULL, EMSDELTA );

    delta = (float)amp.data[0];
    setf_( field_delta, &delta, &nerr );

    if( params.verbose == MQ_YES )
	fprintf( stderr, "DELTA set\n" );

    return nerr;
    }

/* ===== Set time-related headers ===== */
long SetTime( MqTime mqtime )
    {
    long nzyear, nzjday, nzhour, nzmin, nzsec, nzmsec,  /* time value */
         nerr;                                          /* error code */

    MqCheckTime( &mqtime );

    if( MqErr.errno != EMQNOERROR ) 
	DisplayError( NULL, EMSILMQTIME );

    nzyear = (long)mqtime.year;
    nzjday = (long)mqtime.day;
    nzhour = (long)mqtime.hour;
    nzmin  = (long)mqtime.minute;
    nzsec  = (long)mqtime.second;
    nzmsec = (long)mqtime.ms;

    setn_( field_nzyear, &nzyear, &nerr );
    setn_( field_nzjday, &nzjday, &nerr );
    setn_( field_nzhour, &nzhour, &nerr );
    setn_( field_nzmin,  &nzmin,  &nerr );
    setn_( field_nzsec,  &nzsec,  &nerr );
    setn_( field_nzmsec, &nzmsec, &nerr );

    return nerr;
    }

/* =========================== END of the program ========================== */
