/* ----------------------- Moonquake database utilities ----------------------+
|                                                                             |
|                              "testgen" ver 1.00                             |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/*
 *  This program generates test data of MQDB format. 
 */

/* ============================= Include files ============================= */
#include "testgen.h"

/* ========================== Program statements  ========================== */
main( int argc, char **argv )
    {
    double   *data;
    MqAccess  access;
    MqHeader  header;

    InitParams( );
    if( argc != 1 ) SetOptions( argc, argv );

    if( params.help == MQ_YES ) {
	fprintf( stderr, "%s", helpmsg );
	exit( HELP );
	}

    MqInit( progsign );
    if( NULL == ( data = GenerateSin() )) {
	exit( ETGGENDATA );
	}

    MqSetAccessNo( &access );

    access.AveAmp    = MQ_YES;
    access.MaxAmp    = MQ_YES;
    access.Channels  = MQ_YES;
    access.FileType  = MQ_YES;
    access.NumOfData = MQ_YES;
    access.StartTime = MQ_YES;
    access.SampRate  = MQ_YES;
    access.DataType  = MQ_YES;
    access.Station   = MQ_YES;

    header = SetHeaderData( data );

    if( params.outfile == TG_STDOUT )
	MqCreate( NULL, &access, &header, data );
    else if( params.outfile == TG_FILE )
	MqCreate( params.filename, &access, &header, data );
    
    }

/* ===== Set optinal argument and parameters ===== */
void SetOptions( int argc, char **argv )
    {
    unsigned short   i, j,        /* loop character */
                     argnum = 1,  /* number of counted arguments */
                     optnum;      /* number of options */
    char *optchar;

    optnum = sizeof( optidx ) / ( OPTLEN + 4 + 4 );

    for( i = 0; i < argc; i++ ) {
	for( j = 0; j < optnum; j++ ) {
	    if( NULL == ( optchar = calloc( 18, 1 )))
		DisplayError( ETGMEMALLOC );
	
	    optchar[0] = '-';
	    strcat( optchar + 1, optidx[j].opt );

	    if( argv[i][0] == '-' ) {
		if( 0 ==
		   strncmp( argv[i], optchar, strlen( optchar ))) {
		    switch( optidx[j].optargs ) {
		    case MQ_NO:
			SetParams( optidx[j].func, NULL );
			argnum += 1;
			break;
		    case MQ_YES:
			SetParams( optidx[j].func, argv[i+1] );
			argnum += 2;
			i++;
			break;
		    default:

			break;
			}
		    }
		}
	    free( optchar );
	    }
	}
    if( argnum == argc )
	params.outfile = TG_STDOUT;  /* output is done for stdout */
    else {
	params.outfile = TG_FILE;    /* output is done for regular file */
	strcpy( params.filename, argv[argc-1] );
	}
    }

/* ===== Prepare header data for output ===== */
MqHeader SetHeaderData( double *data )
    {
    MqHeader header;     /* header data */
    unsigned long  i;    /* loop counter */

    for( i = 0; i < CHANNELS; i++ ) {
	header.AveAmp.data[i] = MqCalcAveAmp( data, DATANUM*i, DATANUM );
	header.MaxAmp.data[i] = MqCalcMaxAmp( data, DATANUM*i, DATANUM );
	}

    switch( params.filetype ) {
    case TEXT:
	header.FileType = MQ_FULLTEXT;
	break;
    case BIN:
	header.FileType = MQ_COMPOSITE;
	break;
    case XDR:
	header.FileType = MQ_XDR;
	break;
    default:
	DisplayError( ETGILFILETYPE );
	}

    header.AveAmp.channels = header.MaxAmp.channels = CHANNELS;
    header.Channels = CHANNELS;
    header.NumOfData = CHANNELS * DATANUM;
    header.SampRate.data[0] = SAMPRATE;
    header.SampRate.channels = 1;

    header.DataType = MQ_FULLTEXT;

    header.StartTime.year   = YEAR;
    header.StartTime.day    = DAY;
    header.StartTime.hour   = HOUR;
    header.StartTime.minute = MINUTE;
    header.StartTime.second = SECOND;
    header.StartTime.ms     = MS;

    header.Station = STATION;

    return header;
    }

/* ===== Generate sine wave data ===== */
double *GenerateSin( void )
    {
    double *data, t;
    unsigned long i;
    
    if( NULL == ( data = calloc( DATANUM*CHANNELS, sizeof( double )))) {
	exit( ETGMEMALLOC );
	}

    for( i = 0; i < DATANUM; i++ ) {
	t = (double)i / 100.0 * 2.0 * M_PI;
	data[i] = data[i+DATANUM] = data[i+DATANUM*2] = sin( t );
	}

    return data;
    }

/* ===== Display error message ===== */
void DisplayError( unsigned short errnum )
    {
    fprintf( stderr, "%s.\n", error[errnum] );

    if( error[errnum].level == TG_HELP )
	fprintf( stderr, "%s", helpmsg );
    else if( error[errnum].level == TG_FATAL )
	exit( errnum );
    }

/* ===== Initialize parameters ===== */
void InitParams( void )
    {
    params.help = MQ_NO;  /* help output is false by default ( of course ! ) */
    params.outfile  = TG_STDOUT;  /* output file is stdout */
    params.filetype = TEXT;   /* file type is FULLTEXT */
    }

/* ===== Set parameters ===== */
void SetParams( unsigned short func, char *args )
    {
    switch( func ) {
    case FILETYPE:
	params.filetype = SetFileType( args );
	break;
    case HELP:
	params.help = MQ_YES;
	break;
    default:
	DisplayError( ETGILFUNC );
	}
    }

/* ===== Set file type ===== */
FileType SetFileType( char *filchar )
    {
    unsigned short i,       /* loop counter */
                   ftnum;   /* number of elements in ftname */
    FileType       ft;      /* obtained file type */
    MqYesNo        detect = MQ_NO;  /* detect flag */

    ftnum = sizeof( ftname ) / sizeof( ftname[0] );

    if( filchar == NULL )
	DisplayError( ETGILFILETYPE );
	
    for( i = 0; i < ftnum; i++ ) {
	if( 0 == strncmp( filchar, ftname[i].name, strlen( filchar ))) {
	    ft = ftname[i].filetype;
	    detect = MQ_YES;
	    }
	}

    if( detect == MQ_NO ) DisplayError( EMQILFILETYPE );

    return ft;
    }

/* =========================== END of the program ========================== */
