/* ----------------------- Moonquake database utilities ----------------------+
|                                                                             |
|                             "testgen.h" ver 1.00                            |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/* ============================= Include files ============================= */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <mqdb.h>      /* MQDB data access library header file */

/* ========== Macro definitions ========== */
   /* basic definitions */
#define OPTLEN     16   /* maximum length of optional switch strings */

   /* error number */
#define ETGNOERROR    0   /* program exited with no error */
#define ETGNOARGS     1   /* no arguments */
#define ETGMEMALLOC   2   /* memory allocation error */
#define ETGGENDATA    3   /* failed to generate data */
#define ETGILFUNC     4   /* illegal function number */
#define ETGILFILETYPE 5   /* illegal file type */
#define ETGERROR      6   /* error ( reason unknown ) */

    /* optional function number */
#define FILETYPE    1   /* file type */
#define HELP        2   /* display help mesasges */

    /* output data specifications */
#define DATANUM   10000
#define CHANNELS      3
#define SAMPRATE      1.0
#define YEAR       1970
#define DAY           1
#define HOUR          0
#define MINUTE        0
#define SECOND        0
#define MS            0
#define STATION      12

/* ============ Structure and Enumeration Variable Declarations  =========== */
typedef enum { TEXT, BIN, XDR } FileType;      /* output file type */
typedef enum { TG_WARNING, TG_FATAL, TG_HELP } Level;   /* error level */
typedef enum { TG_STDOUT, TG_FILE } OutFile;

typedef struct {   /* argument specifications */
    char           opt[16];   /* argument */
    unsigned short func;      /* function number */
    MqYesNo        optargs;   /* if YES, this option has parameter */
    } OPTIDX;

typedef struct {
    char           errmsg[80];  /* error message */
    unsigned short errno;       /* error number */
    Level          level;       /* level of this error */
    } ERROR;

typedef struct {
    char     name[16];   /* arguments in command line as file type */
    FileType filetype;   /* corresponding file type */
    } FTname;

/* ===================== Static Variable Declarations  ===================== */
/* program signature, necessary for MQDB */
static char progsign[] = "testgen-1.00";
                                 
static OPTIDX optidx[] =           /* argument specifications */
    { "filetype", FILETYPE, MQ_YES,
      "help",     HELP,     MQ_NO };

static ERROR  error[] =    /* error message and attributions */
   { "Program exited with no error", ETGNOERROR,    TG_WARNING,
     "No arguments",                 ETGNOARGS,     TG_HELP,
     "Memory allocation error",      ETGMEMALLOC,   TG_FATAL,
     "Failed to generate data",      ETGGENDATA,    TG_FATAL,
     "Illegal function number",      ETGILFUNC,     TG_FATAL,
     "Illegal file type",            ETGILFILETYPE, TG_FATAL,
     "Error occured",                ETGERROR,      TG_FATAL
   };

struct {
    MqYesNo   help;          /* help message output */
    OutFile   outfile;       /* output file specification */
    char      filename[255]; /* output file name */
    FileType  filetype;      /* output file type */
    }    params;

static char helpmsg[] = 
    "  This is a MQDB utility program that outputs MQDB-format-compatible\n"
    "test data ( sine wave ).\n"
    "\n"
    "Usage: testgen [ -help ] [ -filetype filetype ] [ outfile ]\n"
    "\n"
    "    -help: Display this message.\n"
    "    -filetype: set output filetype to filetype.\n"
    "               valid values are one of [ text/bin/xdr ].\n"
    "     outfile:  output file name. \n"
    "               If not specified, standard output will be chosen.\n";

static FTname ftname[] = {    /* FileType specifications */
    "text", TEXT,
    "bin",  BIN,
    "xdr",  XDR,
    "TEXT", TEXT,
    "BIN",  BIN, 
    "XDR",  XDR,
    "FULLTEXT",  TEXT,
    "COMPOSITE", BIN
    };

/* ========================= Function declarations ========================= */
void     DisplayError( unsigned short );
double * GenerateSin( void );
void     InitParams( void );
MqHeader SetHeaderData( double * );
void     SetOptions( int, char ** );
void     SetParams( unsigned short, char * );
FileType SetFileType( char * );

/* =========================== END of the program ========================== */

