/* ----------------------- Moonquake database utilities ----------------------+
|                                                                             |
|                              "wm2mq.h" ver 1.00                             |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/* ============================= Include files ============================= */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <mqdb.h>      /* MQDB data access library header file */

/* ========== Macro definitions ========== */
#define LINEMAX     1024    /* maximum characters in one line */

   /* error number */
#define EWMQNOERROR    0    /* No error */
#define EWMQNOARGS     1    /* No arguments specified */
#define EWMQMANYARGS   2    /* Too many arguments */
#define EWMQLONGFILE   3    /* Too long file name */
#define EWMQOPENWMAS   4    /* Failed to open WaveMaster data file */
#define EWMQOPENINFO   5    /* Failed to open WaveMaster info file */
#define EWMQNULLFILE   6    /* Null file name detected */ 
#define EWMQNOLINE     7    /* No return characters detected */
#define EWMQONELINE    8    /* Only one line */
#define EWMQGETLINE    9    /* Failed to read a line */
#define EWMQNOLINECONT 10   /* No line contents */
#define EWMQMEMALLOC   11   /* Failed for memory allocation */
#define EWMQNULLPTR    12   /* Null pointer detected */
#define EWMQERROR      13   /* Maximum error number */

/* ===================== Static Variable Declarations  ===================== */
static char wmasext[] = ".wmas";   /* file extension of WaveMaster data */
static char infoext[] = ".info";   /* file extension of WaveMaster info */
static char mqfext[] = ".mqf";     /* file extension of MQDB format file */
static char progsign[] = "wm2mq-1.00";
                                   /* program signature, necessary for MQDB */
static char *errormsg[] =          /* error messages */
    { "Program exited with no error.",                /* EWMQNOERROR */
      "No arguments found.",                          /* EWMQNOARGS */
      "Too many arguments",                           /* EWMQMANYARGS */
      "Too long file name",                           /* EWMQLONGFILE */
      "Failed to open WaveMaster data file",          /* EWMQOPENWMAS */
      "Failed to open WaveMaster info file",          /* EWMQOPENINFO */
      "Null file name detected",                      /* EWMQNULLFILE */
      "No return characters detected",                /* EWMQNOLINE */
      "File has only one line, maybe not a WaveMaster file",  /* EWMQONELINE */
      "Failed to read a line from WaveMaster data file",      /* EWMQGETLINE */
      "No line contents",                             /* EWMQNOLINECONT */
      "Failed for memory allocation",                 /* EWMQMEMALLOC */
      "Null pointer detected",                        /* EWMQNULLPTR */
      "Some error occured ( reason unknown )."  };    /* EWMQERROR */

/* ========================= Function declarations ========================= */
unsigned long  CountLineNumber( char * );
unsigned short CountDataSeparator( char * );
void           MakeFileNames( char *, char *, char *, char * );	
unsigned short OutputError( unsigned short );
void           OutputMqFile( char *, MqAccess *, MqHeader *, double * );
void           OutputParams( MqHeader * );
double         ReadWmasData( char *, unsigned long, double * );
MqAccess       SetMqAccess( void );
MqHeader       SetMqHeader( unsigned long, unsigned short, char *,
			                                   double, double * );

/* =========================== END of the program ========================== */

