/* ----------------------- Moonquake database utilities ----------------------+
|                                                                             |
|                        "WaveMaster (wmas)" ver. 3.00                        |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|         copyright (c) 1992-1994 Jun-ya Terazono, all rights reserved.       |
+----------------------------------------------------------------------------*/
        
/* ----------------------- history of reconstructions ----------------------
1.00  (92.06.29)  Wave enhancement is not supported.
                      Programs has the following functions.
                       *change of amplitude scale and time axis.
                       *go to next/previous wave by one mouse button click.
                       *Quit & Hardcopy by one mouse button click.
1.01  (92.07.02)  Temporary file directory changed from /tmp to ~.
1.02  (92.07.03)  Temporary file directory assignment abolished
                      ( changed into Full-path assignments ).
                      Time window length assingment by option supported.
                      ( This function couldn't be used in former versions. )
1.03  (92.07.09)  Inclusion of "math.h" abolished.
1.10  (92.08.12)  Input data file format changed (%F,%F,%F).
1.20  (92.10.15)  Information window facility supported.
                      fileread() renamed to read_data_file().
1.21  (92.10.24)  Bugs in main() was fixed by changing the definition of
                      "interval".
                      ( segmantation fault when assigned -t option ).
1.22  (92.12.14)  Initialization of dparam.timeaxisdraw was added in 
                      initdparam().

2.00  (93.11.17)  Rewritten in ANSI C style.
                  Function name style changed to GNU/X style.

3.00  (94.01.17)  Changed to MQDB ( Moonquake database ) file compatible.
---------------------------------------------------------------------------*/

/* ============================= include files ============================= */
#include <ctype.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#ifdef SYSV
#include <string.h>
#else
#include <strings.h>
#endif
#include <mqdb.h>
#include "wmas.h"            /* local variable and macro definitions */

/* ========================== program statements =========================== */

/* ===== Display usage ===== */
void DisplayUsage()
    {
    ERROUT("  usage: wmas [ -tn ] [ -v ] [ -V ] [ filename ]\n"); 
    ERROUT("  ----- meaning of options -----\n");
    ERROUT("  -t: time length to display ( in minutes ) [ default: 5 ]\n");
    ERROUT("  -v: verbose mode is VERBOSE\n" );
    ERROUT("  -V: verbose mode is VERYVERBOSE\n" );
    ERROUT("If filename is omitted, then regarded as a input from stdin.\n" );
    }

/* ===== error message output ===== */
void DisplayError( int err, int usage, char *errstr )
    {
    if( NULL == errstr )
	fprintf( stderr, "%s\n", errmsg[err] );
    else
	fprintf( stderr, "%s: %s\n", errstr, errmsg[err] );

    if( usage == TRUE )	DisplayUsage();

    exit( err );
    }
    
/* ===== main function ===== */
main( int argc, char **argv )
    {
    double     *mqdata;         /* input data */
    MqHeader    header;         /* MQDB header data */
    char        infile[FILENAME_MAX],   /* input file name */
                interval[20];   /* time interval assigned from command line */

    MqInit( progsign );

    InitOptflag();
    SeparateArgs( argv, argc, infile, interval );

    if( NULL == ( mqdata = ReadDataFile( infile, &header )))
	DisplayError( EWMMEMALLOC, FALSE, NULL );

    if( optflag.verbose >= VERBOSE ) DisplayInfo( &header, mqdata );
    
         /* initial drawings of graph and waves */
    InitDrawParam( mqdata, infile, &header );
    ControlWaveWindow( mqdata );

/*    InitDraw( dparam, indata1, indata2, indata3 );*/

    free( mqdata );
    exit( EXIT_SUCCESS );
    }

/* ===== separation of command line arguments ===== */
void SeparateArgs( char **argv, int argc, char *infile, char *interval )
/*
 *  This function separates argument strings and stores to character strings
 *  ( if valid options specified ).
 *      **argv: assigned argument values ( IN )
 *        argc: number of arguments ( IN )
 *     *infile: input data file name ( OUT )
 *   *interval: time interval assigned from the command line ( OUT )
 */

    {
    int         i;                   /* loop counter */

    for( i = 1; i < argc; i++ )
        {
        if( **(argv+i) != '-' )    {  /* data file name */
            strcpy( infile, *(argv+i) );
	    optflag.filename = TRUE;
	    }
        else
            switch(*(*(argv+i)+1))    {
	    case 't':              /* time interval */
		strcpy( interval, *(argv+i)+2 );
		optflag.timeint = TRUE;
		break;
	    case 'v':              /* verbose mode ( VERBOSE ) */
		optflag.verbose = VERBOSE;
		break;
	    case 'V':              /* verbose mode ( VERYVERBOSE ) */
		optflag.verbose = VERYVERBOSE;
		break;
		}
        }
    }

/* ===== data reading from a file ===== */
double *ReadDataFile( char *infile, MqHeader *header )
/*
 *  This function reads data from specified input file and returns values 
 *  in data file. 
 *     *infile:   input file name ( no file extension parts ) ( IN )
 *     *mqheader: header data ( OUT )
 */

    {
    MqAccess access;
    double *mqdata;

    MqSetAccessYes( &access );

    if( optflag.filename == TRUE )
	 mqdata = MqRead( infile, &access, header );
    else
	mqdata = MqRead( NULL, &access, header );

    if( mqdata != NULL ) 
	return mqdata;
    else
	return NULL;
    }
     
/* ===== initialization of drawing parameters ====== */
void InitDrawParam( double *mqdata, char *filename, MqHeader *header )

/*
 *  This function initializes drawing parameters using default values and 
 *  data from input file.
 *     *mqdata1:   input data ( IN )
 *     *header:    Header data of MQDB file ( IN )
 */   

    {
    int        i;           /* loop counter */
    
    dp.wdp.pos = 0;
    dp.wdp.data = header->NumOfData / header->Channels;
    dp.wdp.channels = header->Channels;
    dp.wdp.points = 600;

    for( i = 0; i < header->Channels; i++ )
	dp.wdp.max.data[i] = dp.wdp.norm.data[i] = header->MaxAmp.data[i];

    dp.wdp.max.channels = dp.wdp.norm.channels = header->MaxAmp.channels;
    SetWaveWindowParams( DEFAULT );

    dp.wdp.bheight = (int)( dp.win.height / dp.wdp.channels );
    strcpy( dp.wdp.file, filename );

    for( i = 0; i < header->Channels; i++ )
	dp.wdp.offset.data[i] = dp.wdp.bheight * i + dp.wdp.bheight / 2;
    }

/* ===== Display Information ===== */
void DisplayInfo( MqHeader *header, double *data )
    {
    int  format,      /* the number of data format */
         file,        /* the number of file type */
         type,        /* the number of data type */
         obsmode;     /* the number of observation mode */

    switch( header->DataFormat )   {
    case MQ_OLD:
	format = 0;	break;
    case MQ_NEW:
	format = 1;	break;
    default:
	break;
	}

    switch( header->DataType )    {
    case MQ_LP:
	type = 0;     break;
    case MQ_SP:
	type = 1;     break;
    case MQ_TIDAL:
	type = 2;     break;
    default:
	break;
	}

    switch( header->FileType )    {
    case MQ_FULLTEXT:
	file = 0;      break;
    case MQ_COMPOSITE:
	file = 1;      break;
    default:
	break;
	}

    switch( header->ObsMode )     {
    case MQ_PEAKED:
	obsmode = 0;  break;
    case MQ_FLAT:
	obsmode = 1;  break;
    default:
	break;
	}

    ERROUT( "===== Diagnostic Information of WaveMaster ===== \n" );
    fprintf( stderr, "Average amp: %f,%f,%f\n", header->AveAmp.data[0],
	    header->AveAmp.data[1], header->AveAmp.data[2] );
    fprintf( stderr, "Channels: \t%d\n", header->Channels );
    fprintf( stderr, "Data Type: %s\n", MqDataTypeName[type] );
    fprintf( stderr, "Data Format: %s\n", MqDataFormatName[format] );
    fprintf( stderr, "File Type: %s\n", MqFileTypeName[file] );
    fprintf( stderr, "Maximum amp: %f,%f,%f\n", header->MaxAmp.data[0],
	    header->MaxAmp.data[1], header->MaxAmp.data[2] );
    fprintf( stderr, "Number of data: %d\n", header->NumOfData );
    fprintf( stderr, "Observation mode: %s\n", MqObsModeName[obsmode] );
    fprintf( stderr, "Original file name: %s\n", header->OrgFile );
    fprintf( stderr, "Sampling rate: %f\n", header->SampRate.data[0] );
    fprintf( stderr, "Station: Apollo %d\n", header->Station );
    fprintf( stderr, "Start Record: %d\n", header->StartRecord );
    fprintf( stderr, "Start Time: %d %d %d %d %d %d\n",
	    header->StartTime.year, header->StartTime.day, 
	    header->StartTime.hour, header->StartTime.minute, 
	    header->StartTime.second, header->StartTime.ms );
    fprintf( stderr, "End Record:   %d\n", header->EndRecord );
    fprintf( stderr, "Tape Number:  %d\n", header->TapeNumber );
    ERROUT( "================================================ \n" );

    fprintf( stderr, "Data 0: %f\n", data[0] );
    }

/* ===== Initialize optflag ===== */
void InitOptflag( void )
    {
    optflag.timeint = FALSE;
    optflag.filename = FALSE;
    optflag.verbose = FALSE;
    }

/* ===== Set Frame value of the window ===== */
void SetWaveWindowParams( WmParType wpt )
   {
   switch( wpt )    {
   case DEFAULT:
       dp.fdp.xmin = dp.fdp.ymin = WINOFFSET;
       dp.fdp.width = FWIDTH_DEF;
       dp.fdp.height = FHEIGHT_DEF;
       dp.win.xmin = WINXMIN_DEF;
       dp.win.ymin = WINYMIN_DEF;
       dp.win.width = FWIDTH_DEF + WINOFFSET * 2 + CTRLAREAWID;
       dp.win.height = FHEIGHT_DEF + WINOFFSET * 2;
       break;
   case CUSTOM:
       break;
   default:
       return;
       }
    }

/* ========================== END of the program =========================== */
