/* ----------------------- Moonquake database utilities ----------------------+
|                                                                             |
|                              "wmas.h" ver 3.00                              |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1992-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/* ---------------------- history of reconstructions ----------------------
1.00   (92.06.29)  Definitions for wmas.c are collected.
1.01   (92.08.12)  variable "ext" changed ( .? -> .wmas )

2.00   (93.11.16)  File name changed from wmas.inc to wmas.h.

3.00   (94.02.01)  File "wmgen.h" and "wmgks.h" merged.
                   Changed to Xlib-based program.
--------------------------------------------------------------------------- */

#ifndef _WMAS_H
#define _WMAS_H

/* ============================= Include files ============================= */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <mqdb.h>       /* Moonquake database library header file */
#include "wmerror.h"

/* ========================== macro definitions =========================== */

#define DATAMAX     50000     /* maximum number of data */
#define TRUE        1
#define FALSE       0
#define ERROUT(x)   fprintf(stderr,(x))

       /* verbose mode */
#define SILENT         0
#define VERBOSE        1
#define VERYVERBOSE    2

       /* frame drawing parameters */
#define WINXMIN_DEF   100   /* default minimum x */
#define WINYMIN_DEF   100   /* default minimum y */
#define FHEIGHT_DEF   600   /* default frame height */
#define FWIDTH_DEF    600   /* default frame width */
#define CTRLAREAWID   200   /* width of control panel area */
#define WINOFFSET       3   /* gap between border of the window and frame */

       /* data format */
#define  LP      0
#define  SP      1
#define  TIDAL   2

      /* function number */
#define  FUNCNUM       8  

      /* for non-ANSI header files */
#ifndef  EXIT_SUCCESS
#define  EXIT_SUCCESS  0
#endif

#ifndef  EXIT_FAILURE
#define  EXIT_FAILURE  1
#endif

/* ======================== structure definitions ========================== */

    /* parameter choice */
typedef enum { DEFAULT, CUSTOM }   WmParType;
typedef enum { PREVIOUS, NEXT, TIMEINT, AMPLITUDE, HELP,
		   JUMP, PRINT, DISPVER, QUIT }  WmFunc;

struct {                /*  --- structure of flag parameters ---  */
    unsigned int  timeint;   /* time interval */
    unsigned int  filename;  /* file name assignment */
    unsigned int  verbose;   /* verbose mode */
    } optflag;

/*
 *  Wave drawing parameters 
 */

typedef struct { 
    unsigned short channels;   /* number of channels */
    unsigned long  pos;        /* starting position */
    unsigned long  points;     /* plotting points in one window */
    unsigned long  data;       /* number of data ( numofdata / channels ) */
    char           file[255];  /* data file name */
    MqAmp          norm;       /* normalizing amplitude */
    MqAmp          max;        /* maximum amplitude */
    MqAmp          offset;     /* Y offset */
    unsigned short bheight;    /* box height */
    }  WDP;

/*
 *  Frame drawing parameters
 */

typedef struct {
    unsigned long xmin;      /* minimum x */
    unsigned long ymin;      /* minimum y */
    unsigned long width;
    unsigned long height;
    } Rect;

struct  {
    WDP    wdp;    /* wave drawing parameters */
    Rect   win;    /* window coordinates */
    Rect   fdp;    /* frame drawing parameters */
    }  dp;

/* ===================== static variable definitions ======================= */

    /* program signature ... required for MQDB system */
const static char progsign[] = "wmas-3.00";

    /* name of the window */
const static char *wavewin = "WaveMaster";
const static char *verwin  = "Program Version";

const static char progname[]  = "WaveMaster Version 3.00";
const static char author[]    = "Programmed by Jun-ya Terazono";
const static char copyright[] =
    "Copyright (c) 1992-1994 Jun-ya Terazono, all rights reserved.";
const static char *ButtonMenu[] = {
    "Previous",         /* PREVIOUS */
    "Next",             /* NEXT */
    "Time interval",    /* TIMEINT */
    "Amplitude",        /* AMPLITUDE */
    "Jump",             /* JUMP */
    "Quit",             /* QUIT */
    "Print",            /* PRINT */
    "Show Version"      /* VERSION */
    };

/* ========================= Function declarations ========================= */
extern void   ChangeNormAmp( void );
extern void   ChangeTimeInt( void );
extern void   ControlWaveWindow( double * );
extern Window ConfigWindow( Display *, Window, Rect, const char * );
extern void   DisplayError( int, int, char * );
extern void   DisplayHelp( Display *, GC );
extern void   DisplayInfo( MqHeader *, double * );
extern void   DisplayUsage( void );
extern void   DisplayVersion( Display *, GC );
extern void   DrawAmpScale( void );
extern void   DrawButton( Display *, Window, GC );
extern void   DrawFrame( Display *, Window, GC );
extern void   DrawTimeAxis( void );
extern void   DrawTimeDisp( void );
extern void   DrawTitle( Display *, Window, GC, char * );
extern void   DrawWave( Display *, Window, GC, double * );

/* extern void   InitDraw( DRAW_PARAM *, double *, double *, double * );*/

extern void   InitDrawParam( double *, char *, MqHeader * );
extern void   InitOptflag( void );
extern void   InputString( void );
extern WmFunc InputWait( Display *, Window, GC );
extern void   JumpTime( void );
extern double *ReadDataFile( char *, MqHeader * );
extern void   SeparateArgs( char **, int, char *, char * );
extern void   SetWaveWindowParams( WmParType );

#endif    /* _WMAS_H */

/* =========================== END of the program ========================== */
