/* ----------------------- Moonquake database utilities ----------------------+
|                                                                             |
|                             "wmdraw.c" ver 3.00                             |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1992-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/* ----------------------- history of reconstructions ----------------------
1.00  (92.06.29)  This program draws button, frame, wave, time axis 
                    and amplitude scales.
1.10  (92.07.28)  Time axis drawings changed.
1.20  (92.10.15)  Information window facilities were added.
                      setgks() renamed to config_wave_window().
                      New function: config_info_window(),
                                   redraw_info_window(),
                                   display_info().
1.30  (92.12.22)  New function: draw_time_disp().
1.40  (93.02.17)  initdraw() was separated into wmginit.c.

2.00  (93.11.17)  Rewritten in ANSI C style.
                  Function names changed to GNU/X style.

3.00  (94.02.15)  File name changed from wmgdraw.c to wmdraw.c.
                  Changed to Xlib-based.
----------------------------------------------------------------------------*/

/* ============================= Include files ============================= */
#include <stdio.h>
#include <stdlib.h>
#ifdef SYSV
#include <string.h>
#else
#include <strings.h>
#endif
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include "wmas.h"

/* ========================== Program statements  ========================== */

/* ===== initialize graph ====== */
void ControlWaveWindow( double *mqdata )
    {
    WmFunc      res;
    Display   *display;
    Window     root, wave;
    XEvent     ev;
    GC gc;

    if( NULL == ( display = XOpenDisplay( NULL )))
       DisplayError( EWMDISPOPEN, FALSE, NULL );

    root = XDefaultRootWindow( display );
    gc = XCreateGC( display, root, 0, 0 );

    XSetFont( display, gc, XLoadFont( display, "variable" ));
    XSetForeground( display, gc, BlackPixel( display, 0 ));
    XSetLineAttributes( display, gc, 1, LineSolid, CapButt, JoinMiter );    

    wave = ConfigWindow( display, root, dp.win, wavewin );

    XSelectInput( display, wave,
		 ExposureMask | ButtonPressMask | KeyPressMask );

    while( 1 ) {
	XNextEvent( display, &ev );
	if( ev.type == Expose ) break;
	}

    DrawFrame( display, wave, gc );      /* window frame drawings */
    DrawWave( display, wave, gc, mqdata );       /* wave drawings */
    DrawTitle( display, wave, gc, dp.wdp.file );
    DrawButton( display, wave, gc );

    XFlush( display );
    while( 1 ) {          /* waiting for events ( keypress, mouse, ... ) */
	res = InputWait( display, wave, gc );
	switch( res )    {
	case NEXT:
	case PREVIOUS:
	    XSetForeground( display, gc, BlackPixel( display, 0 ));
	    XSetLineAttributes( display, gc, 1,
			       LineSolid, CapButt, JoinMiter );    
	    DrawFrame( display, wave, gc );
	    DrawWave( display, wave, gc, mqdata );
	    break;
	case QUIT:
	    break;
	case DISPVER:
	    DisplayVersion( display, gc );
	    break;
	case HELP:
	    DisplayHelp( display, gc );
	    break;
	    }
	if( res == QUIT ) break;
	XFlush( display );
	}

    XFreeGC( display, gc );
    XDestroyWindow( display, wave );
    XCloseDisplay( display );
    }

/* ===== configuring wave plotting window ===== */
Window ConfigWindow( Display *display, Window root,
		    Rect win, const char *winname )
    {
    Window wave;     /* wave window identifier */
    XSetWindowAttributes attr;

    wave = XCreateSimpleWindow( display, root, win.xmin, win.ymin,
			       win.width, win.height, 1,
    BlackPixel( display, 0 ),
			WhitePixel( display, 0 ));

    attr.backing_store = WhenMapped;
    XChangeWindowAttributes( display, wave, CWBackingStore, &attr );

    XStoreName( display, wave, winname );
    XMapWindow( display, wave );

    return wave;
    }

/* ===== setting frames and window borders ===== */
void DrawFrame( Display *display, Window wave, GC gc )
    {
    XDrawRectangle( display, wave, gc,
		   dp.fdp.xmin, dp.fdp.ymin, dp.fdp.width, dp.fdp.height );

    XFlush( display );
    }

/* ===== drawing waves ===== */
void DrawWave( Display *display, Window wave, GC gc, double *mqdata )
    {
    XPoint  dpoint[600];
    int   i, j, k, start;
    double tmpoffset[3] = { 510.0, 510.0, 500.0 };

    for( i = 0; i < dp.wdp.channels; i++ )    {
	start = i * dp.wdp.data + dp.wdp.pos;
	for( j = 0; j < 600; j++ )   {
	    k = i * dp.wdp.data + dp.wdp.pos + j;
	    dpoint[j].x = j + WINOFFSET;
/*	    dpoint[j].y = (int)dp.wdp.offset.data[i] -
		(int)(( mqdata[k] - mqdata[start]) / dp.wdp.norm.data[i] ) * 100 + WINOFFSET;*/
	    dpoint[j].y = (int)dp.wdp.offset.data[i] -
		(int)( mqdata[k] - tmpoffset[i] ) * 10 + WINOFFSET;
	    if( k == 0 ) fprintf( stderr, "mqdata[k]: %f\n", mqdata[0] );

	    }
	XDrawLines( display, wave, gc, dpoint, 600, CoordModeOrigin );
	}

    }

/* ===== drawing time axis ===== */
void DrawTimeAxis( void )
    {
    }

/* ===== drawing time display ===== */
void DrawTimeDisp( void )
    {
    }

/* ===== drawing amplitude scale ===== */

void DrawAmpScale( void )
    {
    }

/* ===== drawing display title ===== */
void DrawTitle( Display *display, Window wave, GC gc, char *text )
    {
    XDrawString( display, wave, gc,
		650, 100, text, 100 );

    }

/* ===== Draw Button ===== */
void DrawButton( Display *display, Window wave, GC gc )
    {
    int   i;

    for( i = 0; i < FUNCNUM; i++ )    {
	XDrawString( display, wave, gc,
	    650, 200 + i * 20, ButtonMenu[i] , strlen( ButtonMenu[i] ));
	}
	

    }

/* ===== Display version of this program ===== */
void DisplayVersion( Display *display, GC gc )
    {
    Window dispver, root;
    Rect   versize;
    XEvent ev;

    versize.xmin = versize.ymin = 500;
    versize.height = versize.width = 300;

    root = XDefaultRootWindow( display );
    dispver = ConfigWindow( display, root, versize, verwin ); 

    XSelectInput( display, dispver, ExposureMask | ButtonPressMask );

    while( 1 ) {
	XNextEvent( display, &ev );
	if( ev.type == Expose ) break;
	}

    XDrawString( display, dispver, gc, 0, 30, progname, strlen( progname ));
    XDrawString( display, dispver, gc, 0, 50, author, strlen( author ));
    XDrawString( display, dispver, gc, 0, 70, copyright, strlen( copyright ));

    while( 1 ) {
	XNextEvent( display, &ev );
	if( ev.type == ButtonPress ) break;
	}
		
    XDestroyWindow( display, dispver );
    }

/* ===== Display version of this program ===== */
void DisplayHelp( Display *display, GC gc )
    {
    Window dispver, root;
    Rect   versize;
    XEvent ev;

    versize.xmin = versize.ymin = 500;
    versize.height = versize.width = 300;

    root = XDefaultRootWindow( display );
    dispver = ConfigWindow( display, root, versize, verwin ); 

    XSelectInput( display, dispver, ExposureMask | ButtonPressMask );

    while( 1 ) {
	XNextEvent( display, &ev );
	if( ev.type == Expose ) break;
	}

    XDrawString( display, dispver, gc, 0, 30, progname, strlen( progname ));
    XDrawString( display, dispver, gc, 0, 50, author, strlen( author ));
    XDrawString( display, dispver, gc, 0, 70, copyright, strlen( copyright ));

    while( 1 ) {
	XNextEvent( display, &ev );
	if( ev.type == ButtonPress ) break;
	}
		
    XDestroyWindow( display, dispver );
    }

/* =========================== END of the program ========================== */
