/* ----------------------- Moonquake database utilities ----------------------+
|                                                                             |
|                              "wprld.c" ver 3.20                             |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|      Copyright (c) 1993-1994 by Jun-ya Terazono, all rights reserved.       |
+----------------------------------------------------------------------------*/

/*
 *  This is one of the component of "wpr", wave output utility. This file 
 *  contains main part of the drawing for the LIPS II+ output.
 */

/* ========================= History of modifications =========================
1.00  (93.05.18)  MS-DOS version written by Hitoshi Mizutani.
1.00  (93.06.23)  UNIX implementation by J.T.

2.00  (93.05.18)  Became MQDB-format compatible.

3.00  (93.08.22)  File name changed from wprlips2.c to wprld.c.
                  Some fundamental functions are moved to wprll.c.
3.10  (93.08.24)  Free amplitude/time scales are supported.
                  L2DrawWave() revised.
============================================================================ */

/* ============================= Include files ============================= */
#include <stdio.h>
#include <stdlib.h>
#include <mqdb.h>       /* moonquake database system include file */
#include "wpr.h"
#include "wprerror.h"

/* ========================== Program statements  ========================== */

/* ===== printer handlings ( currently LIPS II+ only ) ===== */
void printout( DRAW *draw, PARAMS *params, MqHeader *header, double *data )

/*
 *  This is a control function of printing by LIPS II+.
 *    draw:  drawing parameters.
 *    params: general setting parameters.
 *    header: header data obtained from MQDB format file.
 *    data:   rearranged data
 */

    {
    unsigned long    i, j;     /* loop counter */
    unsigned long    point;

    if( params->opts.verbose >= VERBOSE )
	fprintf( stderr, "\n----- Printing page %ld ----- \n",
		draw->currpage );

    SoftReset();
    FullPaintMode();
    ShiftToVectorMode();

    L2Init( params );
    L2DrawFrame( params );
    L2DrawTimeBar( draw );
    L2DrawTimeScale( draw );

    for( i = 0; i < draw->channels; i++ ) {
	draw->currch = i;
	L2DrawWaves( draw, params, data );
	}

    printf( "%%%c", IS2 );    /* End Vector mode */
    printf( "}p%c", IS2 );    /* back to text mode */

    L2DrawStrings( draw, params, header );

    if( params->opts.verbose >= VERBOSE )
	fprintf( stderr, "----- Done -----\n" );

    printf( "%c", FF );
    SoftReset();
    }

/* ===== initializations of printer ( LIPS II+ ) ===== */
void L2Init( PARAMS *params )
    {   
    if( params->opts.verbose >= VERBOSE )
	fprintf( stderr, "Initializing printer ... ");

    /* Vector mode setups. Other instructions cannot be inserted between
       These three instructions */
    printf( "#%c", IS2 );             /* Begin picture */
    printf( "!0%c%c", 0x23, IS2 );    /* Scaling Mode ( set as 1dot ) */
    printf( "$%c", IS2 );             /* Begin Picture Body */

    printf( "(" );              /* set VDC extent */
    cal_out( XMIN - CLIPRANGE );
    cal_out( YMAX - CLIPRANGE );
    cal_out( XMAX + CLIPRANGE );
    cal_out( YMIN + CLIPRANGE );
    printf( "%c", IS2 );
   
   
    /*  Select line type & pattern */

    printf( "E10%c", IS2 );      /* line type is jissen line */
    printf( "F1#%c", IS2 );      /* line width is 5dots  */
    printf( "}G10%c",  IS2 );

    /* Clip rectangle */
    printf( ")" );
    cal_out( XMIN - CLIPRANGE );
    cal_out( XMAX + CLIPRANGE );
    cal_out( YMIN - CLIPRANGE );
    cal_out( YMAX + CLIPRANGE );
    printf( "%c", IS2 );

    /* Set clipping ON */
    printf( "*" );
    cal_out( 0x30 );
    printf( "%c", IS2 );

    if( params->opts.verbose >= VERBOSE )
	fprintf( stderr, "done.\n" );
    }

/* ===== drawing frames ( LIPS II+ ) ===== */
void L2DrawFrame( PARAMS *params )
    {
    long xx, xdv, ydv, xdm,x12,x23,xm1,xm2,xm3;

    if( params->opts.verbose >= VERBOSE )
	fprintf( stderr, "printing frames ... ");

    xx=0; 
    xdv = ( XMAX - XMIN ) /3;
    ydv = YMAX - YMIN; 
    x12 = xdv + XMIN;
    x23 = xdv + x12;
    xdm = ( x12 - XMIN ) / 2;
    xm1 = xdm + XMIN;
    xm2 = xdm + x12;
    xm3 = xdm + x23;
   
    printf("}:");
   
    cal_out( XMIN );
    cal_out( XMAX );
    cal_out( YMIN );    
    cal_out( YMAX );

    printf( "%c", IS2 ); 
 
/* ++++++ Start of draw line of 3 blocks ++++++++ */    
    printf( "1" );
    cal_out( x12 );    cal_out( YMIN );    cal_out( xx );    cal_out( ydv );
    printf( "%c", IS1 );  /*  Data separation  */
    
    cal_out( x23 );    cal_out( YMIN );    cal_out( xx );    cal_out( ydv );
    printf( "%c", IS2 );  /*  end of rectangle  */
 
/* ++++++ End of Draw line of 3 blocks ++++++++++ */   

    printf( "F1!%c", IS2 );   /* line width   1dots  */
    printf( "}G10%c1", IS2 );
      
    cal_out( xm1 );    cal_out( YMIN );    cal_out( xx );    cal_out( ydv );
    printf( "%c", IS1 );  /*  Data separation  */
  
    cal_out( xm2 );    cal_out( YMIN );    cal_out( xx );    cal_out( ydv );
    printf( "%c", IS1 );  /*  Data separation  */
  
    cal_out( xm3 );    cal_out( YMIN );    cal_out( xx );    cal_out( ydv );
    printf( "%c", IS2 );  /*  Data separation  */

    if( params->opts.verbose >= VERBOSE )
	fprintf( stderr, "done.\n" );
    }

/* ===== drawing waves ( LIPS II+ ) ===== */
void L2DrawWaves( DRAW *draw, PARAMS *params, double *data )

/*
 *    draw:   drawing parameters.
 *    params: general parameters.
 *    data:   raw data read from MQDB format data file.
 */

    {
    long  x;           /* step about x component */
    long    *plot;       /* data to be plotted */
    long     clipmin, clipmax;
    FPARAM   fparam;     /* frame drawing parameters */
    long    i, counter, ypos,      /* both loop counter */
            frame, FramePoints, 
            y, j, point;

/*
 *  Settings of general parameters.
 */

    printf( "F1\"%c", IS2 );    /* line width is 3dots  */
    printf( "}G10%c", IS2 );

    fparam = SetFrameParams( params, draw );

    clipmax = XMAX + CLIPRANGE - XMIN - (long)draw->offset[draw->currch];
    clipmin = - CLIPRANGE - (long)draw->offset[draw->currch];

    for( frame = 1; frame <= fparam.DrawFrames; frame++ )    {
	printf( "1" );
	FramePoints = SetFramePoints( draw, fparam, frame );

     /*** calculations of start/end points of data reading ***/

	fparam.DataStart = fparam.PageStart +
	    FRAME_LENGTH * ( frame -1 ) - ( frame - 1 );
	fparam.DataEnd   = fparam.DataStart + FRAME_LENGTH - 1;
	if( fparam.DataEnd > fparam.PageEnd )
	    fparam.DataEnd = fparam.PageEnd;

	if( params->opts.verbose == VERYVERBOSE )
	    fprintf( stderr,
	        " frame: %ld, page start: %d, data start: %ld,"
                " data end: %ld, points: %ld\n",
		    frame, fparam.PageStart, fparam.DataStart, fparam.DataEnd,
		    FramePoints );
/*
 *  Currently, this function is sealed for awaiting future use. 
 */
/*	origin = SetWaveOrigin( frame, currch );*/

	if( NULL == ( plot = calloc( FramePoints, sizeof( long ))))
	    wprerror( EWPRNOMEMORY );

	for( j = 0; j < FramePoints; j++ ) {
	    point = fparam.DataStart + j + draw->currch * draw->datanum;
	    plot[j] = (long)data[point];
	    }

	for( j = 0; j < FramePoints; j++ ) {
	    if( plot[j] > clipmax ) plot[j] = clipmax;
	    if( plot[j] < clipmin ) plot[j] = clipmin;
	    }

	for( j = 0; j < FramePoints; j++ ) {
	    if( j == 0 ) {
		x = plot[j] + XMIN + (long)draw->offset[draw->currch];
		y = fparam.DataStart - fparam.PageStart + YMIN;
		}
	    else    {
		x = plot[j] - plot[j-1];
		y = 1;
		}
	    cal_out( x ) ;
	    cal_out( y );
	    }

	free( plot );
	printf( "%c", IS2 );
	}
    }

/* ===== drawing strings ( LIPS II+ ) ===== */
void L2DrawStrings( DRAW *draw, PARAMS *params, MqHeader *header )

/*
 *  This function draws strings about plotting informations such as current
 *  time and input file name.
 */
    {
    char   ampscale[10];

        /* string handlings to delete \n in the last of *apollo */

    if( params->opts.verbose >= VERBOSE )
	fprintf( stderr, "printing texts ... " );

       /* move cursor to absolute position */
    printf( "%c[", ESC );       /* CSI */
    printf( "1;1f" );           /* Home position */

/*
 *  Actual string output part.  
 *  please be careful that you must not exceed 80 characters totally in one
 *  output line when you add to some output.
 */

    printf( "  %s  Apollo %d %s", draw->outtime, header->Station,
	   MqDataTypeName[header->DataType] );

    L2DrawTime( header->StartTime, draw );

         /* current page and total pages */
    printf( "  Page %2ld / %2ld", draw->currpage, draw->pages );
         /* current page and total pages */

    printf( "%c[3%%r", ESC );    /* 270 degree rotation */
    printf( "%c[11l", ESC );     /* set location mode to character units */

/*
 *  File name output. If filename is NULL, "standard input" will be printed.
 */

    printf( "%c[1;6f", ESC );
    if( params->opts.infile == INPUT_STDIN )
	printf( "< standard input >" );
    else
	printf( "< %s >", params->file );

/*
 *  Prints wave channel names. Actually, this should be different between
 *  LP and SP.  Current implementation is limited to LP.
 */

    printf( "%c[4;8f", ESC );
    printf( "X" );

    printf( "%c[18;8f", ESC );
    printf( "Y" );

    printf( "%c[32;8f", ESC );
    printf( "Z" );

/*
 *  Draw amplitude scale 
 */
    sprintf( ampscale, "%5.1lf", draw->ampscale );

    printf( "%c[15;115f", ESC );
    printf( "%s", ampscale );

    printf( "%c[29;115f", ESC );
    printf( "%s", ampscale );

    printf( "%c[43;115f", ESC );
    printf( "%s", ampscale );

/*
 *  Output time width.
 */

/*    printf( "%c[45;8f", ESC );*/
/*    printf( "<-- %7.1lf sec -->", draw->timescale );*/

    printf( "%c[0%%r", ESC ); 

    if( params->opts.verbose >= VERBOSE )
	fprintf( stderr, "done.\n" );
    }

/* ===== Draw top time ===== */
void L2DrawTime( MqTime mqtime, DRAW *draw )
    {
    double width, ms2;
    MqTime  newtime;
    unsigned long ms, sec, min, hour, day, year, rem;

    ms2 = 0.0;
    year = mqtime.year;

    newtime.year = mqtime.year;
    newtime.day  = mqtime.day;
    newtime.hour = mqtime.hour;
    newtime.ms = mqtime.ms;

    width = draw->timeint *
	(double)( draw->points.ord * ( draw->currpage - 1 ));

    ms  = (unsigned long)(( width - floor( width )) * 1000 );
    sec = (unsigned long)floor( width );

    rem = sec % 60;
    min = ( sec - rem ) / 60;
    sec = rem;

    rem = min % 60;
    hour = ( min - rem ) / 60;
    min = rem;

    rem = hour % 24;
    day = ( hour - rem ) / 24;
    hour = rem;

    rem = day / 365;
    year = ( day - rem ) / 365;
    day = rem;

    newtime.ms = mqtime.ms + (unsigned short)ms;
    if( newtime.ms > 1000 ) {
	newtime.ms %= 1000;
	sec++;
	}

    newtime.second = mqtime.second + (unsigned short)sec;
    if( newtime.second > 60 ) {
	newtime.second %= 60;
	min++;
	}

    newtime.minute = mqtime.minute + (unsigned short)min;
    if( newtime.minute > 60 ) {
	newtime.minute %= 60;
	hour++;
	}

    newtime.hour = mqtime.hour + (unsigned short)hour;
    if( newtime.hour > 24 ) {
	newtime.hour %= 24;
	day++;
	}

    newtime.day = mqtime.day + (unsigned short)day;
    if( newtime.day > 365 && ( mqtime.year % 4 != 0 )) {
	newtime.day %= 365;
	year++;
	}
    if( newtime.day > 366 && ( mqtime.year % 4 == 0 )) {
	newtime.day %= 366;
	year++;
	}

    printf(  "  %4d/%03d/%02d:%02d:%02d.%03d",      
	   newtime.year, newtime.day, newtime.hour, newtime.minute,
	   newtime.second, newtime.ms );
    }

/* ===== Draw Time Reference Bar ===== */
void L2DrawTimeBar( DRAW *draw )

/*
 *   This function draws time bar representing the scale of one minute 
 *   printed at the top of the paper.
 */

    {
    long    barlength;   /* length of the time bar */

    barlength = (long)( 60. / draw->timeint );

    printf( "}G0%c", IS2 );   /* set line attributes */

    printf( "I!1%c", IS2 );   /* paint with faint half tone pattern */

    printf( "%c%c", '}', ':' );    /* rectangle drawing start */
    cal_out( XMAX );                  /* Xmin */
    cal_out( XMAX + TIMEBARWIDTH );   /* Xmax */
    cal_out( YMIN );                  /* Ymin */
    cal_out( YMIN + barlength );      /* Ymax */
    printf( "%c", IS2 );              /* end */

    printf( "I&1%c", IS2 );   /* paint with faint half tone pattern */
    }

/* ===== Draw Time Scale ===== */
void L2DrawTimeScale( DRAW  *draw )
    {
    double width;
    unsigned long  majint, minint, scale, i;

    width = draw->timeint * draw->points.ord ;
   
    majint = (unsigned long)((double)( YMAX - YMIN ) / width * 600 );
    minint = (unsigned long)((double)( YMAX - YMIN ) / width * 60 );

    i = 1;   scale = YMIN;
    while( scale < 2520 ) {
	scale = majint * i;
	
	printf( "F1#%c", IS2 );      /* line width is 5dots  */
	printf( "1" );
	cal_out( 600 );    cal_out( scale + YMIN );
	cal_out( 100 );    cal_out( 0 );    /* relative coordinate !! */
	printf( "%c", IS2 );  /*  Data separation  */

	printf( "1" );
	cal_out( 1150 );   cal_out( scale + YMIN );
	cal_out( 100 );    cal_out( 0 );
	printf( "%c", IS2 );  /*  Data separation  */

	i++;
	}

    scale = YMIN;
    i = 1;
    while( scale < 2520 ) {

	scale = minint * i;

	if( i % 10 != 0 ) {

	    printf( "F1!%c", IS2 );      /* line width is 1 dot */
	    printf( "1" );
	    cal_out( 625 );    cal_out( scale + YMIN );
	    cal_out( 50 );     cal_out( 0 );    /* relative coordinate !! */
	    printf( "%c", IS2 );

	    printf( "1" );
	    cal_out( 1175 );   cal_out( scale + YMIN );
	    cal_out( 50 );      cal_out( 0 );
	    printf( "%c", IS2 );
	    }
	i++;
	}

/* ׻timescaleϥޥ */

    for( i = 0; i < 12; i++ ) {
	printf( "F1!%c", IS2 );      /* line width is 1 dot */
	printf( "1" );

	cal_out( XMIN - 50 );    cal_out( YMIN + 210 * i );
	cal_out( 50 );    cal_out( 0 );
	printf( "%c", IS2 );
	}
    }

/* ===== Draw end procedures ( LIPS II+ ) ===== */
void L2DrawEnd( PARAMS *params )
    {
    printf( "%c", FF );    /* eject paper and ends printing */

    if( params->opts.verbose >= VERBOSE )
       fprintf( stderr, "Printout completed.\n");
    }

/* =========================== END of the program ========================== */
