/* ----------------------- Moonquake database utilities ----------------------+
|                                                                             |
|                              "wprll.c" ver 1.00                             |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|      Copyright (c) 1993-1994 by Jun-ya Terazono, all rights reserved.       |
+----------------------------------------------------------------------------*/

/*
 *  This is one of the component of "wpr", wave output utility. This file is 
 *  a collection of the function for fundamental role of LIPS II+ outputs.
 */

/* ========================= History of modifications =========================
1.00  (93.08.22)  Separated from wprlips2.c ( now wprld.c ).
============================================================================ */

/* ============================= Include files ============================= */
#include <stdio.h>
#include <stdlib.h>
#include <mqdb.h>       /* moonquake database system include file */
#include "wpr.h"
#include "wprerror.h"

/* ========================== Program statements  ========================== */

/* ===== Convert long integers for LIPS II+ output ====== */
void cal_out( long x )

/*
 *  This function converts long integer x to a LIPS II output style. For 
 *  detailed format of output, see explanation in wpr.h.
 */

    {
    LIPSLONG    xout;
    LIPSOUT     lout;
    int         sgn;

    ( x < 0 ) ? ( sgn = FALSE ) : ( sgn = TRUE );

    xout.x = abs( x );
    if( xout.x >= 65536 )    {
	lout.ordbyte.id = 1;
	lout.ordbyte.byte = xout.bit.byte4;
	printf( "%c", lout.lipsout );
	}
    if( xout.x >= 1024 )    {
	lout.ordbyte.id = 1;
	lout.ordbyte.byte = xout.bit.byte3;
	printf( "%c", lout.lipsout );
	}
    if( xout.x >= 16 )    {
	lout.ordbyte.id = 1;
	lout.ordbyte.byte = xout.bit.byte2;
	printf( "%c", lout.lipsout );
	}

    lout.lowestbyte.id = 1;
    lout.lowestbyte.sgn = sgn;
    lout.lowestbyte.byte1 = xout.bit.byte1;
    printf( "%c", lout.lipsout );
    }

/* ===== Set number of frames ===== */
FPARAM SetFrameParams( PARAMS *params, DRAW *draw )

/*
 *  This function sets frame drawing parameters.
 */

    {
    FPARAM   fparam;

/*
 *  If current page is a last, then last page flag are set and appropriate
 *  number is set for frames.
 */

    if( draw->currpage == draw->pages )   {
	fparam.LastPageFlag = TRUE;
	fparam.DrawFrames   = draw->frames.last;
	}
    else    {
	fparam.LastPageFlag = FALSE;
	fparam.DrawFrames   = draw->frames.ord;
	}

    if( params->opts.verbose >= VERBOSE )
	fprintf( stderr, "Drawing Frames are %ld ... \n",
		fparam.DrawFrames );

    fparam.PageStart = ( draw->currpage - 1 ) * draw->points.ord + 1;
    fparam.PageEnd   = fparam.PageStart + draw->points.ord - 1;
    if( fparam.PageEnd > draw->datanum ) fparam.PageEnd = draw->datanum;

    fparam.DataStart = fparam.DataEnd = 0;

    return fparam;
    }

/* ===== Set frame points ===== */
long SetFramePoints( DRAW *draw, FPARAM fparam, long frame )

/*
 *  This function returns number of points in one frame ( FramePoints ).
 *     draw:   drawing parameters ( IN )
 *     fparam: frame drawing parameters ( IN )
 *     frame:  current frame number ( IN )
 */

    {
    long  FramePoints;

    if( frame == fparam.DrawFrames )
	if( fparam.LastPageFlag == TRUE )
	    FramePoints = draw->lastpts.last;
	else
	    FramePoints = draw->lastpts.ord;
    else
	FramePoints = FRAME_LENGTH;

    return FramePoints;
    }

/* ===== Set wave origin ===== */
/*double SetWaveOrigin( long frame, long currch, FPARAM fparam, DRAW draw )*/
/*
 *  Currently, this function is sealed for awaiting future use.
 */
/*    {
    if( frame == 1 ) {
	offset[j] = data[draw->datanum*j+fparam.PageStart-1];
	fprintf( stderr,
		"data number is %d\n", draw->datanum*j+fparam.PageStart-1 );
	   }
    return origin;
    }*/

/* =========================== END of the program ========================== */
